package org.jeecg.modules.ads.controller;

import cn.com.ctop.common.module.annotation.AutoLog;
import cn.com.ctop.common.module.utils.Check;
import cn.com.ctop.common.module.utils.QueryGenerator;
import cn.com.ctop.kuaishou.modules.ai.service.IAiStrategyService;
import cn.com.ctop.kuaishou.modules.batch.entity.KuaishouStrategy;
import cn.com.ctop.kuaishou.modules.batch.service.IKuaishouStrategyService;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.shiro.SecurityUtils;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.system.vo.LoginUser;
import org.jeecg.common.util.oConvertUtils;
import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

/**
 * kuaishou.modules.batch-优化评估策略表
 *
 * @author jeecg-boot
 * @version V1.0
 * @date 2020-12-10
 */
@Slf4j
@Api(tags = "kuaishou.modules.batch-优化评估策略表")
@RestController
@RequestMapping("/kuaishouStrategy")
public class KuaishouStrategyController {
    @Autowired
    private IKuaishouStrategyService kuaishouStrategyService;
    @Autowired
    private IAiStrategyService strategyService;

    /**
     * 分页列表查询
     *
     * @param kuaishouStrategy
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @AutoLog(value = "kuaishou.modules.batch-优化评估策略表-分页列表查询")
    @ApiOperation(value = "kuaishou.modules.batch-优化评估策略表-分页列表查询", notes = "kuaishou.modules.batch-优化评估策略表-分页列表查询")
    @GetMapping(value = "/list")
    public Result<IPage<KuaishouStrategy>> queryPageList(KuaishouStrategy kuaishouStrategy,
                                                         @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                         @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                         HttpServletRequest req) {
        Result<IPage<KuaishouStrategy>> result = new Result<>();
        QueryWrapper<KuaishouStrategy> queryWrapper = QueryGenerator.initQueryWrapper(kuaishouStrategy, req.getParameterMap());
        Page<KuaishouStrategy> page = new Page<KuaishouStrategy>(pageNo, pageSize);
        IPage<KuaishouStrategy> pageList = kuaishouStrategyService.page(page, queryWrapper);
        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }
    /**
     * 添加
     *
     * @param data
     * @return
     */
    @PostMapping(value = "/add")
    public Result<Object> add(@RequestBody JSONObject data) {
        Result<Object> result = new Result<>();
        try {
            if (Check.isNull(data.getLong("accountId"))) {
                return Result.error(-1, "请选择账户");
            }
            LoginUser user = (LoginUser) SecurityUtils.getSubject().getPrincipal();
            KuaishouStrategy strategy = JSONObject.toJavaObject(data, KuaishouStrategy.class);
            JSONArray appStore = data.getJSONArray("appStore");
            if(null!=appStore&&!appStore.isEmpty()){
                strategy.setAppStore(appStore.toJSONString());
            }
            JSONArray siteId = data.getJSONArray("siteId");
            if(null!=siteId&&!siteId.isEmpty()){
                strategy.setSiteId(siteId.toJSONString());
            }
            JSONArray creativeTag = data.getJSONArray("creativeTag");
            if(null!=creativeTag&&!creativeTag.isEmpty()){
                strategy.setCreativeTag(creativeTag.toJSONString());
            }
            strategy.setCreateBy(user.getId());
            kuaishouStrategyService.save(strategy);
            strategyService.createCreativeByStrategy(strategy);
            result.success("添加成功！");
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            result.error500("操作失败");
        }
        return result;
    }

    /**
     * 编辑
     *
     * @param
     * @return
     */
    @AutoLog(value = "kuaishou.modules.batch-优化评估策略表-编辑")
    @ApiOperation(value = "kuaishou.modules.batch-优化评估策略表-编辑", notes = "kuaishou.modules.batch-优化评估策略表-编辑")
    @PostMapping(value = "/edit")
    public Result<?> edit(@RequestBody JSONObject data) {
        try {
            return kuaishouStrategyService.shutDownPlan(data);
        } catch (Exception e) {
            log.error("关停策略下的计划异常", e);
        }
        return Result.error("关停失败");
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @AutoLog(value = "kuaishou.modules.batch-优化评估策略表-通过id删除")
    @ApiOperation(value = "kuaishou.modules.batch-优化评估策略表-通过id删除", notes = "kuaishou.modules.batch-优化评估策略表-通过id删除")
    @DeleteMapping(value = "/delete")
    public Result<?> delete(@RequestParam(name = "id", required = true) String id) {
        try {
            kuaishouStrategyService.removeById(id);
        } catch (Exception e) {
            log.error("删除失败", e.getMessage());
            return Result.error("删除失败!");
        }
        return Result.ok("删除成功!");
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @AutoLog(value = "kuaishou.modules.batch-优化评估策略表-批量删除")
    @ApiOperation(value = "kuaishou.modules.batch-优化评估策略表-批量删除", notes = "kuaishou.modules.batch-优化评估策略表-批量删除")
    @DeleteMapping(value = "/deleteBatch")
    public Result<KuaishouStrategy> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        Result<KuaishouStrategy> result = new Result<>();
        if (ids == null || "".equals(ids.trim())) {
            result.error500("参数不识别！");
        } else {
            this.kuaishouStrategyService.removeByIds(Arrays.asList(ids.split(",")));
            result.success("删除成功!");
        }
        return result;
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    @AutoLog(value = "kuaishou.modules.batch-优化评估策略表-通过id查询")
    @ApiOperation(value = "kuaishou.modules.batch-优化评估策略表-通过id查询", notes = "kuaishou.modules.batch-优化评估策略表-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<Object> queryById(@RequestParam(name = "id", required = true) String id) {
        if (Check.isNull(id)) {
            return Result.error(-1, "请选择查看的策略");
        }
        KuaishouStrategy strategy = kuaishouStrategyService.getById(id);
        if (Check.isNull(strategy)) {
            return Result.error(-1, "策略不存在");
        }
        return Result.ok(strategy);
    }

    /**
     * 导出excel
     *
     * @param request
     * @param response
     */
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(HttpServletRequest request, HttpServletResponse response) {
        // Step.1 组装查询条件
        QueryWrapper<KuaishouStrategy> queryWrapper = null;
        try {
            String paramsStr = request.getParameter("paramsStr");
            if (oConvertUtils.isNotEmpty(paramsStr)) {
                String deString = URLDecoder.decode(paramsStr, "UTF-8");
                KuaishouStrategy kuaishouStrategy = JSON.parseObject(deString, KuaishouStrategy.class);
                queryWrapper = QueryGenerator.initQueryWrapper(kuaishouStrategy, request.getParameterMap());
            }
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }

        //Step.2 AutoPoi 导出Excel
        ModelAndView mv = new ModelAndView(new JeecgEntityExcelView());
        List<KuaishouStrategy> pageList = kuaishouStrategyService.list(queryWrapper);
        //导出文件名称
        mv.addObject(NormalExcelConstants.FILE_NAME, "kuaishou.modules.batch-优化评估策略表列表");
        mv.addObject(NormalExcelConstants.CLASS, KuaishouStrategy.class);
        mv.addObject(NormalExcelConstants.PARAMS, new ExportParams("kuaishou.modules.batch-优化评估策略表列表数据", "导出人:Jeecg", "导出信息"));
        mv.addObject(NormalExcelConstants.DATA_LIST, pageList);
        return mv;
    }

    /**
     * 通过excel导入数据
     *
     * @param request
     * @param response
     * @return
     */
    @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
        MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
        Map<String, MultipartFile> fileMap = multipartRequest.getFileMap();
        for (Map.Entry<String, MultipartFile> entity : fileMap.entrySet()) {
            MultipartFile file = entity.getValue();
            ImportParams params = new ImportParams();
            params.setTitleRows(2);
            params.setHeadRows(1);
            params.setNeedSave(true);
            try {
                List<KuaishouStrategy> listKuaishouStrategys = ExcelImportUtil.importExcel(file.getInputStream(), KuaishouStrategy.class, params);
                kuaishouStrategyService.saveBatch(listKuaishouStrategys);
                return Result.ok("文件导入成功！数据行数:" + listKuaishouStrategys.size());
            } catch (Exception e) {
                log.error(e.getMessage(), e);
                return Result.error("文件导入失败:" + e.getMessage());
            } finally {
                try {
                    file.getInputStream().close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        return Result.ok("文件导入失败！");
    }

}
