package org.jeecg.modules.ctop.controller;

import cn.com.ctop.common.module.annotation.AutoLog;
import cn.com.ctop.common.module.entity.CorpFeishuUser;
import cn.com.ctop.common.module.service.ICorpFeishuUserService;
import cn.com.ctop.common.module.utils.QueryGenerator;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.Arrays;

 /**
 * 飞书用户信息
 * @author jeecg-boot
 * @date   2020-06-23
 * @version V1.0
 */
@Slf4j
@Api(tags="飞书用户信息")
@RestController
@RequestMapping("/ctop/corpFeishuUser")
public class CorpFeishuUserController {
	@Autowired
	private ICorpFeishuUserService corpFeishuUserService;

	/**
	  * 分页列表查询
	 * @param corpFeishuUser
	 * @param pageNo
	 * @param pageSize
	 * @param req
	 * @return
	 */
	@AutoLog(value = "飞书用户信息-分页列表查询")
	@ApiOperation(value="飞书用户信息-分页列表查询", notes="飞书用户信息-分页列表查询")
	@GetMapping(value = "/list")
	public Result<IPage<CorpFeishuUser>> queryPageList(CorpFeishuUser corpFeishuUser,
													   @RequestParam(name="pageNo", defaultValue="1") Integer pageNo,
													   @RequestParam(name="pageSize", defaultValue="10") Integer pageSize,
													   HttpServletRequest req) {
		Result<IPage<CorpFeishuUser>> result = new Result<>();
		QueryWrapper<CorpFeishuUser> queryWrapper = QueryGenerator.initQueryWrapper(corpFeishuUser, req.getParameterMap());
		Page<CorpFeishuUser> page = new Page<CorpFeishuUser>(pageNo, pageSize);
		IPage<CorpFeishuUser> pageList = corpFeishuUserService.page(page, queryWrapper);
		result.setSuccess(true);
		result.setResult(pageList);
		return result;
	}

	/**
	  *   添加
	 * @param corpFeishuUser
	 * @return
	 */
	@AutoLog(value = "飞书用户信息-添加")
	@ApiOperation(value="飞书用户信息-添加", notes="飞书用户信息-添加")
	@PostMapping(value = "/add")
	public Result<CorpFeishuUser> add(@RequestBody CorpFeishuUser corpFeishuUser) {
		Result<CorpFeishuUser> result = new Result<>();
		try {
			corpFeishuUserService.save(corpFeishuUser);
			result.success("添加成功！");
		} catch (Exception e) {
			log.error(e.getMessage(),e);
			result.error500("操作失败");
		}
		return result;
	}

	/**
	  *  编辑
	 * @param corpFeishuUser
	 * @return
	 */
	@AutoLog(value = "飞书用户信息-编辑")
	@ApiOperation(value="飞书用户信息-编辑", notes="飞书用户信息-编辑")
	@PutMapping(value = "/edit")
	public Result<CorpFeishuUser> edit(@RequestBody CorpFeishuUser corpFeishuUser) {
		Result<CorpFeishuUser> result = new Result<>();
		CorpFeishuUser corpFeishuUserEntity = corpFeishuUserService.getById(corpFeishuUser.getId());
		if(corpFeishuUserEntity==null) {
			result.error500("未找到对应实体");
		}else {
			boolean ok = corpFeishuUserService.updateById(corpFeishuUser);
			if(ok) {
				result.success("修改成功!");
			}
		}

		return result;
	}

	/**
	  *   通过id删除
	 * @param id
	 * @return
	 */
	@AutoLog(value = "飞书用户信息-通过id删除")
	@ApiOperation(value="飞书用户信息-通过id删除", notes="飞书用户信息-通过id删除")
	@DeleteMapping(value = "/delete")
	public Result<Object> delete(@RequestParam(name="id",required=true) String id) {
		try {
			corpFeishuUserService.removeById(id);
		} catch (Exception e) {
			log.error("删除失败",e.getMessage());
			return Result.error("删除失败!");
		}
		return Result.ok("删除成功!");
	}

	/**
	  *  批量删除
	 * @param ids
	 * @return
	 */
	@AutoLog(value = "飞书用户信息-批量删除")
	@ApiOperation(value="飞书用户信息-批量删除", notes="飞书用户信息-批量删除")
	@DeleteMapping(value = "/deleteBatch")
	public Result<CorpFeishuUser> deleteBatch(@RequestParam(name="ids",required=true) String ids) {
		Result<CorpFeishuUser> result = new Result<>();
		if(ids==null || "".equals(ids.trim())) {
			result.error500("参数不识别！");
		}else {
			this.corpFeishuUserService.removeByIds(Arrays.asList(ids.split(",")));
			result.success("删除成功!");
		}
		return result;
	}

	/**
	  * 通过id查询
	 * @param id
	 * @return
	 */
	@AutoLog(value = "飞书用户信息-通过id查询")
	@ApiOperation(value="飞书用户信息-通过id查询", notes="飞书用户信息-通过id查询")
	@GetMapping(value = "/queryById")
	public Result<CorpFeishuUser> queryById(@RequestParam(name="id",required=true) String id) {
		Result<CorpFeishuUser> result = new Result<>();
		CorpFeishuUser corpFeishuUser = corpFeishuUserService.getById(id);
		if(corpFeishuUser==null) {
			result.error500("未找到对应实体");
		}else {
			result.setResult(corpFeishuUser);
			result.setSuccess(true);
		}
		return result;
	}
}
