package org.jeecg.modules.ctop.controller;

import cn.com.ctop.common.module.annotation.AutoLog;
import cn.com.ctop.common.module.entity.FileInfo;
import cn.com.ctop.common.module.service.IFileInfoService;
import cn.com.ctop.common.module.utils.QueryGenerator;
import cn.com.ctop.common.module.utils.StringUtils;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

/**
 * 文件信息
 * @author jeecg-boot
 * @date 2019-07-28
 * @version V1.0
 */
@Slf4j
@Api(tags = "文件信息")
@RestController
@RequestMapping("/ctop/fileInfo")
public class FileInfoController {
    @Autowired
    private IFileInfoService fileInfoService;

    /**
     * 分页列表查询
     *
     * @param fileInfo
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @AutoLog(value = "文件信息-分页列表查询")
    @ApiOperation(value = "文件信息-分页列表查询", notes = "文件信息-分页列表查询")
    @GetMapping(value = "/list")
    public Result<IPage<FileInfo>> queryPageList(FileInfo fileInfo,
                                                 @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                 @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                 HttpServletRequest req) {
        Result<IPage<FileInfo>> result = new Result<>();
        QueryWrapper<FileInfo> queryWrapper = QueryGenerator.initQueryWrapper(fileInfo, req.getParameterMap());
        Page<FileInfo> page = new Page<>(pageNo, pageSize);
        IPage<FileInfo> pageList = fileInfoService.page(page, queryWrapper);
        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }

    /**
     * 添加
     *
     * @param fileInfo
     * @return
     */
    @AutoLog(value = "文件信息-添加")
    @ApiOperation(value = "文件信息-添加", notes = "文件信息-添加")
    @PostMapping(value = "/add")
    public Result<FileInfo> add(@RequestBody FileInfo fileInfo) {
        Result<FileInfo> result = new Result<>();
        try {
            fileInfoService.save(fileInfo);
            result.success("添加成功！");
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            result.error500("操作失败");
        }
        return result;
    }

    /**
     * 编辑
     *
     * @param fileInfo
     * @return
     */
    @AutoLog(value = "文件信息-编辑")
    @ApiOperation(value = "文件信息-编辑", notes = "文件信息-编辑")
    @PutMapping(value = "/edit")
    public Result<FileInfo> edit(@RequestBody FileInfo fileInfo) {
        Result<FileInfo> result = new Result<>();
        FileInfo fileInfoEntity = fileInfoService.getById(fileInfo.getId());
        if (fileInfoEntity == null) {
            result.error500("未找到对应实体");
        } else {
            boolean ok = fileInfoService.updateById(fileInfo);
            if (ok) {
                result.success("修改成功!");
            }
        }

        return result;
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @AutoLog(value = "文件信息-通过id删除")
    @ApiOperation(value = "文件信息-通过id删除", notes = "文件信息-通过id删除")
    @DeleteMapping(value = "/delete")
    public Result<Object> delete(@RequestParam(name = "id", required = true) String id) {
        try {
            fileInfoService.removeById(id);
        } catch (Exception e) {
            log.error("删除失败", e.getMessage());
            return Result.error("删除失败!");
        }
        return Result.ok("删除成功!");
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @AutoLog(value = "文件信息-批量删除")
    @ApiOperation(value = "文件信息-批量删除", notes = "文件信息-批量删除")
    @DeleteMapping(value = "/deleteBatch")
    public Result<FileInfo> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        Result<FileInfo> result = new Result<>();
        if (ids == null || "".equals(ids.trim())) {
            result.error500("参数不识别！");
        } else {
            this.fileInfoService.removeByIds(Arrays.asList(ids.split(StringUtils.COMMA)));
            result.success("删除成功!");
        }
        return result;
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    @AutoLog(value = "文件信息-通过id查询")
    @ApiOperation(value = "文件信息-通过id查询", notes = "文件信息-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<FileInfo> queryById(@RequestParam(name = "id", required = true) String id) {
        Result<FileInfo> result = new Result<>();
        FileInfo fileInfo = fileInfoService.getById(id);
        if (fileInfo == null) {
            result.error500("未找到对应实体");
        } else {
            result.setResult(fileInfo);
            result.setSuccess(true);
        }
        return result;
    }

    /**
     * 通过excel导入数据
     *
     * @param request
     * @param response
     * @return
     */
    @PostMapping("/importExcel")
    public Result<Object> importExcel(HttpServletRequest request, HttpServletResponse response) {
        MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
        Map<String, MultipartFile> fileMap = multipartRequest.getFileMap();
        for (Map.Entry<String, MultipartFile> entity : fileMap.entrySet()) {
            MultipartFile file = entity.getValue();
            ImportParams params = new ImportParams();
            params.setTitleRows(2);
            params.setHeadRows(1);
            params.setNeedSave(true);
            try {
                List<FileInfo> listFileInfos = ExcelImportUtil.importExcel(file.getInputStream(), FileInfo.class, params);
                fileInfoService.saveBatch(listFileInfos);
                return Result.ok("文件导入成功！数据行数:" + listFileInfos.size());
            } catch (Exception e) {
                log.error(e.getMessage(), e);
                return Result.error("文件导入失败:" + e.getMessage());
            } finally {
                try {
                    file.getInputStream().close();
                } catch (IOException e) {
                    e.printStackTrace();
                    log.error("服务异常");
                }
            }
        }
        return Result.ok("文件导入失败！");
    }

}
