package org.jeecg.modules.ctop.controller;

import cn.com.ctop.common.module.entity.MaterialImageVideoInfo;
import cn.com.ctop.common.module.service.IMaterialImageVideoInfoService;
import cn.com.ctop.common.module.utils.QueryGenerator;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.Arrays;

/**
 * 视频封面关联表信息
 * @author jeecg-boot
 * @date   2020-12-17
 * @version V1.0
 */
@Slf4j
@Api(tags="视频封面关联表信息")
@RestController
@RequestMapping("/com.cn/materialImageVideoInfo")
public class MaterialImageVideoInfoController {
	@Autowired
	private IMaterialImageVideoInfoService materialImageVideoInfoService;

	/**
	  * 分页列表查询
	 * @param materialImageVideoInfo
	 * @param pageNo
	 * @param pageSize
	 * @param req
	 * @return
	 */
	@GetMapping(value = "/list")
	public Result<IPage<MaterialImageVideoInfo>> queryPageList(MaterialImageVideoInfo materialImageVideoInfo,
															   @RequestParam(name="pageNo", defaultValue="1") Integer pageNo,
															   @RequestParam(name="pageSize", defaultValue="10") Integer pageSize,
															   HttpServletRequest req) {
		Result<IPage<MaterialImageVideoInfo>> result = new Result<>();
		QueryWrapper<MaterialImageVideoInfo> queryWrapper = QueryGenerator.initQueryWrapper(materialImageVideoInfo, req.getParameterMap());
		Page<MaterialImageVideoInfo> page = new Page<MaterialImageVideoInfo>(pageNo, pageSize);
		IPage<MaterialImageVideoInfo> pageList = materialImageVideoInfoService.page(page, queryWrapper);
		result.setSuccess(true);
		result.setResult(pageList);
		return result;
	}

	/**
	  *   添加
	 * @param materialImageVideoInfo
	 * @return
	 */
	@PostMapping(value = "/add")
	public Result<MaterialImageVideoInfo> add(@RequestBody MaterialImageVideoInfo materialImageVideoInfo) {
		Result<MaterialImageVideoInfo> result = new Result<>();
		try {
			materialImageVideoInfoService.save(materialImageVideoInfo);
			result.success("添加成功！");
		} catch (Exception e) {
			log.error(e.getMessage(),e);
			result.error500("操作失败");
		}
		return result;
	}

	/**
	  *  编辑
	 * @param materialImageVideoInfo
	 * @return
	 */
	@PutMapping(value = "/edit")
	public Result<MaterialImageVideoInfo> edit(@RequestBody MaterialImageVideoInfo materialImageVideoInfo) {
		Result<MaterialImageVideoInfo> result = new Result<MaterialImageVideoInfo>();
		MaterialImageVideoInfo materialImageVideoInfoEntity = materialImageVideoInfoService.getById(materialImageVideoInfo.getId());
		if(materialImageVideoInfoEntity==null) {
			result.error500("未找到对应实体");
		}else {
			boolean ok = materialImageVideoInfoService.updateById(materialImageVideoInfo);
			if(ok) {
				result.success("修改成功!");
			}
		}

		return result;
	}

	/**
	  *   通过id删除
	 * @param id
	 * @return
	 */
	@DeleteMapping(value = "/delete")
	public Result<?> delete(@RequestParam(name="id",required=true) String id) {
		try {
			materialImageVideoInfoService.removeById(id);
		} catch (Exception e) {
			log.error("删除失败",e.getMessage());
			return Result.error("删除失败!");
		}
		return Result.ok("删除成功!");
	}

	/**
	  *  批量删除
	 * @param ids
	 * @return
	 */
	@DeleteMapping(value = "/deleteBatch")
	public Result<MaterialImageVideoInfo> deleteBatch(@RequestParam(name="ids",required=true) String ids) {
		Result<MaterialImageVideoInfo> result = new Result<>();
		if(ids==null || "".equals(ids.trim())) {
			result.error500("参数不识别！");
		}else {
			this.materialImageVideoInfoService.removeByIds(Arrays.asList(ids.split(",")));
			result.success("删除成功!");
		}
		return result;
	}

	/**
	  * 通过id查询
	 * @param id
	 * @return
	 */
	@GetMapping(value = "/queryById")
	public Result<MaterialImageVideoInfo> queryById(@RequestParam(name="id",required=true) String id) {
		Result<MaterialImageVideoInfo> result = new Result<>();
		MaterialImageVideoInfo materialImageVideoInfo = materialImageVideoInfoService.getById(id);
		if(materialImageVideoInfo==null) {
			result.error500("未找到对应实体");
		}else {
			result.setResult(materialImageVideoInfo);
			result.setSuccess(true);
		}
		return result;
	}
}
