package org.jeecg.modules.ctop.controller;

import cn.com.ctop.common.module.annotation.AutoLog;
import cn.com.ctop.common.module.entity.MaterialParameter;
import cn.com.ctop.common.module.service.IMaterialParameterService;
import cn.com.ctop.common.module.utils.QueryGenerator;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.Arrays;

/**
 * 素材归属标
 *
 * @author jeecg-boot
 * @version V1.0
 * @date 2019-11-23
 */
@Slf4j
@Api(tags = "素材归属标")
@RestController
@RequestMapping("/ctop/materialParameter")
public class MaterialParameterController {
    @Autowired
    private IMaterialParameterService materialParameterService;

    /**
     * 分页列表查询
     *
     * @param materialParameter
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @AutoLog(value = "素材归属标-分页列表查询")
    @ApiOperation(value = "素材归属标-分页列表查询", notes = "素材归属标-分页列表查询")
    @GetMapping(value = "/list")
    public Result<IPage<MaterialParameter>> queryPageList(MaterialParameter materialParameter,
                                                          @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                          @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                          HttpServletRequest req) {
        Result<IPage<MaterialParameter>> result = new Result<>();
        QueryWrapper<MaterialParameter> queryWrapper = QueryGenerator.initQueryWrapper(materialParameter, req.getParameterMap());
        Page<MaterialParameter> page = new Page<>(pageNo, pageSize);
        IPage<MaterialParameter> pageList = materialParameterService.page(page, queryWrapper);
        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }

    /**
     * 添加
     *
     * @param materialParameter
     * @return
     */
    @AutoLog(value = "素材归属标-添加")
    @ApiOperation(value = "素材归属标-添加", notes = "素材归属标-添加")
    @PostMapping(value = "/add")
    public Result<MaterialParameter> add(@RequestBody MaterialParameter materialParameter) {
        Result<MaterialParameter> result = new Result<>();
        try {
            materialParameterService.save(materialParameter);
            result.success("添加成功！");
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            result.error500("操作失败");
        }
        return result;
    }

    /**
     * 编辑
     *
     * @param materialParameter
     * @return
     */
    @AutoLog(value = "素材归属标-编辑")
    @ApiOperation(value = "素材归属标-编辑", notes = "素材归属标-编辑")
    @PutMapping(value = "/edit")
    public Result<MaterialParameter> edit(@RequestBody MaterialParameter materialParameter) {
        Result<MaterialParameter> result = new Result<>();
        MaterialParameter materialParameterEntity = materialParameterService.getById(materialParameter.getId());
        if (materialParameterEntity == null) {
            result.error500("未找到对应实体");
        } else {
            boolean ok = materialParameterService.updateById(materialParameter);
            if (ok) {
                result.success("修改成功!");
            }
        }

        return result;
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @AutoLog(value = "素材归属标-通过id删除")
    @ApiOperation(value = "素材归属标-通过id删除", notes = "素材归属标-通过id删除")
    @DeleteMapping(value = "/delete")
    public Result<Object> delete(@RequestParam(name = "id", required = true) String id) {
        try {
            materialParameterService.removeById(id);
        } catch (Exception e) {
            log.error("删除失败", e.getMessage());
            return Result.error("删除失败!");
        }
        return Result.ok("删除成功!");
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @AutoLog(value = "素材归属标-批量删除")
    @ApiOperation(value = "素材归属标-批量删除", notes = "素材归属标-批量删除")
    @DeleteMapping(value = "/deleteBatch")
    public Result<MaterialParameter> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        Result<MaterialParameter> result = new Result<>();
        if (ids == null || "".equals(ids.trim())) {
            result.error500("参数不识别！");
        } else {
            this.materialParameterService.removeByIds(Arrays.asList(ids.split(",")));
            result.success("删除成功!");
        }
        return result;
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    @AutoLog(value = "素材归属标-通过id查询")
    @ApiOperation(value = "素材归属标-通过id查询", notes = "素材归属标-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<MaterialParameter> queryById(@RequestParam(name = "id", required = true) String id) {
        Result<MaterialParameter> result = new Result<>();
        MaterialParameter materialParameter = materialParameterService.getById(id);
        if (materialParameter == null) {
            result.error500("未找到对应实体");
        } else {
            result.setResult(materialParameter);
            result.setSuccess(true);
        }
        return result;
    }
}
