package org.jeecg.modules.ctop.controller;

import cn.com.ctop.common.module.material.entity.MaterialVideoCover;
import cn.com.ctop.common.module.material.service.IMaterialVideoCoverService;
import cn.com.ctop.common.module.utils.QueryGenerator;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.Arrays;

 /**
 * 视频素材封面信息
 */
@Slf4j
@RestController
@RequestMapping("/com.cn/materialVideoCover")
public class MaterialVideoCoverController {

	@Autowired
	private IMaterialVideoCoverService materialVideoCoverService;

	/**
	 * 分页列表查询
	 */
	@GetMapping(value = "/list")
	public Result<IPage<MaterialVideoCover>> queryPageList(MaterialVideoCover materialVideoCover,
														   @RequestParam(name="pageNo", defaultValue="1") Integer pageNo,
														   @RequestParam(name="pageSize", defaultValue="10") Integer pageSize,
														   HttpServletRequest req) {
		Result<IPage<MaterialVideoCover>> result = new Result<>();
		QueryWrapper<MaterialVideoCover> queryWrapper = QueryGenerator.initQueryWrapper(materialVideoCover, req.getParameterMap());
		Page<MaterialVideoCover> page = new Page<>(pageNo, pageSize);
		IPage<MaterialVideoCover> pageList = materialVideoCoverService.page(page, queryWrapper);
		result.setSuccess(true);
		result.setResult(pageList);
		return result;
	}

	/**
	 *  编辑
	 */
	@PostMapping(value = "/edit")
	public Result<MaterialVideoCover> edit(@RequestBody MaterialVideoCover materialVideoCover) {
		Result<MaterialVideoCover> result = new Result<MaterialVideoCover>();
		MaterialVideoCover materialVideoCoverEntity = materialVideoCoverService.getById(materialVideoCover.getId());
		if(materialVideoCoverEntity==null) {
			result.error500("未找到对应实体");
		}else {
			boolean ok = materialVideoCoverService.updateById(materialVideoCover);
			if(ok) {
				result.success("修改成功!");
			}
		}
		return result;
	}

	/**
	 *   通过id删除
	 */
	@PostMapping(value = "/delete")
	public Result<?> delete(@RequestParam(name="id",required=true) String id) {
		try {
			materialVideoCoverService.removeById(id);
		} catch (Exception e) {
			log.error("删除失败",e.getMessage());
			return Result.error("删除失败!");
		}
		return Result.ok("删除成功!");
	}

	/**
	 *  批量删除
	 */
	@ApiOperation(value="视频素材封面信息-批量删除", notes="视频素材封面信息-批量删除")
	@DeleteMapping(value = "/deleteBatch")
	public Result<MaterialVideoCover> deleteBatch(@RequestParam(name="ids",required=true) String ids) {
		Result<MaterialVideoCover> result = new Result<>();
		if(ids==null || "".equals(ids.trim())) {
			result.error500("参数不识别！");
		}else {
			this.materialVideoCoverService.removeByIds(Arrays.asList(ids.split(",")));
			result.success("删除成功!");
		}
		return result;
	}

	/**
	 * 通过id查询
	 */
	@GetMapping(value = "/queryById")
	public Result<MaterialVideoCover> queryById(@RequestParam(name="id",required=true) String id) {
		Result<MaterialVideoCover> result = new Result<>();
		MaterialVideoCover materialVideoCover = materialVideoCoverService.getById(id);
		if(materialVideoCover==null) {
			result.error500("未找到对应实体");
		}else {
			result.setResult(materialVideoCover);
			result.setSuccess(true);
		}
		return result;
	}

}
