package org.jeecg.modules.ctop.controller;

import cn.com.ctop.common.module.utils.QueryGenerator;
import cn.com.ctop.crawler.modules.music.entity.MusicKuaishouType;
import cn.com.ctop.crawler.modules.music.service.IMusicKuaishouTypeService;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.Arrays;

 /**
 * 快手音乐类别
 * @author jeecg-boot
 * @date   2020-09-11
 * @version V1.0
 */
@Slf4j
@Api(tags="快手音乐类别")
@RestController
@RequestMapping("/ctop/musicKuaishouType")
public class MusicKuaishouTypeController {
	@Autowired
	private IMusicKuaishouTypeService musicKuaishouTypeService;

	/**
	  * 分页列表查询
	 * @param musicKuaishouType
	 * @param pageNo
	 * @param pageSize
	 * @param req
	 * @return
	 */
	@GetMapping(value = "/list")
	public Result<IPage<MusicKuaishouType>> queryPageList(MusicKuaishouType musicKuaishouType,
														  @RequestParam(name="pageNo", defaultValue="1") Integer pageNo,
														  @RequestParam(name="pageSize", defaultValue="10") Integer pageSize,
														  HttpServletRequest req) {
		Result<IPage<MusicKuaishouType>> result = new Result<>();
		QueryWrapper<MusicKuaishouType> queryWrapper = QueryGenerator.initQueryWrapper(musicKuaishouType, req.getParameterMap());
		Page<MusicKuaishouType> page = new Page<>(pageNo, pageSize);
		IPage<MusicKuaishouType> pageList = musicKuaishouTypeService.page(page, queryWrapper);
		result.setSuccess(true);
		result.setResult(pageList);
		return result;
	}

	/**
	  *   添加
	 * @param musicKuaishouType
	 * @return
	 */
	@PostMapping(value = "/add")
	public Result<MusicKuaishouType> add(@RequestBody MusicKuaishouType musicKuaishouType) {
		Result<MusicKuaishouType> result = new Result<>();
		try {
			musicKuaishouTypeService.save(musicKuaishouType);
			result.success("添加成功！");
		} catch (Exception e) {
			log.error(e.getMessage(),e);
			result.error500("操作失败");
		}
		return result;
	}

	/**
	  *  编辑
	 * @param musicKuaishouType
	 * @return
	 */
	@PutMapping(value = "/edit")
	public Result<MusicKuaishouType> edit(@RequestBody MusicKuaishouType musicKuaishouType) {
		Result<MusicKuaishouType> result = new Result<>();
		MusicKuaishouType musicKuaishouTypeEntity = musicKuaishouTypeService.getById(musicKuaishouType.getId());
		if(musicKuaishouTypeEntity==null) {
			result.error500("未找到对应实体");
		}else {
			boolean ok = musicKuaishouTypeService.updateById(musicKuaishouType);
			if(ok) {
				result.success("修改成功!");
			}
		}

		return result;
	}

	/**
	  *   通过id删除
	 * @param id
	 * @return
	 */
	@DeleteMapping(value = "/delete")
	public Result<?> delete(@RequestParam(name="id",required=true) String id) {
		try {
			musicKuaishouTypeService.removeById(id);
		} catch (Exception e) {
			log.error("删除失败",e.getMessage());
			return Result.error("删除失败!");
		}
		return Result.ok("删除成功!");
	}

	/**
	  *  批量删除
	 * @param ids
	 * @return
	 */
	@DeleteMapping(value = "/deleteBatch")
	public Result<MusicKuaishouType> deleteBatch(@RequestParam(name="ids",required=true) String ids) {
		Result<MusicKuaishouType> result = new Result<>();
		if(ids==null || "".equals(ids.trim())) {
			result.error500("参数不识别！");
		}else {
			this.musicKuaishouTypeService.removeByIds(Arrays.asList(ids.split(",")));
			result.success("删除成功!");
		}
		return result;
	}

	/**
	  * 通过id查询
	 * @param id
	 * @return
	 */
	@GetMapping(value = "/queryById")
	public Result<MusicKuaishouType> queryById(@RequestParam(name="id",required=true) String id) {
		Result<MusicKuaishouType> result = new Result<>();
		MusicKuaishouType musicKuaishouType = musicKuaishouTypeService.getById(id);
		if(musicKuaishouType==null) {
			result.error500("未找到对应实体");
		}else {
			result.setResult(musicKuaishouType);
			result.setSuccess(true);
		}
		return result;
	}
}
