package org.jeecg.modules.ctop.controller;

import cn.com.ctop.common.module.annotation.AutoLog;
import cn.com.ctop.common.module.utils.QueryGenerator;
import cn.com.ctop.common.module.utils.ResultMapUtils;
import cn.com.ctop.common.module.utils.StatusCode;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.shiro.SecurityUtils;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.system.vo.LoginUser;
import org.jeecg.modules.ctop.entity.ProjectTransferRecord;
import org.jeecg.modules.ctop.service.IProjectTransferRecordService;
import org.jeecg.modules.system.service.ISysRoleService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Map;

/**
 * 账户转项目记录
 *
 * @author jeecg-boot
 * @version V1.0
 * @date 2020-07-22
 */
@Slf4j
@Api(tags = "账户转项目记录")
@RestController
@RequestMapping("/ctop/projectTransferRecord")
public class ProjectTransferRecordController {
    @Autowired
    private IProjectTransferRecordService projectTransferRecordService;
    @Autowired
    private ISysRoleService sysRoleService;

    /**
     * 我发起的
     *
     * @param projectTransferRecord
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @AutoLog(value = "账户转项目记录-分页列表查询")
    @ApiOperation(value = "账户转项目记录-分页列表查询", notes = "账户转项目记录-分页列表查询")
    @GetMapping(value = "/initiatedList")
    public Result<IPage<ProjectTransferRecord>> initiatedList(ProjectTransferRecord projectTransferRecord,
                                                              @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                              @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                              HttpServletRequest req) {
        Integer status = projectTransferRecord.getStatus();
        if (null == status || status.equals(0)) {
            projectTransferRecord.setStatus(null);
        }
        Result<IPage<ProjectTransferRecord>> result = new Result<>();
        QueryWrapper<ProjectTransferRecord> queryWrapper = QueryGenerator.initQueryWrapper(projectTransferRecord, req.getParameterMap());
        LoginUser sysUser = (LoginUser) SecurityUtils.getSubject().getPrincipal();
        String rodeCode = sysRoleService.getRoleCodeByUserId(sysUser.getId());
        if (null != rodeCode && "admin".equals(rodeCode.trim())) {
            queryWrapper.eq("user_id", sysUser.getId());
        }
        queryWrapper.ne("status", 0);
        Page<ProjectTransferRecord> page = new Page<>(pageNo, pageSize);
        IPage<ProjectTransferRecord> pageList = projectTransferRecordService.page(page, queryWrapper);
        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }

    /**
     * 待审核的
     *
     * @param projectTransferRecord
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @AutoLog(value = "账户转项目记录-分页列表查询")
    @ApiOperation(value = "账户转项目记录-分页列表查询", notes = "账户转项目记录-分页列表查询")
    @GetMapping(value = "/auditList")
    public Result<IPage<ProjectTransferRecord>> auditList(ProjectTransferRecord projectTransferRecord,
                                                          @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                          @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                          HttpServletRequest req) {
        Integer status = projectTransferRecord.getStatus();
        if (null == status || status.equals(0)) {
            projectTransferRecord.setStatus(null);
        }
        Result<IPage<ProjectTransferRecord>> result = new Result<>();
        QueryWrapper<ProjectTransferRecord> queryWrapper = QueryGenerator.initQueryWrapper(projectTransferRecord, req.getParameterMap());
        LoginUser sysUser = (LoginUser) SecurityUtils.getSubject().getPrincipal();
        String rodeCode = sysRoleService.getRoleCodeByUserId(sysUser.getId());
        if (null != rodeCode && !"admin".equals(rodeCode.trim())) {
            queryWrapper.eq("reviewer_id", sysUser.getId());
        }
        queryWrapper.ne("status", 0);
        Page<ProjectTransferRecord> page = new Page<>(pageNo, pageSize);
        IPage<ProjectTransferRecord> pageList = projectTransferRecordService.page(page, queryWrapper);
        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }

    @PostMapping("editStatus")
    public Map<String, Object> editStatus(Long id, Integer status) {
        Map<String, Object> result = new HashMap<>();
        if (id == null || id == 0 || status == null) {
            ResultMapUtils.setResultMap(result, StatusCode.COMMON_PARAM_ERROR);
            return result;
        }
        return projectTransferRecordService.updateStatus(id, status);
    }

    /**
     * 添加
     *
     * @return
     */
    @AutoLog(value = "账户转项目记录-添加")
    @ApiOperation(value = "账户转项目记录-添加", notes = "账户转项目记录-添加")
    @PostMapping(value = "/add")
    public Map<String, Object> add(Long accountId, Long projectId) {
        return projectTransferRecordService.addRecord(accountId, projectId);
    }

    /**
     * 编辑
     *
     * @param projectTransferRecord
     * @return
     */
    @AutoLog(value = "账户转项目记录-编辑")
    @ApiOperation(value = "账户转项目记录-编辑", notes = "账户转项目记录-编辑")
    @PutMapping(value = "/edit")
    public Result<ProjectTransferRecord> edit(@RequestBody ProjectTransferRecord projectTransferRecord) {
        Result<ProjectTransferRecord> result = new Result<>();
        ProjectTransferRecord projectTransferRecordEntity = projectTransferRecordService.getById(projectTransferRecord.getId());
        if (projectTransferRecordEntity == null) {
            result.error500("未找到对应实体");
        } else {
            boolean ok = projectTransferRecordService.updateById(projectTransferRecord);
            if (ok) {
                result.success("修改成功!");
            }
        }

        return result;
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @AutoLog(value = "账户转项目记录-通过id删除")
    @ApiOperation(value = "账户转项目记录-通过id删除", notes = "账户转项目记录-通过id删除")
    @DeleteMapping(value = "/delete")
    public Result<Object> delete(@RequestParam(name = "id", required = true) Long id, @RequestParam(name = "status", defaultValue = "0") Integer status) {
        try {
            projectTransferRecordService.updateStatus(id, status);
        } catch (Exception e) {
            log.error("删除失败", e.getMessage());
            return Result.error("删除失败!");
        }
        return Result.ok("删除成功!");
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @AutoLog(value = "账户转项目记录-批量删除")
    @ApiOperation(value = "账户转项目记录-批量删除", notes = "账户转项目记录-批量删除")
    @DeleteMapping(value = "/deleteBatch")
    public Result<ProjectTransferRecord> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        Result<ProjectTransferRecord> result = new Result<>();
        if (ids == null || "".equals(ids.trim())) {
            result.error500("参数不识别！");
        } else {
            this.projectTransferRecordService.removeByIds(Arrays.asList(ids.split(",")));
            result.success("删除成功!");
        }
        return result;
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    @AutoLog(value = "账户转项目记录-通过id查询")
    @ApiOperation(value = "账户转项目记录-通过id查询", notes = "账户转项目记录-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<ProjectTransferRecord> queryById(@RequestParam(name = "id", required = true) String id) {
        Result<ProjectTransferRecord> result = new Result<>();
        ProjectTransferRecord projectTransferRecord = projectTransferRecordService.getById(id);
        if (projectTransferRecord == null) {
            result.error500("未找到对应实体");
        } else {
            result.setResult(projectTransferRecord);
            result.setSuccess(true);
        }
        return result;
    }
}
