package org.jeecg.modules.ctop.controller;

import cn.com.ctop.common.module.annotation.AutoLog;
import cn.com.ctop.common.module.utils.QueryGenerator;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.jeecg.modules.ctop.entity.ReleaseRecord;
import org.jeecg.modules.ctop.service.IReleaseRecordService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.Arrays;

 /**
 * 发版记录信息
 * @author jeecg-boot
 * @date   2020-06-10
 * @version V1.0
 */
@Slf4j
@Api(tags="发版记录信息")
@RestController
@RequestMapping("/ctop/releaseRecord")
public class ReleaseRecordController {
	@Autowired
	private IReleaseRecordService releaseRecordService;

	/**
	  * 分页列表查询
	 * @param releaseRecord
	 * @param pageNo
	 * @param pageSize
	 * @param req
	 * @return
	 */
	@AutoLog(value = "发版记录信息-分页列表查询")
	@ApiOperation(value="发版记录信息-分页列表查询", notes="发版记录信息-分页列表查询")
	@GetMapping(value = "/list")
	public Result<IPage<ReleaseRecord>> queryPageList(ReleaseRecord releaseRecord,
													  @RequestParam(name="pageNo", defaultValue="1") Integer pageNo,
													  @RequestParam(name="pageSize", defaultValue="10") Integer pageSize,
													  HttpServletRequest req) {
		Result<IPage<ReleaseRecord>> result = new Result<>();
		QueryWrapper<ReleaseRecord> queryWrapper = QueryGenerator.initQueryWrapper(releaseRecord, req.getParameterMap());
		Page<ReleaseRecord> page = new Page<ReleaseRecord>(pageNo, pageSize);
		IPage<ReleaseRecord> pageList = releaseRecordService.page(page, queryWrapper);
		result.setSuccess(true);
		result.setResult(pageList);
		return result;
	}

	/**
	  *   添加
	 * @param releaseRecord
	 * @return
	 */
	@AutoLog(value = "发版记录信息-添加")
	@ApiOperation(value="发版记录信息-添加", notes="发版记录信息-添加")
	@PostMapping(value = "/add")
	public Result<ReleaseRecord> add(@RequestBody ReleaseRecord releaseRecord) {
		Result<ReleaseRecord> result = new Result<>();
		try {
			releaseRecordService.save(releaseRecord);
			result.success("添加成功！");
		} catch (Exception e) {
			log.error(e.getMessage(),e);
			result.error500("操作失败");
		}
		return result;
	}

	/**
	  *  编辑
	 * @param releaseRecord
	 * @return
	 */
	@AutoLog(value = "发版记录信息-编辑")
	@ApiOperation(value="发版记录信息-编辑", notes="发版记录信息-编辑")
	@PutMapping(value = "/edit")
	public Result<ReleaseRecord> edit(@RequestBody ReleaseRecord releaseRecord) {
		Result<ReleaseRecord> result = new Result<ReleaseRecord>();
		ReleaseRecord releaseRecordEntity = releaseRecordService.getById(releaseRecord.getId());
		if(releaseRecordEntity==null) {
			result.error500("未找到对应实体");
		}else {
			boolean ok = releaseRecordService.updateById(releaseRecord);
			if(ok) {
				result.success("修改成功!");
			}
		}

		return result;
	}

	/**
	  *   通过id删除
	 * @param id
	 * @return
	 */
	@AutoLog(value = "发版记录信息-通过id删除")
	@ApiOperation(value="发版记录信息-通过id删除", notes="发版记录信息-通过id删除")
	@DeleteMapping(value = "/delete")
	public Result<?> delete(@RequestParam(name="id",required=true) String id) {
		try {
			releaseRecordService.removeById(id);
		} catch (Exception e) {
			log.error("删除失败",e.getMessage());
			return Result.error("删除失败!");
		}
		return Result.ok("删除成功!");
	}

	/**
	  *  批量删除
	 * @param ids
	 * @return
	 */
	@AutoLog(value = "发版记录信息-批量删除")
	@ApiOperation(value="发版记录信息-批量删除", notes="发版记录信息-批量删除")
	@DeleteMapping(value = "/deleteBatch")
	public Result<ReleaseRecord> deleteBatch(@RequestParam(name="ids",required=true) String ids) {
		Result<ReleaseRecord> result = new Result<>();
		if(ids==null || "".equals(ids.trim())) {
			result.error500("参数不识别！");
		}else {
			this.releaseRecordService.removeByIds(Arrays.asList(ids.split(",")));
			result.success("删除成功!");
		}
		return result;
	}

	/**
	  * 通过id查询
	 * @param id
	 * @return
	 */
	@AutoLog(value = "发版记录信息-通过id查询")
	@ApiOperation(value="发版记录信息-通过id查询", notes="发版记录信息-通过id查询")
	@GetMapping(value = "/queryById")
	public Result<ReleaseRecord> queryById(@RequestParam(name="id",required=true) String id) {
		Result<ReleaseRecord> result = new Result<>();
		ReleaseRecord releaseRecord = releaseRecordService.getById(id);
		if(releaseRecord==null) {
			result.error500("未找到对应实体");
		}else {
			result.setResult(releaseRecord);
			result.setSuccess(true);
		}
		return result;
	}
}
