package org.jeecg.modules.ctop.controller;

import cn.com.ctop.common.module.annotation.AutoLog;
import cn.com.ctop.common.module.utils.QueryGenerator;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.shiro.SecurityUtils;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.system.vo.LoginUser;
import org.jeecg.modules.ctop.entity.ReleaseViewRecord;
import org.jeecg.modules.ctop.service.IReleaseViewRecordService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.Arrays;
import java.util.Map;

/**
 * 发版记录信息
 * @author jeecg-boot
 * @date   2020-06-10
 * @version V1.0
 */
@Slf4j
@Api(tags="发版记录信息")
@RestController
@RequestMapping("/ctop/releaseViewRecord")
public class ReleaseViewRecordController {
	@Autowired
	private IReleaseViewRecordService releaseViewRecordService;

	@PostMapping("check")
	public Map<String,Object>check(){
		LoginUser user = (LoginUser) SecurityUtils.getSubject().getPrincipal();
		return releaseViewRecordService.check(user.getId());
	}
	@PostMapping("view")
	public Map<String,Object>view(@RequestBody JSONObject data){
		LoginUser user = (LoginUser) SecurityUtils.getSubject().getPrincipal();
		return releaseViewRecordService.view(user.getId(),data);
	}
	/**
	  * 分页列表查询
	 * @param releaseViewRecord
	 * @param pageNo
	 * @param pageSize
	 * @param req
	 * @return
	 */
	@AutoLog(value = "发版记录信息-分页列表查询")
	@ApiOperation(value="发版记录信息-分页列表查询", notes="发版记录信息-分页列表查询")
	@GetMapping(value = "/list")
	public Result<IPage<ReleaseViewRecord>> queryPageList(ReleaseViewRecord releaseViewRecord,
														  @RequestParam(name="pageNo", defaultValue="1") Integer pageNo,
														  @RequestParam(name="pageSize", defaultValue="10") Integer pageSize,
														  HttpServletRequest req) {
		Result<IPage<ReleaseViewRecord>> result = new Result<>();
		QueryWrapper<ReleaseViewRecord> queryWrapper = QueryGenerator.initQueryWrapper(releaseViewRecord, req.getParameterMap());
		Page<ReleaseViewRecord> page = new Page<>(pageNo, pageSize);
		IPage<ReleaseViewRecord> pageList = releaseViewRecordService.page(page, queryWrapper);
		result.setSuccess(true);
		result.setResult(pageList);
		return result;
	}

	/**
	  *   添加
	 * @param releaseViewRecord
	 * @return
	 */
	@AutoLog(value = "发版记录信息-添加")
	@ApiOperation(value="发版记录信息-添加", notes="发版记录信息-添加")
	@PostMapping(value = "/add")
	public Result<ReleaseViewRecord> add(@RequestBody ReleaseViewRecord releaseViewRecord) {
		Result<ReleaseViewRecord> result = new Result<>();
		try {
			releaseViewRecordService.save(releaseViewRecord);
			result.success("添加成功！");
		} catch (Exception e) {
			log.error(e.getMessage(),e);
			result.error500("操作失败");
		}
		return result;
	}

	/**
	  *  编辑
	 * @param releaseViewRecord
	 * @return
	 */
	@AutoLog(value = "发版记录信息-编辑")
	@ApiOperation(value="发版记录信息-编辑", notes="发版记录信息-编辑")
	@PutMapping(value = "/edit")
	public Result<ReleaseViewRecord> edit(@RequestBody ReleaseViewRecord releaseViewRecord) {
		Result<ReleaseViewRecord> result = new Result<>();
		ReleaseViewRecord releaseViewRecordEntity = releaseViewRecordService.getById(releaseViewRecord.getId());
		if(releaseViewRecordEntity==null) {
			result.error500("未找到对应实体");
		}else {
			boolean ok = releaseViewRecordService.updateById(releaseViewRecord);
			if(ok) {
				result.success("修改成功!");
			}
		}

		return result;
	}

	/**
	  *   通过id删除
	 * @param id
	 * @return
	 */
	@AutoLog(value = "发版记录信息-通过id删除")
	@ApiOperation(value="发版记录信息-通过id删除", notes="发版记录信息-通过id删除")
	@DeleteMapping(value = "/delete")
	public Result<?> delete(@RequestParam(name="id",required=true) String id) {
		try {
			releaseViewRecordService.removeById(id);
		} catch (Exception e) {
			log.error("删除失败",e.getMessage());
			return Result.error("删除失败!");
		}
		return Result.ok("删除成功!");
	}

	/**
	  *  批量删除
	 * @param ids
	 * @return
	 */
	@AutoLog(value = "发版记录信息-批量删除")
	@ApiOperation(value="发版记录信息-批量删除", notes="发版记录信息-批量删除")
	@DeleteMapping(value = "/deleteBatch")
	public Result<ReleaseViewRecord> deleteBatch(@RequestParam(name="ids",required=true) String ids) {
		Result<ReleaseViewRecord> result = new Result<>();
		if(ids==null || "".equals(ids.trim())) {
			result.error500("参数不识别！");
		}else {
			this.releaseViewRecordService.removeByIds(Arrays.asList(ids.split(",")));
			result.success("删除成功!");
		}
		return result;
	}

	/**
	  * 通过id查询
	 * @param id
	 * @return
	 */
	@AutoLog(value = "发版记录信息-通过id查询")
	@ApiOperation(value="发版记录信息-通过id查询", notes="发版记录信息-通过id查询")
	@GetMapping(value = "/queryById")
	public Result<ReleaseViewRecord> queryById(@RequestParam(name="id",required=true) String id) {
		Result<ReleaseViewRecord> result = new Result<>();
		ReleaseViewRecord releaseViewRecord = releaseViewRecordService.getById(id);
		if(releaseViewRecord==null) {
			result.error500("未找到对应实体");
		}else {
			result.setResult(releaseViewRecord);
			result.setSuccess(true);
		}
		return result;
	}
}
