package org.jeecg.modules.ctop.controller;

import cn.com.ctop.common.module.entity.RuleDataCreative;
import cn.com.ctop.common.module.service.IRuleDataCreativeService;
import cn.com.ctop.common.module.utils.QueryGenerator;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.Arrays;

 /**
 * 规则创意清洗数据
 * @author jeecg-boot
 * @date   2020-11-16
 * @version V1.0
 */
@Slf4j
@Api(tags="规则创意清洗数据")
@RestController
@RequestMapping("/ctop/rule/creative")
public class RuleDataCreativeController {
	@Autowired
	private IRuleDataCreativeService ruleDataCreativeService;

	/**
	  * 分页列表查询
	 * @param ruleDataCreative
	 * @param pageNo
	 * @param pageSize
	 * @param req
	 * @return
	 */
	@ApiOperation(value="规则创意清洗数据-分页列表查询", notes="规则创意清洗数据-分页列表查询")
	@GetMapping(value = "/list")
	public Result<IPage<RuleDataCreative>> queryPageList(RuleDataCreative ruleDataCreative,
														 @RequestParam(name="pageNo", defaultValue="1") Integer pageNo,
														 @RequestParam(name="pageSize", defaultValue="10") Integer pageSize,
														 HttpServletRequest req) {
		Result<IPage<RuleDataCreative>> result = new Result<>();
		QueryWrapper<RuleDataCreative> queryWrapper = QueryGenerator.initQueryWrapper(ruleDataCreative, req.getParameterMap());
		Page<RuleDataCreative> page = new Page<>(pageNo, pageSize);
		IPage<RuleDataCreative> pageList = ruleDataCreativeService.page(page, queryWrapper);
		result.setSuccess(true);
		result.setResult(pageList);
		return result;
	}

	/**
	  *   添加
	 * @param ruleDataCreative
	 * @return
	 */
	@ApiOperation(value="规则创意清洗数据-添加", notes="规则创意清洗数据-添加")
	@PostMapping(value = "/add")
	public Result<RuleDataCreative> add(@RequestBody RuleDataCreative ruleDataCreative) {
		Result<RuleDataCreative> result = new Result<>();
		try {
			ruleDataCreativeService.save(ruleDataCreative);
			result.success("添加成功！");
		} catch (Exception e) {
			log.error(e.getMessage(),e);
			result.error500("操作失败");
		}
		return result;
	}

	/**
	  *  编辑
	 * @param ruleDataCreative
	 * @return
	 */
	@ApiOperation(value="规则创意清洗数据-编辑", notes="规则创意清洗数据-编辑")
	@PutMapping(value = "/edit")
	public Result<RuleDataCreative> edit(@RequestBody RuleDataCreative ruleDataCreative) {
		Result<RuleDataCreative> result = new Result<RuleDataCreative>();
		RuleDataCreative ruleDataCreativeEntity = ruleDataCreativeService.getById(ruleDataCreative.getId());
		if(ruleDataCreativeEntity==null) {
			result.error500("未找到对应实体");
		}else {
			boolean ok = ruleDataCreativeService.updateById(ruleDataCreative);
			if(ok) {
				result.success("修改成功!");
			}
		}

		return result;
	}

	/**
	  *   通过id删除
	 * @param id
	 * @return
	 */
	@ApiOperation(value="规则创意清洗数据-通过id删除", notes="规则创意清洗数据-通过id删除")
	@DeleteMapping(value = "/delete")
	public Result<?> delete(@RequestParam(name="id",required=true) String id) {
		try {
			ruleDataCreativeService.removeById(id);
		} catch (Exception e) {
			log.error("删除失败",e.getMessage());
			return Result.error("删除失败!");
		}
		return Result.ok("删除成功!");
	}

	/**
	  *  批量删除
	 * @param ids
	 * @return
	 */
	@ApiOperation(value="规则创意清洗数据-批量删除", notes="规则创意清洗数据-批量删除")
	@DeleteMapping(value = "/deleteBatch")
	public Result<RuleDataCreative> deleteBatch(@RequestParam(name="ids",required=true) String ids) {
		Result<RuleDataCreative> result = new Result<>();
		if(ids==null || "".equals(ids.trim())) {
			result.error500("参数不识别！");
		}else {
			this.ruleDataCreativeService.removeByIds(Arrays.asList(ids.split(",")));
			result.success("删除成功!");
		}
		return result;
	}

	/**
	  * 通过id查询
	 * @param id
	 * @return
	 */
	@ApiOperation(value="规则创意清洗数据-通过id查询", notes="规则创意清洗数据-通过id查询")
	@GetMapping(value = "/queryById")
	public Result<RuleDataCreative> queryById(@RequestParam(name="id",required=true) String id) {
		Result<RuleDataCreative> result = new Result<>();
		RuleDataCreative ruleDataCreative = ruleDataCreativeService.getById(id);
		if(ruleDataCreative==null) {
			result.error500("未找到对应实体");
		}else {
			result.setResult(ruleDataCreative);
			result.setSuccess(true);
		}
		return result;
	}
}
