package org.jeecg.modules.ctop.entity;

import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableName;
import com.fasterxml.jackson.annotation.JsonFormat;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.experimental.Accessors;
import org.jeecgframework.poi.excel.annotation.Excel;
import org.springframework.format.annotation.DateTimeFormat;

import java.util.Date;

/**
 * 转户记录表
 *
 * @author jeecg-boot
 * @version V1.0
 * @date 2020-03-26
 */
@Data
@TableName("ctop_transfer_account")
@EqualsAndHashCode(callSuper = false)
@Accessors(chain = true)
@ApiModel(value = "ctop_transfer_account对象", description = "转户记录表")
public class TransferAccount {

    /**
     * 主键ID
     */
    @TableId(type = IdType.AUTO)
    @ApiModelProperty(value = "主键ID")
    private Long id;
    /**
     * 转户账号
     */
    @Excel(name = "转户账号", width = 15)
    @ApiModelProperty(value = "转户账号")
    private Long accountId;
    /**
     * 项目id
     */
    @Excel(name = "项目id", width = 15)
    @ApiModelProperty(value = "项目id")
    private Long projectId;
    /**
     * 转户人
     */
    @Excel(name = "转户人", width = 15)
    @ApiModelProperty(value = "转户人")
    private String transferor;
    /**
     * 受让人
     */
    @Excel(name = "受让人", width = 15)
    @ApiModelProperty(value = "受让人")
    private String assignee;
    /**
     * 转户人绩效统计截止日期
     */
    @Excel(name = "转户人绩效统计截止日期", width = 15)
    @ApiModelProperty(value = "转户人绩效统计截止日期")
    private String transferorDate;
    /**
     * 受让人绩效统计开始日期
     */
    @Excel(name = "受让人绩效统计开始日期", width = 15)
    @ApiModelProperty(value = "受让人绩效统计开始日期")
    private String assigneeDate;
    /**
     * 审核人
     */
    @Excel(name = "审核人", width = 15)
    @ApiModelProperty(value = "审核人")
    private String auditor;
    /**
     * 审核状态 0：新建 1:通过 2：拒绝
     */
    @Excel(name = "审核状态 0：新建 1:通过 2：拒绝", width = 15)
    @ApiModelProperty(value = "审核状态 0：新建 1:通过 2：拒绝")
    private Integer status;
    /**
     * 备注
     */
    @Excel(name = "备注", width = 15)
    @ApiModelProperty(value = "备注")
    private Object remarks;
    /**
     * 创建时间
     */
    @Excel(name = "创建时间", width = 20, format = "yyyy-MM-dd HH:mm:ss")
    @JsonFormat(timezone = "GMT+8", pattern = "yyyy-MM-dd HH:mm:ss")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @ApiModelProperty(value = "创建时间")
    private Date createTime;
    /**
     * 修改时间
     */
    @Excel(name = "修改时间", width = 20, format = "yyyy-MM-dd HH:mm:ss")
    @JsonFormat(timezone = "GMT+8", pattern = "yyyy-MM-dd HH:mm:ss")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @ApiModelProperty(value = "修改时间")
    private Date updateTime;

    @TableField(exist = false)
    private String transferorName;


    @TableField(exist = false)
    private String assigneeName;
}
