package org.jeecg.modules.ctop.job;

import cn.com.ctop.common.module.entity.CtopOauthToken;
import cn.com.ctop.common.module.service.ICtopOauthTokenService;
import cn.com.ctop.toutiao.modules.report.entity.BytedanceReportMaterialRetry;
import cn.com.ctop.toutiao.modules.report.service.IByteDanceVideoReportDailyService;
import cn.com.ctop.toutiao.modules.report.service.IBytedanceReportService;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.util.DateUtils;
import org.quartz.Job;
import org.quartz.JobExecutionContext;
import org.quartz.JobExecutionException;
import org.springframework.beans.factory.annotation.Autowired;

import java.util.List;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

/**
 * 头条素材报表错误重试定时任务
 *
 * @author sunzhen
 */
@Slf4j
public class BytedanceDailyMaterialReportRetryJob implements Job {
    @Autowired
    private ICtopOauthTokenService tokenService;
    @Autowired
    private IBytedanceReportService bytedanceReportService;
    @Autowired
    private IByteDanceVideoReportDailyService byteDanceVideoReportDailyService;

    @Override
    public void execute(JobExecutionContext jobExecutionContext) throws JobExecutionException {
        log.info("头条素材报表重试定时任务开始");
        Thread thread = new Thread() {
            @Override
            public void run() {

                List<BytedanceReportMaterialRetry> retryList = bytedanceReportService.getRetryList();
                //
                //多线程
                final ExecutorService executorService = Executors.newFixedThreadPool(3);
                retryList.forEach(retry -> {
                    executorService.submit(new Runnable() {
                        @Override
                        public void run() {
                            try {
                                String startDate = retry.getStartDate();
                                String endDate = retry.getEndDate();
                                Long accountId = retry.getAccountId();
                                Integer type = retry.getType();
                                CtopOauthToken token = tokenService.getOauthTokenByAccountId(String.valueOf(accountId));
                                log.info("头条素材报表重试定时任务，当前accountId为：" + token.getAccountId());
                                Long days = DateUtils.getDiscrepantDays(startDate, endDate); //间隔天数
                                String start = null;
                                String end = null;
                                for (int i = 0; i <= days; i++) {
                                    start = DateUtils.addDay(startDate, i);
                                    end = start;
                                    //获取头条素材报表数据
                                    int code = bytedanceReportService.bytedanceMaterialReportRetry(type, token, start, end);
                                    if (code == 200 || code == 1) {
                                        if (type == 1) {
                                            //重试成功清洗数据
                                            byteDanceVideoReportDailyService.videoInfoListByAccountId(start, end, accountId);
                                        }
                                    }
                                }
                            } catch (Exception e) {
                                e.printStackTrace();
                            } finally {
                            }
                        }
                    });
                });
            }
        };
        thread.start();

    }
}
