package org.jeecg.modules.ctop.job;

import cn.com.ctop.common.module.entity.CtopOauthToken;
import cn.com.ctop.common.module.service.ICtopOauthTokenService;
import cn.com.ctop.kuaishou.modules.batch.service.IKuaishouInterfaceService;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.util.DateUtils;
import org.quartz.Job;
import org.quartz.JobExecutionContext;
import org.quartz.JobExecutionException;
import org.springframework.beans.factory.annotation.Autowired;

import java.util.List;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

@Slf4j
public class KuaishouMaterialsLoadJob implements Job {
    @Autowired
    private ICtopOauthTokenService tokenService;
    @Autowired
    private IKuaishouInterfaceService kuaishouInterfaceService;

    static ExecutorService executorService = null;

    @Override
    public void execute(JobExecutionContext jobExecutionContext) throws JobExecutionException {
        Thread thread = new Thread() {
            @Override
            public void run() {
                List<CtopOauthToken> tokens = tokenService.selectKuaiShouToken();
                String endDate = DateUtils.getNowDate("yyyy-MM-dd");
                String startDate = DateUtils.getAnotherDay("yyyy-MM-dd", endDate, -1);
                executorService = Executors.newFixedThreadPool(8);
                tokens.forEach(token -> {
                    executorService.submit(new Runnable() {
                        @Override
                        public void run() {
                            try {

                                //获取全量视频素材数据
                                kuaishouInterfaceService.getVideoList(token, startDate, endDate);
                                // 获取图片信息数据
                                kuaishouInterfaceService.getImageList(token, DateUtils.getNowDate("yyyy-MM-dd"));
                            } catch (Exception e) {
                                e.printStackTrace();
                            } finally {
                            }
                        }
                    });
                });
                log.info("快手物料数据同步完成");
            }
        };
        thread.start();
    }
}
