package org.jeecg.modules.media.controller;

import cn.com.ctop.common.module.annotation.AutoLog;
import cn.com.ctop.common.module.utils.QueryGenerator;
import cn.com.ctop.common.module.utils.StringUtils;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.util.oConvertUtils;
import org.jeecg.modules.media.entity.Media;
import org.jeecg.modules.media.service.IMediaService;
import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

 /**
 * @Description: 媒体表
 * @Author: jeecg-boot
 * @Date:   2019-06-10
 * @Version: V1.0
 */
@Slf4j
@Api(tags="媒体表")
@RestController
@RequestMapping("/ctop/media")
public class MediaController {
	@Autowired
	private IMediaService mediaService;

	/**
	  * 分页列表查询
	 * @param media
	 * @param pageNo
	 * @param pageSize
	 * @param req
	 * @return
	 */
	@AutoLog(value = "媒体表-分页列表查询")
	@ApiOperation(value="媒体表-分页列表查询", notes="媒体表-分页列表查询")
	@GetMapping(value = "/list")
	public Result<IPage<Media>> queryPageList(Media media,
									  @RequestParam(name="pageNo", defaultValue="1") Integer pageNo,
									  @RequestParam(name="pageSize", defaultValue="10") Integer pageSize,
									  HttpServletRequest req) {
		Result<IPage<Media>> result = new Result<>();
		QueryWrapper<Media> queryWrapper = QueryGenerator.initQueryWrapper(media, req.getParameterMap());
		Page<Media> page = new Page<>(pageNo, pageSize);
		IPage<Media> pageList = mediaService.page(page, queryWrapper);
		result.setSuccess(true);
		result.setResult(pageList);
		return result;
	}

	/**
	  *   添加
	 * @param media
	 * @return
	 */
	@AutoLog(value = "媒体表-添加")
	@ApiOperation(value="媒体表-添加", notes="媒体表-添加")
	@PostMapping(value = "/add")
	public Result<Media> add(@RequestBody Media media) {
		Result<Media> result = new Result<>();
		try {
			mediaService.save(media);
			result.success("添加成功！");
		} catch (Exception e) {
			log.error(e.getMessage(),e);
			result.error500("操作失败");
		}
		return result;
	}

	/**
	  *  编辑
	 * @param media
	 * @return
	 */
	@AutoLog(value = "媒体表-编辑")
	@ApiOperation(value="媒体表-编辑", notes="媒体表-编辑")
	@PutMapping(value = "/edit")
	public Result<Media> edit(@RequestBody Media media) {
		Result<Media> result = new Result<>();
		Media mediaEntity = mediaService.getById(media.getId());
		if(mediaEntity==null) {
			result.error500("未找到对应实体");
		}else {
			boolean ok = mediaService.updateById(media);
			//TODO 返回false说明什么？
			if(ok) {
				result.success("修改成功!");
			}
		}

		return result;
	}

	 /**
	  *   通过id删除
	 * @param id
	 * @return
	 */
	@AutoLog(value = "媒体表-通过id删除")
	@ApiOperation(value="媒体表-通过id删除", notes="媒体表-通过id删除")
	@DeleteMapping(value = "/delete")
	public Result<Media> delete(@RequestParam(name="id",required=true) String id) {
		Result<Media> result = new Result<>();
		Media media = mediaService.getById(id);
		if(media==null) {
			result.error500("未找到对应实体");
		}else {
			boolean ok = mediaService.removeById(id);
			if(ok) {
				result.success("删除成功!");
			}
		}

		return result;
	}

	 /**
	  *  批量删除
	 * @param ids
	 * @return
	 */
	@AutoLog(value = "媒体表-批量删除")
	@ApiOperation(value="媒体表-批量删除", notes="媒体表-批量删除")
	@DeleteMapping(value = "/deleteBatch")
	public Result<Media> deleteBatch(@RequestParam(name="ids",required=true) String ids) {
		Result<Media> result = new Result<>();
		if(ids==null || "".equals(ids.trim())) {
			result.error500("参数不识别！");
		}else {
			this.mediaService.removeByIds(Arrays.asList(ids.split(StringUtils.COMMA)));
			result.success("删除成功!");
		}
		return result;
	}

	 /**
	  * 通过id查询
	 * @param id
	 * @return
	 */
	@AutoLog(value = "媒体表-通过id查询")
	@ApiOperation(value="媒体表-通过id查询", notes="媒体表-通过id查询")
	@GetMapping(value = "/queryById")
	public Result<Media> queryById(@RequestParam(name="id",required=true) String id) {
		Result<Media> result = new Result<>();
		Media media = mediaService.getById(id);
		if(media==null) {
			result.error500("未找到对应实体");
		}else {
			result.setResult(media);
			result.setSuccess(true);
		}
		return result;
	}

  /**
      * 导出excel
   *
   * @param request
   * @param response
   */
  @RequestMapping(value = "/exportXls")
  public ModelAndView exportXls(HttpServletRequest request, HttpServletResponse response) {
      // Step.1 组装查询条件
      QueryWrapper<Media> queryWrapper = null;
      try {
          String paramsStr = request.getParameter("paramsStr");
          if (oConvertUtils.isNotEmpty(paramsStr)) {
              String deString = URLDecoder.decode(paramsStr, "UTF-8");
              Media media = JSON.parseObject(deString, Media.class);
              queryWrapper = QueryGenerator.initQueryWrapper(media, request.getParameterMap());
          }
      } catch (UnsupportedEncodingException e) {
          e.printStackTrace();
      }

      //Step.2 AutoPoi 导出Excel
      ModelAndView mv = new ModelAndView(new JeecgEntityExcelView());
      List<Media> pageList = mediaService.list(queryWrapper);
      //导出文件名称
      mv.addObject(NormalExcelConstants.FILE_NAME, "媒体表列表");
      mv.addObject(NormalExcelConstants.CLASS, Media.class);
      mv.addObject(NormalExcelConstants.PARAMS, new ExportParams("媒体表列表数据", "导出人:Jeecg", "导出信息"));
      mv.addObject(NormalExcelConstants.DATA_LIST, pageList);
      return mv;
  }

  /**
      * 通过excel导入数据
   *
   * @param request
   * @param response
   * @return
   */
  @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
  public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
      MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
      Map<String, MultipartFile> fileMap = multipartRequest.getFileMap();
      for (Map.Entry<String, MultipartFile> entity : fileMap.entrySet()) {
          MultipartFile file = entity.getValue();
          ImportParams params = new ImportParams();
          params.setTitleRows(2);
          params.setHeadRows(1);
          params.setNeedSave(true);
          try {
              List<Media> listMedias = ExcelImportUtil.importExcel(file.getInputStream(), Media.class, params);
              for (Media mediaExcel : listMedias) {
                  mediaService.save(mediaExcel);
              }
              return Result.ok("文件导入成功！数据行数:" + listMedias.size());
          } catch (Exception e) {
              log.error(e.getMessage(),e);
              return Result.error("文件导入失败:"+e.getMessage());
          } finally {
              try {
                  file.getInputStream().close();
              } catch (IOException e) {
                  e.printStackTrace();
              }
          }
      }
      return Result.ok("文件导入失败！");
  }

}
