package org.jeecg.modules.opencv.controller;

import cn.com.ctop.common.module.annotation.AutoLog;
import cn.com.ctop.common.module.utils.QueryGenerator;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.shiro.SecurityUtils;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.system.vo.LoginUser;
import org.jeecg.modules.opencv.entity.ModelVideoInfoConfig;
import org.jeecg.modules.opencv.service.IFfmpegService;
import org.jeecg.modules.opencv.service.IModelVideoInfoConfigService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.Arrays;
import java.util.Map;

/**
 * 模板视频配置信息
 *
 * @author jeecg-boot
 * @version V1.0
 * @date 2020-04-17
 */
@Slf4j
@Api(tags = "模板视频配置信息")
@RestController
@RequestMapping("/ctop/modelVideoInfoConfig")
public class ModelVideoInfoConfigController {
    @Autowired
    private IModelVideoInfoConfigService modelVideoInfoConfigService;

    /**
     * 分页列表查询
     *
     * @param modelVideoInfoConfig
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @AutoLog(value = "模板视频配置信息-分页列表查询")
    @ApiOperation(value = "模板视频配置信息-分页列表查询", notes = "模板视频配置信息-分页列表查询")
    @GetMapping(value = "/list")
    public Result<IPage<ModelVideoInfoConfig>> queryPageList(ModelVideoInfoConfig modelVideoInfoConfig,
                                                             @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                             @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                             HttpServletRequest req) {
        Result<IPage<ModelVideoInfoConfig>> result = new Result<IPage<ModelVideoInfoConfig>>();
        QueryWrapper<ModelVideoInfoConfig> queryWrapper = QueryGenerator.initQueryWrapper(modelVideoInfoConfig, req.getParameterMap());
        Page<ModelVideoInfoConfig> page = new Page<ModelVideoInfoConfig>(pageNo, pageSize);
        IPage<ModelVideoInfoConfig> pageList = modelVideoInfoConfigService.page(page, queryWrapper);
        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }

    @RequestMapping("/videoMerge")
    public Map<String, Object> videoMerge(@RequestBody JSONObject data) {
        LoginUser user = (LoginUser) SecurityUtils.getSubject().getPrincipal();
        return ffmpegService.videoMerge(data, user.getId());
    }

    @Autowired
    IFfmpegService ffmpegService;

    /**
     * 添加
     *
     * @param modelVideoInfoConfig
     * @return
     */
    @AutoLog(value = "模板视频配置信息-添加")
    @ApiOperation(value = "模板视频配置信息-添加", notes = "模板视频配置信息-添加")
    @PostMapping(value = "/add")
    public Result<ModelVideoInfoConfig> add(@RequestBody ModelVideoInfoConfig modelVideoInfoConfig) {
        Result<ModelVideoInfoConfig> result = new Result<ModelVideoInfoConfig>();
        try {
            modelVideoInfoConfigService.save(modelVideoInfoConfig);
            result.success("添加成功！");
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            result.error500("操作失败");
        }
        return result;
    }

    /**
     * 编辑
     *
     * @param modelVideoInfoConfig
     * @return
     */
    @AutoLog(value = "模板视频配置信息-编辑")
    @ApiOperation(value = "模板视频配置信息-编辑", notes = "模板视频配置信息-编辑")
    @PutMapping(value = "/edit")
    public Result<ModelVideoInfoConfig> edit(@RequestBody ModelVideoInfoConfig modelVideoInfoConfig) {
        Result<ModelVideoInfoConfig> result = new Result<ModelVideoInfoConfig>();
        ModelVideoInfoConfig modelVideoInfoConfigEntity = modelVideoInfoConfigService.getById(modelVideoInfoConfig.getId());
        if (modelVideoInfoConfigEntity == null) {
            result.error500("未找到对应实体");
        } else {
            boolean ok = modelVideoInfoConfigService.updateById(modelVideoInfoConfig);
            if (ok) {
                result.success("修改成功!");
            }
        }

        return result;
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @AutoLog(value = "模板视频配置信息-通过id删除")
    @ApiOperation(value = "模板视频配置信息-通过id删除", notes = "模板视频配置信息-通过id删除")
    @DeleteMapping(value = "/delete")
    public Result<?> delete(@RequestParam(name = "id", required = true) String id) {
        try {
            modelVideoInfoConfigService.removeById(id);
        } catch (Exception e) {
            log.error("删除失败", e.getMessage());
            return Result.error("删除失败!");
        }
        return Result.ok("删除成功!");
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @AutoLog(value = "模板视频配置信息-批量删除")
    @ApiOperation(value = "模板视频配置信息-批量删除", notes = "模板视频配置信息-批量删除")
    @DeleteMapping(value = "/deleteBatch")
    public Result<ModelVideoInfoConfig> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        Result<ModelVideoInfoConfig> result = new Result<ModelVideoInfoConfig>();
        if (ids == null || "".equals(ids.trim())) {
            result.error500("参数不识别！");
        } else {
            this.modelVideoInfoConfigService.removeByIds(Arrays.asList(ids.split(",")));
            result.success("删除成功!");
        }
        return result;
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    @AutoLog(value = "模板视频配置信息-通过id查询")
    @ApiOperation(value = "模板视频配置信息-通过id查询", notes = "模板视频配置信息-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<ModelVideoInfoConfig> queryById(@RequestParam(name = "id", required = true) String id) {
        Result<ModelVideoInfoConfig> result = new Result<ModelVideoInfoConfig>();
        ModelVideoInfoConfig modelVideoInfoConfig = modelVideoInfoConfigService.getById(id);
        if (modelVideoInfoConfig == null) {
            result.error500("未找到对应实体");
        } else {
            result.setResult(modelVideoInfoConfig);
            result.setSuccess(true);
        }
        return result;
    }
}
