package org.jeecg.modules.system.controller;

import cn.com.ctop.common.module.annotation.AutoLog;
import cn.com.ctop.common.module.entity.SysUser;
import cn.com.ctop.common.module.entity.UserCompany;
import cn.com.ctop.common.module.mapper.SysUserMapper;
import cn.com.ctop.common.module.mapper.UserCompanyMapper;
import cn.com.ctop.common.module.service.IUserCompanyService;
import cn.com.ctop.common.module.utils.Check;
import cn.com.ctop.common.module.utils.QueryGenerator;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.util.oConvertUtils;
import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

/**
 * 用户够公司
 *
 * @author jeecg-boot
 * @version V1.0
 * @date 2019-11-14
 */
@Slf4j
@Api(tags = "用户够公司")
@RestController
@RequestMapping("/system/userCompany")
public class UserCompanyController {
    @Autowired
    private IUserCompanyService userCompanyService;
    @Resource
    private SysUserMapper sysUserMapper;
    @Resource
    private UserCompanyMapper userCompanyMapper;


    @GetMapping(value = "/companyUser")
    public void companyUser() {

        List<SysUser> sysUsers = sysUserMapper.selectAllUser();
        if (!Check.isNull(sysUsers)) {
            for (SysUser user : sysUsers) {
                if (!Check.isNull(user)) {
                    userCompanyService.addOrUpdateUserWithCompany(user);
                }
            }

        }
    }


    /**
     * 获取公司所有用户信息
     *
     * @param userId
     * @return
     */
    @PostMapping(value = "/getUserByCompany")
    public List<UserCompany> companyUser(@RequestBody String userId) {
        QueryWrapper<UserCompany> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("user_id", userId);
        UserCompany userCompany = userCompanyService.getOne(queryWrapper);
        if (!Check.isNull(userCompany)) {
            String companyId = userCompany.getCompanyId();
            queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("company_id", companyId);
            List<UserCompany> userCompanies = userCompanyMapper.selectList(queryWrapper);
            return userCompanies;
        }

        return null;
    }


    /**
     * 分页列表查询
     *
     * @param userCompany
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @AutoLog(value = "用户够公司-分页列表查询")
    @ApiOperation(value = "用户够公司-分页列表查询", notes = "用户够公司-分页列表查询")
    @GetMapping(value = "/list")
    public Result<IPage<UserCompany>> queryPageList(UserCompany userCompany,
                                                    @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                    @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                    HttpServletRequest req) {
        Result<IPage<UserCompany>> result = new Result<IPage<UserCompany>>();
        QueryWrapper<UserCompany> queryWrapper = QueryGenerator.initQueryWrapper(userCompany, req.getParameterMap());
        Page<UserCompany> page = new Page<UserCompany>(pageNo, pageSize);
        IPage<UserCompany> pageList = userCompanyService.page(page, queryWrapper);
        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }

    /**
     * 添加
     *
     * @param userCompany
     * @return
     */
    @AutoLog(value = "用户够公司-添加")
    @ApiOperation(value = "用户够公司-添加", notes = "用户够公司-添加")
    @PostMapping(value = "/add")
    public Result<UserCompany> add(@RequestBody UserCompany userCompany) {
        Result<UserCompany> result = new Result<UserCompany>();
        try {
            userCompanyService.save(userCompany);
            result.success("添加成功！");
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            result.error500("操作失败");
        }
        return result;
    }

    /**
     * 编辑
     *
     * @param userCompany
     * @return
     */
    @AutoLog(value = "用户够公司-编辑")
    @ApiOperation(value = "用户够公司-编辑", notes = "用户够公司-编辑")
    @PutMapping(value = "/edit")
    public Result<UserCompany> edit(@RequestBody UserCompany userCompany) {
        Result<UserCompany> result = new Result<UserCompany>();
        UserCompany userCompanyEntity = userCompanyService.getById(userCompany.getId());
        if (userCompanyEntity == null) {
            result.error500("未找到对应实体");
        } else {
            boolean ok = userCompanyService.updateById(userCompany);
            if (ok) {
                result.success("修改成功!");
            }
        }

        return result;
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @AutoLog(value = "用户够公司-通过id删除")
    @ApiOperation(value = "用户够公司-通过id删除", notes = "用户够公司-通过id删除")
    @DeleteMapping(value = "/delete")
    public Result<?> delete(@RequestParam(name = "id", required = true) String id) {
        try {
            userCompanyService.removeById(id);
        } catch (Exception e) {
            log.error("删除失败", e.getMessage());
            return Result.error("删除失败!");
        }
        return Result.ok("删除成功!");
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @AutoLog(value = "用户够公司-批量删除")
    @ApiOperation(value = "用户够公司-批量删除", notes = "用户够公司-批量删除")
    @DeleteMapping(value = "/deleteBatch")
    public Result<UserCompany> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        Result<UserCompany> result = new Result<UserCompany>();
        if (ids == null || "".equals(ids.trim())) {
            result.error500("参数不识别！");
        } else {
            this.userCompanyService.removeByIds(Arrays.asList(ids.split(",")));
            result.success("删除成功!");
        }
        return result;
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    @AutoLog(value = "用户够公司-通过id查询")
    @ApiOperation(value = "用户够公司-通过id查询", notes = "用户够公司-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<UserCompany> queryById(@RequestParam(name = "id", required = true) String id) {
        Result<UserCompany> result = new Result<UserCompany>();
        UserCompany userCompany = userCompanyService.getById(id);
        if (userCompany == null) {
            result.error500("未找到对应实体");
        } else {
            result.setResult(userCompany);
            result.setSuccess(true);
        }
        return result;
    }

    /**
     * 导出excel
     *
     * @param request
     * @param response
     */
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(HttpServletRequest request, HttpServletResponse response) {
        // Step.1 组装查询条件
        QueryWrapper<UserCompany> queryWrapper = null;
        try {
            String paramsStr = request.getParameter("paramsStr");
            if (oConvertUtils.isNotEmpty(paramsStr)) {
                String deString = URLDecoder.decode(paramsStr, "UTF-8");
                UserCompany userCompany = JSON.parseObject(deString, UserCompany.class);
                queryWrapper = QueryGenerator.initQueryWrapper(userCompany, request.getParameterMap());
            }
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }

        //Step.2 AutoPoi 导出Excel
        ModelAndView mv = new ModelAndView(new JeecgEntityExcelView());
        List<UserCompany> pageList = userCompanyService.list(queryWrapper);
        //导出文件名称
        mv.addObject(NormalExcelConstants.FILE_NAME, "用户够公司列表");
        mv.addObject(NormalExcelConstants.CLASS, UserCompany.class);
        mv.addObject(NormalExcelConstants.PARAMS, new ExportParams("用户够公司列表数据", "导出人:Jeecg", "导出信息"));
        mv.addObject(NormalExcelConstants.DATA_LIST, pageList);
        return mv;
    }

    /**
     * 通过excel导入数据
     *
     * @param request
     * @param response
     * @return
     */
    @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
        MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
        Map<String, MultipartFile> fileMap = multipartRequest.getFileMap();
        for (Map.Entry<String, MultipartFile> entity : fileMap.entrySet()) {
            MultipartFile file = entity.getValue();
            ImportParams params = new ImportParams();
            params.setTitleRows(2);
            params.setHeadRows(1);
            params.setNeedSave(true);
            try {
                List<UserCompany> listUserCompanys = ExcelImportUtil.importExcel(file.getInputStream(), UserCompany.class, params);
                userCompanyService.saveBatch(listUserCompanys);
                return Result.ok("文件导入成功！数据行数:" + listUserCompanys.size());
            } catch (Exception e) {
                log.error(e.getMessage(), e);
                return Result.error("文件导入失败:" + e.getMessage());
            } finally {
                try {
                    file.getInputStream().close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        return Result.ok("文件导入失败！");
    }

}
