package org.jeecg.modules.wps.controller;

import cn.com.ctop.common.module.annotation.AutoLog;
import cn.com.ctop.common.module.utils.QueryGenerator;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.jeecg.modules.wps.entity.WpsUser;
import org.jeecg.modules.wps.service.IWpsUserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.Arrays;

 /**
 * wps用户
 * @author jeecg-boot
 * @date   2020-06-01
 * @version V1.0
 */
@Slf4j
@Api(tags="wps用户")
@RestController
@RequestMapping("/ctop/wpsUser")
public class WpsUserController {
	@Autowired
	private IWpsUserService wpsUserService;

	/**
	  * 分页列表查询
	 * @param wpsUser
	 * @param pageNo
	 * @param pageSize
	 * @param req
	 * @return
	 */
	@AutoLog(value = "wps用户-分页列表查询")
	@ApiOperation(value="wps用户-分页列表查询", notes="wps用户-分页列表查询")
	@GetMapping(value = "/list")
	public Result<IPage<WpsUser>> queryPageList(WpsUser wpsUser,
												@RequestParam(name="pageNo", defaultValue="1") Integer pageNo,
												@RequestParam(name="pageSize", defaultValue="10") Integer pageSize,
												HttpServletRequest req) {
		Result<IPage<WpsUser>> result = new Result<>();
		QueryWrapper<WpsUser> queryWrapper = QueryGenerator.initQueryWrapper(wpsUser, req.getParameterMap());
		Page<WpsUser> page = new Page<>(pageNo, pageSize);
		IPage<WpsUser> pageList = wpsUserService.page(page, queryWrapper);
		result.setSuccess(true);
		result.setResult(pageList);
		return result;
	}

	/**
	  *   添加
	 * @param wpsUser
	 * @return
	 */
	@AutoLog(value = "wps用户-添加")
	@ApiOperation(value="wps用户-添加", notes="wps用户-添加")
	@PostMapping(value = "/add")
	public Result<WpsUser> add(@RequestBody WpsUser wpsUser) {
		Result<WpsUser> result = new Result<>();
		try {
			wpsUserService.save(wpsUser);
			result.success("添加成功！");
		} catch (Exception e) {
			log.error(e.getMessage(),e);
			result.error500("操作失败");
		}
		return result;
	}

	/**
	  *  编辑
	 * @param wpsUser
	 * @return
	 */
	@AutoLog(value = "wps用户-编辑")
	@ApiOperation(value="wps用户-编辑", notes="wps用户-编辑")
	@PutMapping(value = "/edit")
	public Result<WpsUser> edit(@RequestBody WpsUser wpsUser) {
		Result<WpsUser> result = new Result<>();
		WpsUser wpsUserEntity = wpsUserService.getById(wpsUser.getId());
		if(wpsUserEntity==null) {
			result.error500("未找到对应实体");
		}else {
			boolean ok = wpsUserService.updateById(wpsUser);
			if(ok) {
				result.success("修改成功!");
			}
		}

		return result;
	}

	/**
	  *   通过id删除
	 * @param id
	 * @return
	 */
	@AutoLog(value = "wps用户-通过id删除")
	@ApiOperation(value="wps用户-通过id删除", notes="wps用户-通过id删除")
	@DeleteMapping(value = "/delete")
	public Result<Object> delete(@RequestParam(name="id",required=true) String id) {
		try {
			wpsUserService.removeById(id);
		} catch (Exception e) {
			log.error("删除失败",e.getMessage());
			return Result.error("删除失败!");
		}
		return Result.ok("删除成功!");
	}

	/**
	  *  批量删除
	 * @param ids
	 * @return
	 */
	@AutoLog(value = "wps用户-批量删除")
	@ApiOperation(value="wps用户-批量删除", notes="wps用户-批量删除")
	@DeleteMapping(value = "/deleteBatch")
	public Result<WpsUser> deleteBatch(@RequestParam(name="ids",required=true) String ids) {
		Result<WpsUser> result = new Result<>();
		if(ids==null || "".equals(ids.trim())) {
			result.error500("参数不识别！");
		}else {
			this.wpsUserService.removeByIds(Arrays.asList(ids.split(",")));
			result.success("删除成功!");
		}
		return result;
	}

	/**
	  * 通过id查询
	 * @param id
	 * @return
	 */
	@AutoLog(value = "wps用户-通过id查询")
	@ApiOperation(value="wps用户-通过id查询", notes="wps用户-通过id查询")
	@GetMapping(value = "/queryById")
	public Result<WpsUser> queryById(@RequestParam(name="id",required=true) String id) {
		Result<WpsUser> result = new Result<>();
		WpsUser wpsUser = wpsUserService.getById(id);
		if(wpsUser==null) {
			result.error500("未找到对应实体");
		}else {
			result.setResult(wpsUser);
			result.setSuccess(true);
		}
		return result;
	}
}
