package cn.com.ctop.alarm.modules.controller;

import cn.com.ctop.alarm.modules.entity.AlarmEventSend;
import cn.com.ctop.alarm.modules.service.IAlarmEventSendService;
import cn.com.ctop.common.module.annotation.AutoLog;
import cn.com.ctop.common.module.utils.Check;
import cn.com.ctop.common.module.utils.QueryGenerator;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.util.oConvertUtils;
import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

/**
 * 预警事件发送表
 *
 * @author jeecg-boot
 * @version V1.0
 * @date 2020-10-29
 */
@Slf4j
@Api(tags = "预警事件发送表")
@RestController
@RequestMapping("/alarm/alarmEventSend")
public class AlarmEventSendController {
    @Autowired
    private IAlarmEventSendService alarmEventSendService;

    /**
     * 分页列表查询
     *
     * @param alarmEventSend
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @AutoLog(value = "预警事件发送表-分页列表查询")
    @ApiOperation(value = "预警事件发送表-分页列表查询", notes = "预警事件发送表-分页列表查询")
    @GetMapping(value = "/list")
    public Result<IPage<AlarmEventSend>> queryPageList(AlarmEventSend alarmEventSend,
                                                       @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                       @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                       HttpServletRequest req) {
        Result<IPage<AlarmEventSend>> result = new Result<>();
        QueryWrapper<AlarmEventSend> queryWrapper = QueryGenerator.initQueryWrapper(alarmEventSend, req.getParameterMap());
        Page<AlarmEventSend> page = new Page<AlarmEventSend>(pageNo, pageSize);
        IPage<AlarmEventSend> pageList = alarmEventSendService.page(page, queryWrapper);
        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }

    @AutoLog(value = "预警事件发送表-分页列表查询")
    @ApiOperation(value = "预警事件发送表-分页列表查询", notes = "预警事件发送表-分页列表查询")
    @GetMapping(value = "/pageList")
    public Result<Object> queryPageLists(AlarmEventSend alarmEventSend,
                                         @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                         @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                         HttpServletRequest req) {
        try {
            QueryWrapper<AlarmEventSend> queryWrapper = new QueryWrapper<>();
            if (!Check.isNull(alarmEventSend.getGroupName())) {
                queryWrapper.like("group_name", alarmEventSend.getGroupName());
            }
            if (!Check.isNull(alarmEventSend.getUserId())) {
                queryWrapper.like("user_id", alarmEventSend.getUserId());
            }
            if (!Check.isNull(alarmEventSend.getAlarmDetail())) {
                queryWrapper.like("alarm_detail", alarmEventSend.getAlarmDetail());
            }
            if (!Check.isNull(alarmEventSend.getMediaType())) {
                queryWrapper.like("media_type", alarmEventSend.getMediaType());
            }
            if (!Check.isNull(alarmEventSend.getAccountId())) {
                queryWrapper.eq("account_id", alarmEventSend.getAccountId());
            }
            if (!Check.isNull(alarmEventSend.getRuleDimension())) {
                queryWrapper.eq("rule_dimension", alarmEventSend.getRuleDimension());
            }
            queryWrapper.orderByDesc("create_time");
            PageHelper.startPage(pageNo, pageSize);
            List<AlarmEventSend> list = alarmEventSendService.list(queryWrapper);
            PageInfo<AlarmEventSend> pageInfo = new PageInfo<>(list);
            return Result.ok(pageInfo);
        } catch (Exception e) {
            log.error("预警历史发送数据-分页列表查询异常", e);
        }
        return Result.error("查询失败");
    }

    /**
     * 添加
     *
     * @param alarmEventSend
     * @return
     */
    @AutoLog(value = "预警事件发送表-添加")
    @ApiOperation(value = "预警事件发送表-添加", notes = "预警事件发送表-添加")
    @PostMapping(value = "/add")
    public Result<AlarmEventSend> add(@RequestBody AlarmEventSend alarmEventSend) {
        Result<AlarmEventSend> result = new Result<>();
        try {
            alarmEventSendService.save(alarmEventSend);
            result.success("添加成功！");
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            result.error500("操作失败");
        }
        return result;
    }

    /**
     * 编辑
     *
     * @param alarmEventSend
     * @return
     */
    @AutoLog(value = "预警事件发送表-编辑")
    @ApiOperation(value = "预警事件发送表-编辑", notes = "预警事件发送表-编辑")
    @PutMapping(value = "/edit")
    public Result<AlarmEventSend> edit(@RequestBody AlarmEventSend alarmEventSend) {
        Result<AlarmEventSend> result = new Result<AlarmEventSend>();
        AlarmEventSend alarmEventSendEntity = alarmEventSendService.getById(alarmEventSend.getAlarmSendId());
        if (alarmEventSendEntity == null) {
            result.error500("未找到对应实体");
        } else {
            boolean ok = alarmEventSendService.updateById(alarmEventSend);
            if (ok) {
                result.success("修改成功!");
            }
        }

        return result;
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @AutoLog(value = "预警事件发送表-通过id删除")
    @ApiOperation(value = "预警事件发送表-通过id删除", notes = "预警事件发送表-通过id删除")
    @DeleteMapping(value = "/delete")
    public Result<?> delete(@RequestParam(name = "id", required = true) String id) {
        try {
            alarmEventSendService.removeById(id);
        } catch (Exception e) {
            log.error("删除失败", e.getMessage());
            return Result.error("删除失败!");
        }
        return Result.ok("删除成功!");
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @AutoLog(value = "预警事件发送表-批量删除")
    @ApiOperation(value = "预警事件发送表-批量删除", notes = "预警事件发送表-批量删除")
    @DeleteMapping(value = "/deleteBatch")
    public Result<AlarmEventSend> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        Result<AlarmEventSend> result = new Result<>();
        if (ids == null || "".equals(ids.trim())) {
            result.error500("参数不识别！");
        } else {
            this.alarmEventSendService.removeByIds(Arrays.asList(ids.split(",")));
            result.success("删除成功!");
        }
        return result;
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    @AutoLog(value = "预警事件发送表-通过id查询")
    @ApiOperation(value = "预警事件发送表-通过id查询", notes = "预警事件发送表-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<AlarmEventSend> queryById(@RequestParam(name = "id", required = true) String id) {
        Result<AlarmEventSend> result = new Result<>();
        AlarmEventSend alarmEventSend = alarmEventSendService.getById(id);
        if (alarmEventSend == null) {
            result.error500("未找到对应实体");
        } else {
            result.setResult(alarmEventSend);
            result.setSuccess(true);
        }
        return result;
    }

    /**
     * 导出excel
     *
     * @param request
     * @param response
     */
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(HttpServletRequest request, HttpServletResponse response) {
        // Step.1 组装查询条件
        QueryWrapper<AlarmEventSend> queryWrapper = null;
        try {
            String paramsStr = request.getParameter("paramsStr");
            if (oConvertUtils.isNotEmpty(paramsStr)) {
                String deString = URLDecoder.decode(paramsStr, "UTF-8");
                AlarmEventSend alarmEventSend = JSON.parseObject(deString, AlarmEventSend.class);
                queryWrapper = QueryGenerator.initQueryWrapper(alarmEventSend, request.getParameterMap());
            }
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }

        //Step.2 AutoPoi 导出Excel
        ModelAndView mv = new ModelAndView(new JeecgEntityExcelView());
        List<AlarmEventSend> pageList = alarmEventSendService.list(queryWrapper);
        //导出文件名称
        mv.addObject(NormalExcelConstants.FILE_NAME, "预警事件发送表列表");
        mv.addObject(NormalExcelConstants.CLASS, AlarmEventSend.class);
        mv.addObject(NormalExcelConstants.PARAMS, new ExportParams("预警事件发送表列表数据", "导出人:Jeecg", "导出信息"));
        mv.addObject(NormalExcelConstants.DATA_LIST, pageList);
        return mv;
    }

    /**
     * 通过excel导入数据
     *
     * @param request
     * @param response
     * @return
     */
    @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
        MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
        Map<String, MultipartFile> fileMap = multipartRequest.getFileMap();
        for (Map.Entry<String, MultipartFile> entity : fileMap.entrySet()) {
            MultipartFile file = entity.getValue();
            ImportParams params = new ImportParams();
            params.setTitleRows(2);
            params.setHeadRows(1);
            params.setNeedSave(true);
            try {
                List<AlarmEventSend> listAlarmEventSends = ExcelImportUtil.importExcel(file.getInputStream(), AlarmEventSend.class, params);
                alarmEventSendService.saveBatch(listAlarmEventSends);
                return Result.ok("文件导入成功！数据行数:" + listAlarmEventSends.size());
            } catch (Exception e) {
                log.error(e.getMessage(), e);
                return Result.error("文件导入失败:" + e.getMessage());
            } finally {
                try {
                    file.getInputStream().close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        return Result.ok("文件导入失败！");
    }

}
