package cn.com.ctop.common.module.utils;


import lombok.extern.slf4j.Slf4j;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import java.io.File;
import java.io.IOException;
import java.sql.Timestamp;
import java.text.DecimalFormat;
import java.util.Arrays;
import java.util.Iterator;

@Slf4j
public class FileUploadTool {


    TransfMediaTool transfMediaTool = new TransfMediaTool();
    /**
     * 文件最大500M
     */
    private static long uploadMaxsize = 800 * 1024 * 1024L;
    /**
     * 文件允许格式
     */
    private static String[] allowfiles = {".rar", ".doc", ".docx", ".zip",
            ".pdf", ".txt", ".swf", ".xlsx", ".gif", ".png", ".jpg", ".jpeg",
            ".bmp", ".xls", ".mp4", ".flv", ".ppt", ".avi", ".mpg", ".wmv",
            ".3gp", ".mov", ".asf", ".asx", ".vob", ".wmv9", ".rm", ".rmvb", ".apk"};
    /**
     * 允许转码的视频格式（ffmpeg）
     */
    private static String[] allowflv = {".avi", ".mpg", ".wmv", ".3gp",
            ".mov", ".asf", ".asx", ".vob"};

    /**
     * 允许的视频转码格式(mencoder)
     */
    private static String[] allowavi = {".wmv9", ".rm", ".rmvb"};

    public FileEntity createFile(MultipartFile multipartFile, HttpServletRequest request, String savaPath) {
        FileEntity entity = new FileEntity();
        String fileName = multipartFile.getOriginalFilename();
        // 判断文件为空
        if (multipartFile.isEmpty() || multipartFile.getSize() == 0) {
            return null;
        }

        // 判断文件大小(大于文件大小限制)
        if (multipartFile.getSize() > uploadMaxsize) {
            return null;
        }
        // 文件类型判断
        if (!this.checkFileType(fileName)) {
            return null;
        }

        File logoSaveFile = new File(savaPath);
        if (!logoSaveFile.exists()) {
            logoSaveFile.mkdirs();
        }
        String name = fileName.substring(0, fileName.lastIndexOf('.'));
        // 新的文件名
        String newFileName = this.getName(fileName);
        // 文件扩展名
        String fileEnd = this.getFileExt(fileName);
        // 绝对路径
        String fileNamedirs = savaPath + File.separator + newFileName + fileEnd;
        File filedirs = new File(fileNamedirs);
        // 转入文件
        try {
            multipartFile.transferTo(filedirs);
        } catch (IllegalStateException | IOException e) {
            log.error(e.getMessage(), e);
        }
        // 相对路径
        entity.setType(fileEnd);
        // size存储为String
        String size = this.getSize(filedirs);
        // 源文件保存路径
        String aviPath = filedirs.getAbsolutePath();
        // 转码Avi
        if (this.checkAviType(fileEnd)) {
            // 设置转换为AVI格式后文件的保存路径
            String codcAviPath = savaPath + File.separator + newFileName + ".avi";
            // 获取配置的转换工具（mencoder.exe）的存放路径
            String mencoderPath = request.getSession().getServletContext().getRealPath("/tools/mencoder.exe");
            aviPath = transfMediaTool.processAvi(mencoderPath, filedirs.getAbsolutePath(), codcAviPath);
            fileEnd = this.getFileExt(codcAviPath);
        }
        if (aviPath == null) {
            return null;
        }
        // 转码Flv
        if (this.checkMediaType(fileEnd)) {
            try {
                // 设置转换为flv格式后文件的保存路径
                String codcFilePath = savaPath + File.separator + newFileName + ".flv";
                // 获取配置的转换工具（ffmpeg.exe）的存放路径
                String ffmpegPath = request.getSession().getServletContext().getRealPath("/tools/ffmpeg.exe");
                transfMediaTool.processFlv(ffmpegPath, aviPath, codcFilePath);
            } catch (Exception e) {
                log.error(e.getMessage(), e);
            }
        }
        entity.setSize(size);
        entity.setPath(fileNamedirs);
        entity.setTitleOrig(name);
        entity.setTitleAlter(newFileName);
        Timestamp timestamp = new Timestamp(System.currentTimeMillis());
        entity.setUploadTime(timestamp);
        return entity;
    }

    /**
     * 文件类型判断
     *
     * @param fileName
     * @return
     */
    private boolean checkFileType(String fileName) {
        Iterator<String> type = Arrays.asList(allowfiles).iterator();
        while (type.hasNext()) {
            String ext = type.next();
            if (fileName.toLowerCase().endsWith(ext)) {
                return true;
            }
        }
        return false;
    }

    /**
     * 视频类型判断(flv)
     *
     * @param
     * @return
     */
    private boolean checkMediaType(String fileEnd) {
        Iterator<String> type = Arrays.asList(allowflv).iterator();
        while (type.hasNext()) {
            String ext = type.next();
            if (fileEnd.equals(ext)) {
                return true;
            }
        }
        return false;
    }

    /**
     * 视频类型判断(AVI)
     *
     * @param
     * @return
     */
    private boolean checkAviType(String fileEnd) {
        Iterator<String> type = Arrays.asList(allowavi).iterator();
        while (type.hasNext()) {
            String ext = type.next();
            if (fileEnd.equals(ext)) {
                return true;
            }
        }
        return false;
    }

    /**
     * 获取文件扩展名
     *
     * @return string
     */
    private String getFileExt(String fileName) {
        return fileName.substring(fileName.lastIndexOf('.'));
    }

    /**
     * 依据原始文件名生成新文件名
     *
     * @return
     */
    private String getName(String fileName) {
        Iterator<String> type = Arrays.asList(allowfiles).iterator();
        while (type.hasNext()) {
            String ext = type.next();
            if (fileName.contains(ext)) {
                return fileName.substring(0, fileName.lastIndexOf(ext));
            }
        }
        return "";
    }

    /**
     * 文件大小，返回kb.mb
     *
     * @return
     */
    private String getSize(File file) {
        String size = "";
        long fileLength = file.length();
        DecimalFormat df = new DecimalFormat("#.00");
        if (fileLength < 1024) {
            size = df.format((double) fileLength) + "BT";
        } else if (fileLength < 1048576) {
            size = df.format((double) fileLength / 1024) + "KB";
        } else if (fileLength < 1073741824) {
            size = df.format((double) fileLength / 1048576) + "MB";
        } else {
            size = df.format((double) fileLength / 1073741824) + "GB";
        }
        return size;
    }
}



