package cn.com.ctop.job.bytedance.handler;

import cn.com.ctop.common.module.entity.CtopOauthToken;
import cn.com.ctop.common.module.service.ICtopOauthTokenService;
import cn.com.ctop.common.module.utils.CtopAdConstant;
import cn.com.ctop.toutiao.modules.report.service.IReportService;
import com.xxl.job.core.context.XxlJobHelper;
import com.xxl.job.core.handler.annotation.XxlJob;
import org.jeecg.common.util.DateUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Date;
import java.util.List;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

/**
 * @author 宋英豪
 * 头条广告主日报数据获取任务
 * 第二天凌晨跑前一天的数据
 */
@Component
public class BytedanceAdvertiserDailyReportLoadJob {
    @Autowired
    private ICtopOauthTokenService tokenService;
    @Autowired
    private IReportService reportService;
    static ExecutorService executorService = Executors.newFixedThreadPool(5);;

    @XxlJob("bytedanceAdvertiserDailyReport")
    public void execute() throws Exception {
        XxlJobHelper.log("头条广告主日报获取任务执行开始");
        Date getDate = DateUtils.addDay(new Date(), -1);
        //1:查询当日数据
        List<CtopOauthToken> tokens = tokenService.selectToutiaoToken();
        if (null == tokens || tokens.isEmpty()) {
            XxlJobHelper.log("定时获取头条数据异常：为获取到可用的token");
            XxlJobHelper.handleFail();
            return ;
        }
        CountDownLatch countDownLatch = new CountDownLatch(tokens.size());
        tokens.forEach(token -> executorService.submit(() -> {
            try {
                reportService.getAdvertiserReport(token, getDate, getDate, CtopAdConstant.BYTEDANCE_REPORT_TYPE_DAILY);
            } catch (Exception e) {
                XxlJobHelper.log(e);
            } finally {
                countDownLatch.countDown();
            }
        }));
        countDownLatch.await();
        XxlJobHelper.log("头条广告主日报获取任务执行结束");
    }
}
