package cn.com.ctop.kuaishou.modules.batch.controller;

import cn.com.ctop.common.module.annotation.AutoLog;
import cn.com.ctop.common.module.utils.QueryGenerator;
import cn.com.ctop.common.module.utils.StringUtils;
import cn.com.ctop.kuaishou.modules.batch.entity.KuaiShouAdvertiserBaseInfo;
import cn.com.ctop.kuaishou.modules.batch.service.IKuaiShouAdvertiserBaseInfoService;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.Arrays;

/**
 * 快手-广告主基本信息
 *
 * @author jeecg-boot
 * @version V1.0
 * @date 2019-09-18
 */
@Slf4j
@Api(tags = "快手-广告主基本信息")
@RestController
@RequestMapping("/kuaishou/kuaiShouAdvertiserBaseInfo")
public class KuaiShouAdvertiserBaseInfoController {
    @Autowired
    private IKuaiShouAdvertiserBaseInfoService kuaiShouAdvertiserBaseInfoService;

    /**
     * 分页列表查询
     *
     * @param kuaiShouAdvertiserBaseInfo
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @AutoLog(value = "快手-广告主基本信息-分页列表查询")
    @ApiOperation(value = "快手-广告主基本信息-分页列表查询", notes = "快手-广告主基本信息-分页列表查询")
    @GetMapping(value = "/list")
    public Result<IPage<KuaiShouAdvertiserBaseInfo>> queryPageList(KuaiShouAdvertiserBaseInfo kuaiShouAdvertiserBaseInfo,
                                                                   @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                                   @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                                   HttpServletRequest req) {
        Result<IPage<KuaiShouAdvertiserBaseInfo>> result = new Result<>();
        QueryWrapper<KuaiShouAdvertiserBaseInfo> queryWrapper = QueryGenerator.initQueryWrapper(kuaiShouAdvertiserBaseInfo, req.getParameterMap());
        Page<KuaiShouAdvertiserBaseInfo> page = new Page<>(pageNo, pageSize);
        IPage<KuaiShouAdvertiserBaseInfo> pageList = kuaiShouAdvertiserBaseInfoService.page(page, queryWrapper);
        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }

    /**
     * 添加
     *
     * @param kuaiShouAdvertiserBaseInfo
     * @return
     */
    @AutoLog(value = "快手-广告主基本信息-添加")
    @ApiOperation(value = "快手-广告主基本信息-添加", notes = "快手-广告主基本信息-添加")
    @PostMapping(value = "/add")
    public Result<KuaiShouAdvertiserBaseInfo> add(@RequestBody KuaiShouAdvertiserBaseInfo kuaiShouAdvertiserBaseInfo) {
        Result<KuaiShouAdvertiserBaseInfo> result = new Result<>();
        try {
            kuaiShouAdvertiserBaseInfoService.save(kuaiShouAdvertiserBaseInfo);
            result.success("添加成功！");
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            result.error500("操作失败");
        }
        return result;
    }

    /**
     * 编辑
     *
     * @param kuaiShouAdvertiserBaseInfo
     * @return
     */
    @AutoLog(value = "快手-广告主基本信息-编辑")
    @ApiOperation(value = "快手-广告主基本信息-编辑", notes = "快手-广告主基本信息-编辑")
    @PutMapping(value = "/edit")
    public Result<KuaiShouAdvertiserBaseInfo> edit(@RequestBody KuaiShouAdvertiserBaseInfo kuaiShouAdvertiserBaseInfo) {
        Result<KuaiShouAdvertiserBaseInfo> result = new Result<>();
        KuaiShouAdvertiserBaseInfo kuaiShouAdvertiserBaseInfoEntity = kuaiShouAdvertiserBaseInfoService.getById(kuaiShouAdvertiserBaseInfo.getId());
        if (kuaiShouAdvertiserBaseInfoEntity == null) {
            result.error500("未找到对应实体");
        } else {
            boolean ok = kuaiShouAdvertiserBaseInfoService.updateById(kuaiShouAdvertiserBaseInfo);
            if (ok) {
                result.success("修改成功!");
            }
        }

        return result;
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @AutoLog(value = "快手-广告主基本信息-通过id删除")
    @ApiOperation(value = "快手-广告主基本信息-通过id删除", notes = "快手-广告主基本信息-通过id删除")
    @DeleteMapping(value = "/delete")
    public Result<Object> delete(@RequestParam(name = "id", required = true) String id) {
        try {
            kuaiShouAdvertiserBaseInfoService.removeById(id);
        } catch (Exception e) {
            log.error("删除失败", e.getMessage());
            return Result.error("删除失败!");
        }
        return Result.ok("删除成功!");
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @AutoLog(value = "快手-广告主基本信息-批量删除")
    @ApiOperation(value = "快手-广告主基本信息-批量删除", notes = "快手-广告主基本信息-批量删除")
    @DeleteMapping(value = "/deleteBatch")
    public Result<KuaiShouAdvertiserBaseInfo> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        Result<KuaiShouAdvertiserBaseInfo> result = new Result<>();
        if (ids == null || "".equals(ids.trim())) {
            result.error500("参数不识别！");
        } else {
            this.kuaiShouAdvertiserBaseInfoService.removeByIds(Arrays.asList(ids.split(StringUtils.COMMA)));
            result.success("删除成功!");
        }
        return result;
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    @AutoLog(value = "快手-广告主基本信息-通过id查询")
    @ApiOperation(value = "快手-广告主基本信息-通过id查询", notes = "快手-广告主基本信息-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<KuaiShouAdvertiserBaseInfo> queryById(@RequestParam(name = "id", required = true) String id) {
        Result<KuaiShouAdvertiserBaseInfo> result = new Result<>();
        KuaiShouAdvertiserBaseInfo kuaiShouAdvertiserBaseInfo = kuaiShouAdvertiserBaseInfoService.getById(id);
        if (kuaiShouAdvertiserBaseInfo == null) {
            result.error500("未找到对应实体");
        } else {
            result.setResult(kuaiShouAdvertiserBaseInfo);
            result.setSuccess(true);
        }
        return result;
    }
}
