package cn.com.ctop.kuaishou.modules.batch.controller;

import cn.com.ctop.common.module.annotation.AutoLog;
import cn.com.ctop.common.module.utils.QueryGenerator;
import cn.com.ctop.common.module.utils.StringUtils;
import cn.com.ctop.kuaishou.modules.batch.entity.KuaiShouAppList;
import cn.com.ctop.kuaishou.modules.batch.service.IKuaiShouAppListService;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.Arrays;

/**
 * @Description: 快手-应用列表
 * @Author: jeecg-boot
 * @Date: 2019-07-26
 * @Version: V1.0
 */
@Slf4j
@Api(tags = "快手-应用列表")
@RestController
@RequestMapping("/kuaishou/kuaiShouAppList")
public class KuaiShouAppListController {
    @Autowired
    private IKuaiShouAppListService kuaiShouAppListService;

    /**
     * 分页列表查询
     *
     * @param kuaiShouAppList
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @AutoLog(value = "快手-应用列表-分页列表查询")
    @ApiOperation(value = "快手-应用列表-分页列表查询", notes = "快手-应用列表-分页列表查询")
    @GetMapping(value = "/list")
    public Result<IPage<KuaiShouAppList>> queryPageList(KuaiShouAppList kuaiShouAppList,
                                                        @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                        @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                        HttpServletRequest req) {
        Result<IPage<KuaiShouAppList>> result = new Result<>();
        QueryWrapper<KuaiShouAppList> queryWrapper = QueryGenerator.initQueryWrapper(kuaiShouAppList, req.getParameterMap());
        Page<KuaiShouAppList> page = new Page<>(pageNo, pageSize);
        IPage<KuaiShouAppList> pageList = kuaiShouAppListService.page(page, queryWrapper);
        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }

    /**
     * 添加
     *
     * @param kuaiShouAppList
     * @return
     */
    @AutoLog(value = "快手-应用列表-添加")
    @ApiOperation(value = "快手-应用列表-添加", notes = "快手-应用列表-添加")
    @PostMapping(value = "/add")
    public Result<KuaiShouAppList> add(@RequestBody KuaiShouAppList kuaiShouAppList) {
        Result<KuaiShouAppList> result = new Result<>();
        try {
            kuaiShouAppListService.save(kuaiShouAppList);
            result.success("添加成功！");
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            result.error500("操作失败");
        }
        return result;
    }

    /**
     * 编辑
     *
     * @param kuaiShouAppList
     * @return
     */
    @AutoLog(value = "快手-应用列表-编辑")
    @ApiOperation(value = "快手-应用列表-编辑", notes = "快手-应用列表-编辑")
    @PutMapping(value = "/edit")
    public Result<KuaiShouAppList> edit(@RequestBody KuaiShouAppList kuaiShouAppList) {
        Result<KuaiShouAppList> result = new Result<>();
        KuaiShouAppList kuaiShouAppListEntity = kuaiShouAppListService.getById(kuaiShouAppList.getId());
        if (kuaiShouAppListEntity == null) {
            result.error500("未找到对应实体");
        } else {
            boolean ok = kuaiShouAppListService.updateById(kuaiShouAppList);
            if (ok) {
                result.success("修改成功!");
            }
        }

        return result;
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @AutoLog(value = "快手-应用列表-通过id删除")
    @ApiOperation(value = "快手-应用列表-通过id删除", notes = "快手-应用列表-通过id删除")
    @DeleteMapping(value = "/delete")
    public Result<Object> delete(@RequestParam(name = "id", required = true) String id) {
        try {
            kuaiShouAppListService.removeById(id);
        } catch (Exception e) {
            log.error("删除失败", e.getMessage());
            return Result.error("删除失败!");
        }
        return Result.ok("删除成功!");
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @AutoLog(value = "快手-应用列表-批量删除")
    @ApiOperation(value = "快手-应用列表-批量删除", notes = "快手-应用列表-批量删除")
    @DeleteMapping(value = "/deleteBatch")
    public Result<KuaiShouAppList> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        Result<KuaiShouAppList> result = new Result<>();
        if (ids == null || "".equals(ids.trim())) {
            result.error500("参数不识别！");
        } else {
            this.kuaiShouAppListService.removeByIds(Arrays.asList(ids.split(StringUtils.COMMA)));
            result.success("删除成功!");
        }
        return result;
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    @AutoLog(value = "快手-应用列表-通过id查询")
    @ApiOperation(value = "快手-应用列表-通过id查询", notes = "快手-应用列表-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<KuaiShouAppList> queryById(@RequestParam(name = "id", required = true) String id) {
        Result<KuaiShouAppList> result = new Result<>();
        KuaiShouAppList kuaiShouAppList = kuaiShouAppListService.getById(id);
        if (kuaiShouAppList == null) {
            result.error500("未找到对应实体");
        } else {
            result.setResult(kuaiShouAppList);
            result.setSuccess(true);
        }
        return result;
    }
}
