package cn.com.ctop.kuaishou.modules.batch.controller;

import cn.com.ctop.common.module.annotation.AutoLog;
import cn.com.ctop.common.module.utils.Check;
import cn.com.ctop.common.module.utils.QueryGenerator;
import cn.com.ctop.kuaishou.modules.batch.entity.KuaiShouBatchCampaignTemplate;
import cn.com.ctop.kuaishou.modules.batch.service.IKuaiShouBatchCampaignTemplateService;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.util.oConvertUtils;
import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

/**
 * 计划模板
 *
 * @author jeecg-boot
 * @version V1.0
 * @date 2021-01-19
 */
@Slf4j
@Api(tags = "计划模板")
@RestController
@RequestMapping("/template/kuaiShouBatchCampaignTemplate")
public class KuaiShouBatchCampaignTemplateController {
    @Autowired
    private IKuaiShouBatchCampaignTemplateService kuaiShouBatchCampaignTemplateService;

    @AutoLog(value = "计划模板-分页列表查询")
    @ApiOperation(value = "计划模板-分页列表查询", notes = "计划模板-分页列表查询")
    @GetMapping(value = "/getCampaignTemplate")
    public Result<KuaiShouBatchCampaignTemplate> queryPageList(Long accountId) {
        Result<KuaiShouBatchCampaignTemplate> result = new Result<>();
        QueryWrapper<KuaiShouBatchCampaignTemplate> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("account_id", accountId);
        queryWrapper.orderByDesc("create_time");
        queryWrapper.last("limit 1");
        KuaiShouBatchCampaignTemplate one = kuaiShouBatchCampaignTemplateService.getOne(queryWrapper);
        result.setSuccess(true);
        result.setResult(one);
        return result;
    }

    /**
     * 添加
     *
     * @param kuaiShouBatchCampaignTemplate
     * @return
     */
    @AutoLog(value = "计划模板-添加")
    @ApiOperation(value = "计划模板-添加", notes = "计划模板-添加")
    @PostMapping(value = "/add")
    public Result<KuaiShouBatchCampaignTemplate> add(@RequestBody KuaiShouBatchCampaignTemplate kuaiShouBatchCampaignTemplate) {
        Result<KuaiShouBatchCampaignTemplate> result = new Result<>();
        try {
            kuaiShouBatchCampaignTemplateService.save(kuaiShouBatchCampaignTemplate);
            result.success("添加成功！");
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            result.error500("操作失败");
        }
        return result;
    }

    /**
     * 编辑
     *
     * @param kuaiShouBatchCampaignTemplate
     * @return
     */
    @AutoLog(value = "计划模板-编辑")
    @ApiOperation(value = "计划模板-编辑", notes = "计划模板-编辑")
    @PostMapping(value = "/saveOrUpdate")
    public Result<Object> edit(@RequestBody KuaiShouBatchCampaignTemplate kuaiShouBatchCampaignTemplate) {
        boolean ok;
        try {
            Long id = kuaiShouBatchCampaignTemplate.getId();
            if (Check.isNull(id)) {
                ok = kuaiShouBatchCampaignTemplateService.save(kuaiShouBatchCampaignTemplate);
            } else {
                KuaiShouBatchCampaignTemplate kuaiShouBatchCampaignTemplateEntity = kuaiShouBatchCampaignTemplateService.getById(id);
                if (!Check.isNull(kuaiShouBatchCampaignTemplateEntity)) {
                    ok = kuaiShouBatchCampaignTemplateService.updateById(kuaiShouBatchCampaignTemplate);
                } else {
                    throw new Exception("未获取到账户id对应的详细信息");
                }
            }
            if (ok) {
                return Result.ok(kuaiShouBatchCampaignTemplate.getId());
            }
            return Result.error("操作失败");
        } catch (Exception e) {
            return Result.error(e.getMessage());
        }
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @AutoLog(value = "计划模板-通过id删除")
    @ApiOperation(value = "计划模板-通过id删除", notes = "计划模板-通过id删除")
    @DeleteMapping(value = "/delete")
    public Result<?> delete(@RequestParam(name = "id", required = true) String id) {
        try {
            kuaiShouBatchCampaignTemplateService.removeById(id);
        } catch (Exception e) {
            log.error("删除失败", e.getMessage());
            return Result.error("删除失败!");
        }
        return Result.ok("删除成功!");
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @AutoLog(value = "计划模板-批量删除")
    @ApiOperation(value = "计划模板-批量删除", notes = "计划模板-批量删除")
    @DeleteMapping(value = "/deleteBatch")
    public Result<KuaiShouBatchCampaignTemplate> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        Result<KuaiShouBatchCampaignTemplate> result = new Result<>();
        if (ids == null || "".equals(ids.trim())) {
            result.error500("参数不识别！");
        } else {
            this.kuaiShouBatchCampaignTemplateService.removeByIds(Arrays.asList(ids.split(",")));
            result.success("删除成功!");
        }
        return result;
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    @AutoLog(value = "计划模板-通过id查询")
    @ApiOperation(value = "计划模板-通过id查询", notes = "计划模板-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<KuaiShouBatchCampaignTemplate> queryById(@RequestParam(name = "id", required = true) String id) {
        Result<KuaiShouBatchCampaignTemplate> result = new Result<>();
        KuaiShouBatchCampaignTemplate kuaiShouBatchCampaignTemplate = kuaiShouBatchCampaignTemplateService.getById(id);
        if (kuaiShouBatchCampaignTemplate == null) {
            result.error500("未找到对应实体");
        } else {
            result.setResult(kuaiShouBatchCampaignTemplate);
            result.setSuccess(true);
        }
        return result;
    }

    /**
     * 导出excel
     *
     * @param request
     * @param response
     */
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(HttpServletRequest request, HttpServletResponse response) {
        // Step.1 组装查询条件
        QueryWrapper<KuaiShouBatchCampaignTemplate> queryWrapper = null;
        try {
            String paramsStr = request.getParameter("paramsStr");
            if (oConvertUtils.isNotEmpty(paramsStr)) {
                String deString = URLDecoder.decode(paramsStr, "UTF-8");
                KuaiShouBatchCampaignTemplate kuaiShouBatchCampaignTemplate = JSON.parseObject(deString, KuaiShouBatchCampaignTemplate.class);
                queryWrapper = QueryGenerator.initQueryWrapper(kuaiShouBatchCampaignTemplate, request.getParameterMap());
            }
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }

        //Step.2 AutoPoi 导出Excel
        ModelAndView mv = new ModelAndView(new JeecgEntityExcelView());
        List<KuaiShouBatchCampaignTemplate> pageList = kuaiShouBatchCampaignTemplateService.list(queryWrapper);
        //导出文件名称
        mv.addObject(NormalExcelConstants.FILE_NAME, "计划模板列表");
        mv.addObject(NormalExcelConstants.CLASS, KuaiShouBatchCampaignTemplate.class);
        mv.addObject(NormalExcelConstants.PARAMS, new ExportParams("计划模板列表数据", "导出人:Jeecg", "导出信息"));
        mv.addObject(NormalExcelConstants.DATA_LIST, pageList);
        return mv;
    }

    /**
     * 通过excel导入数据
     *
     * @param request
     * @param response
     * @return
     */
    @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
        MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
        Map<String, MultipartFile> fileMap = multipartRequest.getFileMap();
        for (Map.Entry<String, MultipartFile> entity : fileMap.entrySet()) {
            MultipartFile file = entity.getValue();
            ImportParams params = new ImportParams();
            params.setTitleRows(2);
            params.setHeadRows(1);
            params.setNeedSave(true);
            try {
                List<KuaiShouBatchCampaignTemplate> listKuaiShouBatchCampaignTemplates = ExcelImportUtil.importExcel(file.getInputStream(), KuaiShouBatchCampaignTemplate.class, params);
                kuaiShouBatchCampaignTemplateService.saveBatch(listKuaiShouBatchCampaignTemplates);
                return Result.ok("文件导入成功！数据行数:" + listKuaiShouBatchCampaignTemplates.size());
            } catch (Exception e) {
                log.error(e.getMessage(), e);
                return Result.error("文件导入失败:" + e.getMessage());
            } finally {
                try {
                    file.getInputStream().close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        return Result.ok("文件导入失败！");
    }

}
