package cn.com.ctop.kuaishou.modules.batch.controller;

import cn.com.ctop.common.module.annotation.AutoLog;
import cn.com.ctop.common.module.utils.QueryGenerator;
import cn.com.ctop.common.module.utils.StringUtils;
import cn.com.ctop.kuaishou.modules.batch.entity.KuaiShouConversionInfos;
import cn.com.ctop.kuaishou.modules.batch.service.IKuaiShouConversionInfosService;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.util.oConvertUtils;
import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

/**
 * @Description: 快手-可选深度转化类型
 * @Author: jeecg-boot
 * @Date: 2019-07-24
 * @Version: V1.0
 */
@Slf4j
@Api(tags = "快手-可选深度转化类型")
@RestController
@RequestMapping("/kuaishou/kuaiShouConversionInfos")
public class KuaiShouConversionInfosController {
    @Autowired
    private IKuaiShouConversionInfosService kuaiShouConversionInfosService;

    /**
     * 分页列表查询
     *
     * @param kuaiShouConversionInfos
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @AutoLog(value = "快手-可选深度转化类型-分页列表查询")
    @ApiOperation(value = "快手-可选深度转化类型-分页列表查询", notes = "快手-可选深度转化类型-分页列表查询")
    @GetMapping(value = "/list")
    public Result<IPage<KuaiShouConversionInfos>> queryPageList(KuaiShouConversionInfos kuaiShouConversionInfos,
                                                                @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                                @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                                HttpServletRequest req) {
        Result<IPage<KuaiShouConversionInfos>> result = new Result<IPage<KuaiShouConversionInfos>>();
        QueryWrapper<KuaiShouConversionInfos> queryWrapper = QueryGenerator.initQueryWrapper(kuaiShouConversionInfos, req.getParameterMap());
        Page<KuaiShouConversionInfos> page = new Page<KuaiShouConversionInfos>(pageNo, pageSize);
        IPage<KuaiShouConversionInfos> pageList = kuaiShouConversionInfosService.page(page, queryWrapper);
        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }

    /**
     * 添加
     *
     * @param kuaiShouConversionInfos
     * @return
     */
    @AutoLog(value = "快手-可选深度转化类型-添加")
    @ApiOperation(value = "快手-可选深度转化类型-添加", notes = "快手-可选深度转化类型-添加")
    @PostMapping(value = "/add")
    public Result<KuaiShouConversionInfos> add(@RequestBody KuaiShouConversionInfos kuaiShouConversionInfos) {
        Result<KuaiShouConversionInfos> result = new Result<KuaiShouConversionInfos>();
        try {
            kuaiShouConversionInfosService.save(kuaiShouConversionInfos);
            result.success("添加成功！");
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            result.error500("操作失败");
        }
        return result;
    }

    /**
     * 编辑
     *
     * @param kuaiShouConversionInfos
     * @return
     */
    @AutoLog(value = "快手-可选深度转化类型-编辑")
    @ApiOperation(value = "快手-可选深度转化类型-编辑", notes = "快手-可选深度转化类型-编辑")
    @PutMapping(value = "/edit")
    public Result<KuaiShouConversionInfos> edit(@RequestBody KuaiShouConversionInfos kuaiShouConversionInfos) {
        Result<KuaiShouConversionInfos> result = new Result<KuaiShouConversionInfos>();
        KuaiShouConversionInfos kuaiShouConversionInfosEntity = kuaiShouConversionInfosService.getById(kuaiShouConversionInfos.getId());
        if (kuaiShouConversionInfosEntity == null) {
            result.error500("未找到对应实体");
        } else {
            boolean ok = kuaiShouConversionInfosService.updateById(kuaiShouConversionInfos);
            //TODO 返回false说明什么？
            if (ok) {
                result.success("修改成功!");
            }
        }

        return result;
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @AutoLog(value = "快手-可选深度转化类型-通过id删除")
    @ApiOperation(value = "快手-可选深度转化类型-通过id删除", notes = "快手-可选深度转化类型-通过id删除")
    @DeleteMapping(value = "/delete")
    public Result<?> delete(@RequestParam(name = "id", required = true) String id) {
        try {
            kuaiShouConversionInfosService.removeById(id);
        } catch (Exception e) {
            log.error("删除失败", e.getMessage());
            return Result.error("删除失败!");
        }
        return Result.ok("删除成功!");
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @AutoLog(value = "快手-可选深度转化类型-批量删除")
    @ApiOperation(value = "快手-可选深度转化类型-批量删除", notes = "快手-可选深度转化类型-批量删除")
    @DeleteMapping(value = "/deleteBatch")
    public Result<KuaiShouConversionInfos> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        Result<KuaiShouConversionInfos> result = new Result<>();
        if (ids == null || "".equals(ids.trim())) {
            result.error500("参数不识别！");
        } else {
            this.kuaiShouConversionInfosService.removeByIds(Arrays.asList(ids.split(StringUtils.COMMA)));
            result.success("删除成功!");
        }
        return result;
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    @AutoLog(value = "快手-可选深度转化类型-通过id查询")
    @ApiOperation(value = "快手-可选深度转化类型-通过id查询", notes = "快手-可选深度转化类型-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<KuaiShouConversionInfos> queryById(@RequestParam(name = "id", required = true) String id) {
        Result<KuaiShouConversionInfos> result = new Result<KuaiShouConversionInfos>();
        KuaiShouConversionInfos kuaiShouConversionInfos = kuaiShouConversionInfosService.getById(id);
        if (kuaiShouConversionInfos == null) {
            result.error500("未找到对应实体");
        } else {
            result.setResult(kuaiShouConversionInfos);
            result.setSuccess(true);
        }
        return result;
    }

    /**
     * 导出excel
     *
     * @param request
     * @param response
     */
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(HttpServletRequest request, HttpServletResponse response) {
        // Step.1 组装查询条件
        QueryWrapper<KuaiShouConversionInfos> queryWrapper = null;
        try {
            String paramsStr = request.getParameter("paramsStr");
            if (oConvertUtils.isNotEmpty(paramsStr)) {
                String deString = URLDecoder.decode(paramsStr, "UTF-8");
                KuaiShouConversionInfos kuaiShouConversionInfos = JSON.parseObject(deString, KuaiShouConversionInfos.class);
                queryWrapper = QueryGenerator.initQueryWrapper(kuaiShouConversionInfos, request.getParameterMap());
            }
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }

        //Step.2 AutoPoi 导出Excel
        ModelAndView mv = new ModelAndView(new JeecgEntityExcelView());
        List<KuaiShouConversionInfos> pageList = kuaiShouConversionInfosService.list(queryWrapper);
        //导出文件名称
        mv.addObject(NormalExcelConstants.FILE_NAME, "快手-可选深度转化类型列表");
        mv.addObject(NormalExcelConstants.CLASS, KuaiShouConversionInfos.class);
        mv.addObject(NormalExcelConstants.PARAMS, new ExportParams("快手-可选深度转化类型列表数据", "导出人:Jeecg", "导出信息"));
        mv.addObject(NormalExcelConstants.DATA_LIST, pageList);
        return mv;
    }

    /**
     * 通过excel导入数据
     *
     * @param request
     * @param response
     * @return
     */
    @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
        MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
        Map<String, MultipartFile> fileMap = multipartRequest.getFileMap();
        for (Map.Entry<String, MultipartFile> entity : fileMap.entrySet()) {
            MultipartFile file = entity.getValue();
            ImportParams params = new ImportParams();
            params.setTitleRows(2);
            params.setHeadRows(1);
            params.setNeedSave(true);
            try {
                List<KuaiShouConversionInfos> listKuaiShouConversionInfoss = ExcelImportUtil.importExcel(file.getInputStream(), KuaiShouConversionInfos.class, params);
                kuaiShouConversionInfosService.saveBatch(listKuaiShouConversionInfoss);
                return Result.ok("文件导入成功！数据行数:" + listKuaiShouConversionInfoss.size());
            } catch (Exception e) {
                log.error(e.getMessage(), e);
                return Result.error("文件导入失败:" + e.getMessage());
            } finally {
                try {
                    file.getInputStream().close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        return Result.ok("文件导入失败！");
    }

}
