package cn.com.ctop.kuaishou.modules.batch.controller;

import cn.com.ctop.common.module.utils.QueryGenerator;
import cn.com.ctop.kuaishou.modules.batch.entity.KuaishouAccountBudget;
import cn.com.ctop.kuaishou.modules.batch.service.IKuaishouAccountBudgetService;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.Arrays;

/**
 * 账户日预算
 *
 * @author jeecg-boot
 * @version V1.0
 * @date 2020-04-26
 */
@Slf4j
@Api(tags = "账户日预算")
@RestController
@RequestMapping("/batch/kuaishouAccountBudget")
public class KuaishouAccountBudgetController {
    @Autowired
    private IKuaishouAccountBudgetService kuaishouAccountBudgetService;

    /**
     * 分页列表查询
     *
     * @param kuaishouAccountBudget
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @ApiOperation(value = "账户日预算-分页列表查询", notes = "账户日预算-分页列表查询")
    @GetMapping(value = "/list")
    public Result<IPage<KuaishouAccountBudget>> queryPageList(KuaishouAccountBudget kuaishouAccountBudget,
                                                              @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                              @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                              HttpServletRequest req) {
        Result<IPage<KuaishouAccountBudget>> result = new Result<>();
        QueryWrapper<KuaishouAccountBudget> queryWrapper = QueryGenerator.initQueryWrapper(kuaishouAccountBudget, req.getParameterMap());
        Page<KuaishouAccountBudget> page = new Page<>(pageNo, pageSize);
        IPage<KuaishouAccountBudget> pageList = kuaishouAccountBudgetService.page(page, queryWrapper);
        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }

    /**
     * 添加
     *
     * @param kuaishouAccountBudget
     * @return
     */
    @ApiOperation(value = "账户日预算-添加", notes = "账户日预算-添加")
    @PostMapping(value = "/add")
    public Result<KuaishouAccountBudget> add(@RequestBody KuaishouAccountBudget kuaishouAccountBudget) {
        Result<KuaishouAccountBudget> result = new Result<>();
        try {
            kuaishouAccountBudgetService.save(kuaishouAccountBudget);
            result.success("添加成功！");
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            result.error500("操作失败");
        }
        return result;
    }

    /**
     * 编辑
     *
     * @param kuaishouAccountBudget
     * @return
     */
    @PutMapping(value = "/edit")
    public Result<KuaishouAccountBudget> edit(@RequestBody KuaishouAccountBudget kuaishouAccountBudget) {
        Result<KuaishouAccountBudget> result = new Result<>();
        KuaishouAccountBudget kuaishouAccountBudgetEntity = kuaishouAccountBudgetService.getById(kuaishouAccountBudget.getId());
        if (kuaishouAccountBudgetEntity == null) {
            result.error500("未找到对应实体");
        } else {
            boolean ok = kuaishouAccountBudgetService.updateById(kuaishouAccountBudget);
            if (ok) {
                result.success("修改成功!");
            }
        }

        return result;
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @ApiOperation(value = "账户日预算-通过id删除", notes = "账户日预算-通过id删除")
    @DeleteMapping(value = "/delete")
    public Result<Object> delete(@RequestParam(name = "id", required = true) String id) {
        try {
            kuaishouAccountBudgetService.removeById(id);
        } catch (Exception e) {
            log.error("删除失败", e.getMessage());
            return Result.error("删除失败!");
        }
        return Result.ok("删除成功!");
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @ApiOperation(value = "账户日预算-批量删除", notes = "账户日预算-批量删除")
    @DeleteMapping(value = "/deleteBatch")
    public Result<KuaishouAccountBudget> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        Result<KuaishouAccountBudget> result = new Result<>();
        if (ids == null || "".equals(ids.trim())) {
            result.error500("参数不识别！");
        } else {
            this.kuaishouAccountBudgetService.removeByIds(Arrays.asList(ids.split(",")));
            result.success("删除成功!");
        }
        return result;
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    @ApiOperation(value = "账户日预算-通过id查询", notes = "账户日预算-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<KuaishouAccountBudget> queryById(@RequestParam(name = "id", required = true) String id) {
        Result<KuaishouAccountBudget> result = new Result<>();
        KuaishouAccountBudget kuaishouAccountBudget = kuaishouAccountBudgetService.getById(id);
        if (kuaishouAccountBudget == null) {
            result.error500("未找到对应实体");
        } else {
            result.setResult(kuaishouAccountBudget);
            result.setSuccess(true);
        }
        return result;
    }
}
