package cn.com.ctop.kuaishou.modules.batch.controller;

import cn.com.ctop.common.module.annotation.AutoLog;
import cn.com.ctop.common.module.utils.Check;
import cn.com.ctop.common.module.utils.QueryGenerator;
import cn.com.ctop.kuaishou.modules.batch.entity.KuaishouCampaignCopyData;
import cn.com.ctop.kuaishou.modules.batch.service.IKuaishouCampaignCopyDataService;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.shiro.SecurityUtils;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.system.vo.LoginUser;
import org.jeecg.common.util.oConvertUtils;
import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

/**
 * 待复制广告计划数据
 *
 * @author jeecg-boot
 * @version V1.0
 * @date 2020-12-22
 */
@Slf4j
@Api(tags = "待复制广告计划数据")
@RestController
@RequestMapping("/kuaishouCampaignCopyData")
public class KuaishouCampaignCopyDataController {
    @Autowired
    private IKuaishouCampaignCopyDataService kuaishouCampaignCopyDataService;

    /**
     * 分页列表查询
     *
     * @param kuaishouCampaignCopyData
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @AutoLog(value = "待复制广告计划数据-分页列表查询")
    @ApiOperation(value = "待复制广告计划数据-分页列表查询", notes = "待复制广告计划数据-分页列表查询")
    @GetMapping(value = "/list")
    public Result<IPage<KuaishouCampaignCopyData>> queryPageList(KuaishouCampaignCopyData kuaishouCampaignCopyData,
                                                                 @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                                 @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                                 HttpServletRequest req) {
        Result<IPage<KuaishouCampaignCopyData>> result = new Result<>();
        QueryWrapper<KuaishouCampaignCopyData> queryWrapper = QueryGenerator.initQueryWrapper(kuaishouCampaignCopyData, req.getParameterMap());
        Page<KuaishouCampaignCopyData> page = new Page<KuaishouCampaignCopyData>(pageNo, pageSize);
        IPage<KuaishouCampaignCopyData> pageList = kuaishouCampaignCopyDataService.page(page, queryWrapper);
        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }

    /**
     * 添加
     *
     * @param
     * @return
     */
    static ExecutorService executorService = Executors.newFixedThreadPool(5);

    @AutoLog(value = "待复制广告计划数据-添加")
    @ApiOperation(value = "待复制广告计划数据-添加", notes = "待复制广告计划数据-添加")
    @PostMapping(value = "/add")
    public Result<KuaishouCampaignCopyData> add(@RequestBody JSONObject data) {
        Result<KuaishouCampaignCopyData> result = new Result<>();
        try {
            KuaishouCampaignCopyData kuaishouCampaignCopyData = JSONObject.toJavaObject(data, KuaishouCampaignCopyData.class);
            if (Check.isNull(kuaishouCampaignCopyData.getCreaterId())) {
                LoginUser sysUser = (LoginUser) SecurityUtils.getSubject().getPrincipal();
                if (!Check.isNull(sysUser)) {
                    kuaishouCampaignCopyData.setCreaterId(sysUser.getId());
                }
            }
            boolean save = kuaishouCampaignCopyDataService.save(kuaishouCampaignCopyData);
            if (save) {
                executorService.submit(new Runnable() {
                    @Override
                    public void run() {
                        log.info("批量复制计划开始,accountId:{},老计划id：{},新计划id：{}", kuaishouCampaignCopyData.getAccountId(), kuaishouCampaignCopyData.getOldCampaignId(), kuaishouCampaignCopyData.getNewCampaignId());
                        kuaishouCampaignCopyDataService.batchCopyCampaign(kuaishouCampaignCopyData);
                    }
                });
            }
            result.success("提交成功,异步创建中");
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            result.error500("操作失败");
        }
        return result;
    }

    /**
     * 编辑
     *
     * @param kuaishouCampaignCopyData
     * @return
     */
    @AutoLog(value = "待复制广告计划数据-编辑")
    @ApiOperation(value = "待复制广告计划数据-编辑", notes = "待复制广告计划数据-编辑")
    @PutMapping(value = "/edit")
    public Result<KuaishouCampaignCopyData> edit(@RequestBody KuaishouCampaignCopyData kuaishouCampaignCopyData) {
        Result<KuaishouCampaignCopyData> result = new Result<KuaishouCampaignCopyData>();
        KuaishouCampaignCopyData kuaishouCampaignCopyDataEntity = kuaishouCampaignCopyDataService.getById(kuaishouCampaignCopyData.getId());
        if (kuaishouCampaignCopyDataEntity == null) {
            result.error500("未找到对应实体");
        } else {
            boolean ok = kuaishouCampaignCopyDataService.updateById(kuaishouCampaignCopyData);
            if (ok) {
                result.success("修改成功!");
            }
        }

        return result;
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @AutoLog(value = "待复制广告计划数据-通过id删除")
    @ApiOperation(value = "待复制广告计划数据-通过id删除", notes = "待复制广告计划数据-通过id删除")
    @DeleteMapping(value = "/delete")
    public Result<?> delete(@RequestParam(name = "id", required = true) String id) {
        try {
            kuaishouCampaignCopyDataService.removeById(id);
        } catch (Exception e) {
            log.error("删除失败", e.getMessage());
            return Result.error("删除失败!");
        }
        return Result.ok("删除成功!");
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @AutoLog(value = "待复制广告计划数据-批量删除")
    @ApiOperation(value = "待复制广告计划数据-批量删除", notes = "待复制广告计划数据-批量删除")
    @DeleteMapping(value = "/deleteBatch")
    public Result<KuaishouCampaignCopyData> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        Result<KuaishouCampaignCopyData> result = new Result<>();
        if (ids == null || "".equals(ids.trim())) {
            result.error500("参数不识别！");
        } else {
            this.kuaishouCampaignCopyDataService.removeByIds(Arrays.asList(ids.split(",")));
            result.success("删除成功!");
        }
        return result;
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    @AutoLog(value = "待复制广告计划数据-通过id查询")
    @ApiOperation(value = "待复制广告计划数据-通过id查询", notes = "待复制广告计划数据-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<KuaishouCampaignCopyData> queryById(@RequestParam(name = "id", required = true) String id) {
        Result<KuaishouCampaignCopyData> result = new Result<>();
        KuaishouCampaignCopyData kuaishouCampaignCopyData = kuaishouCampaignCopyDataService.getById(id);
        if (kuaishouCampaignCopyData == null) {
            result.error500("未找到对应实体");
        } else {
            result.setResult(kuaishouCampaignCopyData);
            result.setSuccess(true);
        }
        return result;
    }

    /**
     * 导出excel
     *
     * @param request
     * @param response
     */
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(HttpServletRequest request, HttpServletResponse response) {
        // Step.1 组装查询条件
        QueryWrapper<KuaishouCampaignCopyData> queryWrapper = null;
        try {
            String paramsStr = request.getParameter("paramsStr");
            if (oConvertUtils.isNotEmpty(paramsStr)) {
                String deString = URLDecoder.decode(paramsStr, "UTF-8");
                KuaishouCampaignCopyData kuaishouCampaignCopyData = JSON.parseObject(deString, KuaishouCampaignCopyData.class);
                queryWrapper = QueryGenerator.initQueryWrapper(kuaishouCampaignCopyData, request.getParameterMap());
            }
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }

        //Step.2 AutoPoi 导出Excel
        ModelAndView mv = new ModelAndView(new JeecgEntityExcelView());
        List<KuaishouCampaignCopyData> pageList = kuaishouCampaignCopyDataService.list(queryWrapper);
        //导出文件名称
        mv.addObject(NormalExcelConstants.FILE_NAME, "待复制广告计划数据列表");
        mv.addObject(NormalExcelConstants.CLASS, KuaishouCampaignCopyData.class);
        mv.addObject(NormalExcelConstants.PARAMS, new ExportParams("待复制广告计划数据列表数据", "导出人:Jeecg", "导出信息"));
        mv.addObject(NormalExcelConstants.DATA_LIST, pageList);
        return mv;
    }

    /**
     * 通过excel导入数据
     *
     * @param request
     * @param response
     * @return
     */
    @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
        MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
        Map<String, MultipartFile> fileMap = multipartRequest.getFileMap();
        for (Map.Entry<String, MultipartFile> entity : fileMap.entrySet()) {
            MultipartFile file = entity.getValue();
            ImportParams params = new ImportParams();
            params.setTitleRows(2);
            params.setHeadRows(1);
            params.setNeedSave(true);
            try {
                List<KuaishouCampaignCopyData> listKuaishouCampaignCopyDatas = ExcelImportUtil.importExcel(file.getInputStream(), KuaishouCampaignCopyData.class, params);
                kuaishouCampaignCopyDataService.saveBatch(listKuaishouCampaignCopyDatas);
                return Result.ok("文件导入成功！数据行数:" + listKuaishouCampaignCopyDatas.size());
            } catch (Exception e) {
                log.error(e.getMessage(), e);
                return Result.error("文件导入失败:" + e.getMessage());
            } finally {
                try {
                    file.getInputStream().close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        return Result.ok("文件导入失败！");
    }

}
