package cn.com.ctop.kuaishou.modules.batch.controller;

import cn.com.ctop.common.module.annotation.AutoLog;
import cn.com.ctop.common.module.entity.CtopOauthToken;
import cn.com.ctop.common.module.service.ICtopOauthTokenService;
import cn.com.ctop.common.module.utils.Check;
import cn.com.ctop.common.module.utils.QueryGenerator;
import cn.com.ctop.kuaishou.modules.batch.entity.KuaishouTemplate;
import cn.com.ctop.kuaishou.modules.batch.service.IKuaishouTemplateService;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.util.oConvertUtils;
import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

/**
 * 快手-定向模板
 *
 * @author jeecg-boot
 * @version V1.0
 * @date 2020-11-09
 */
@Slf4j
@Api(tags = "快手-定向模板")
@RestController
@RequestMapping("/batch/kuaishouTemplate")
public class KuaishouTemplateController {
    @Autowired
    private IKuaishouTemplateService kuaishouTemplateService;

    @Autowired
    private ICtopOauthTokenService oauthTokenService;


    @AutoLog(value = "快手-定向模板-分页列表查询")
    @ApiOperation(value = "快手-定向模板-分页列表查询", notes = "快手-定向模板-分页列表查询")
    @GetMapping(value = "/list")
    public Result<List<JSONObject>> list(Long accountId, Integer platformOs) {
        Result<List<JSONObject>> result = new Result<>();
        List<JSONObject> list = kuaishouTemplateService.getJsonArrByAccountId(accountId, platformOs);
        result.setSuccess(true);
        result.setResult(list);
        return result;
    }


    /**
     * 分页列表查询
     *
     * @param kuaishouTemplate
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
  /*  @AutoLog(value = "快手-定向模板-分页列表查询")
    @ApiOperation(value = "快手-定向模板-分页列表查询", notes = "快手-定向模板-分页列表查询")
    @GetMapping(value = "/list")
    public Result<IPage<KuaishouTemplate>> queryPageList(KuaishouTemplate kuaishouTemplate,
                                                         @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                         @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                         HttpServletRequest req) {
        Result<IPage<KuaishouTemplate>> result = new Result<>();
        QueryWrapper<KuaishouTemplate> queryWrapper = QueryGenerator.initQueryWrapper(kuaishouTemplate, req.getParameterMap());
        Page<KuaishouTemplate> page = new Page<KuaishouTemplate>(pageNo, pageSize);
        IPage<KuaishouTemplate> pageList = kuaishouTemplateService.page(page, queryWrapper);
        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }*/


    /**
     * 根据账户获取定向模板列表
     *
     * @param accountId
     * @return
     */
    @GetMapping(value = "/getTemplateByAccountId")
    public Result<Boolean> queryPageList(Long accountId) {

        Result<Boolean> result = new Result<>();
        try {
            if (Check.isNull(accountId)) {
                throw new Exception("请选择账户id");
            }

            CtopOauthToken oauthToken = oauthTokenService.getTokenByAccountId(accountId);
            if (Check.isNull(oauthToken)) {
                throw new Exception("未获取到账户信息");
            }

            kuaishouTemplateService.getTemplateByAccountId(accountId, oauthToken.getAccessToken(), 1);
            result.setSuccess(true);
        } catch (Exception e) {
            e.printStackTrace();
            result.setSuccess(false);
            result.setMessage(e.getMessage());
        }
        return result;
    }


    /**
     * 创建定向模板
     *
     * @param
     * @return
     */
    @PostMapping(value = "/createTemplate")
    public Result<JSONObject> createTemplate(@RequestBody JSONObject requestJson) {
        Result<JSONObject> result = new Result<>();
        try {

            Long accountId = requestJson.getLong("accountId");
            if (Check.isNull(accountId)) {
                throw new Exception("请选择账户id");
            }
            CtopOauthToken oauthToken = oauthTokenService.getTokenByAccountId(accountId);
            if (Check.isNull(oauthToken)) {
                throw new Exception("未获取到账户信息");
            }
            JSONObject returnJson = kuaishouTemplateService.createTemplate(accountId, oauthToken.getAccessToken(), requestJson);
            result.setSuccess(true);
            result.setResult(returnJson);
        } catch (Exception e) {
            e.printStackTrace();
            result.setSuccess(false);
            result.setMessage(e.getMessage());
        }
        return result;
    }


    @PostMapping(value = "/updateTemplate")
    public Result<JSONObject> updateTemplate(@RequestBody JSONObject requestJson) {
        Result<JSONObject> result = new Result<>();
        try {

            Long accountId = requestJson.getLong("accountId");
            if (Check.isNull(accountId)) {
                throw new Exception("请选择账户id");
            }
            CtopOauthToken oauthToken = oauthTokenService.getTokenByAccountId(accountId);
            if (Check.isNull(oauthToken)) {
                throw new Exception("未获取到账户信息");
            }
            JSONObject returnJson = kuaishouTemplateService.updateTemplate(accountId, oauthToken.getAccessToken(), requestJson);
            Integer code = returnJson.getInteger("code");
            if (code == 0) {
                result.setSuccess(true);
                result.setMessage("修改成功");
            } else {
                result.setSuccess(false);
                result.setMessage(returnJson.getString("message"));
            }


            // result.setResult(returnJson);
        } catch (Exception e) {
            e.printStackTrace();
            result.setSuccess(false);
            result.setMessage(e.getMessage());
        }
        return result;
    }


    /**
     * 添加
     *
     * @param kuaishouTemplate
     * @return
     */
    @AutoLog(value = "快手-定向模板-添加")
    @ApiOperation(value = "快手-定向模板-添加", notes = "快手-定向模板-添加")
    @PostMapping(value = "/add")
    public Result<KuaishouTemplate> add(@RequestBody KuaishouTemplate kuaishouTemplate) {
        Result<KuaishouTemplate> result = new Result<>();
        try {
            kuaishouTemplateService.save(kuaishouTemplate);
            result.success("添加成功！");
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            result.error500("操作失败");
        }
        return result;
    }

    /**
     * 编辑
     *
     * @param kuaishouTemplate
     * @return
     */
    @AutoLog(value = "快手-定向模板-编辑")
    @ApiOperation(value = "快手-定向模板-编辑", notes = "快手-定向模板-编辑")
    @PutMapping(value = "/edit")
    public Result<KuaishouTemplate> edit(@RequestBody KuaishouTemplate kuaishouTemplate) {
        Result<KuaishouTemplate> result = new Result<KuaishouTemplate>();
        KuaishouTemplate kuaishouTemplateEntity = kuaishouTemplateService.getById(kuaishouTemplate.getId());
        if (kuaishouTemplateEntity == null) {
            result.error500("未找到对应实体");
        } else {
            boolean ok = kuaishouTemplateService.updateById(kuaishouTemplate);
            if (ok) {
                result.success("修改成功!");
            }
        }

        return result;
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @AutoLog(value = "快手-定向模板-通过id删除")
    @ApiOperation(value = "快手-定向模板-通过id删除", notes = "快手-定向模板-通过id删除")
    @DeleteMapping(value = "/delete")
    public Result<?> delete(@RequestParam(name = "id", required = true) String id) {
        try {
            kuaishouTemplateService.removeById(id);
        } catch (Exception e) {
            log.error("删除失败", e.getMessage());
            return Result.error("删除失败!");
        }
        return Result.ok("删除成功!");
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @AutoLog(value = "快手-定向模板-批量删除")
    @ApiOperation(value = "快手-定向模板-批量删除", notes = "快手-定向模板-批量删除")
    @DeleteMapping(value = "/deleteBatch")
    public Result<KuaishouTemplate> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        Result<KuaishouTemplate> result = new Result<>();
        if (ids == null || "".equals(ids.trim())) {
            result.error500("参数不识别！");
        } else {
            this.kuaishouTemplateService.removeByIds(Arrays.asList(ids.split(",")));
            result.success("删除成功!");
        }
        return result;
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    @AutoLog(value = "快手-定向模板-通过id查询")
    @ApiOperation(value = "快手-定向模板-通过id查询", notes = "快手-定向模板-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<KuaishouTemplate> queryById(@RequestParam(name = "id", required = true) String id) {
        Result<KuaishouTemplate> result = new Result<>();
        KuaishouTemplate kuaishouTemplate = kuaishouTemplateService.getById(id);
        if (kuaishouTemplate == null) {
            result.error500("未找到对应实体");
        } else {
            result.setResult(kuaishouTemplate);
            result.setSuccess(true);
        }
        return result;
    }

    /**
     * 导出excel
     *
     * @param request
     * @param response
     */
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(HttpServletRequest request, HttpServletResponse response) {
        // Step.1 组装查询条件
        QueryWrapper<KuaishouTemplate> queryWrapper = null;
        try {
            String paramsStr = request.getParameter("paramsStr");
            if (oConvertUtils.isNotEmpty(paramsStr)) {
                String deString = URLDecoder.decode(paramsStr, "UTF-8");
                KuaishouTemplate kuaishouTemplate = JSON.parseObject(deString, KuaishouTemplate.class);
                queryWrapper = QueryGenerator.initQueryWrapper(kuaishouTemplate, request.getParameterMap());
            }
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }

        //Step.2 AutoPoi 导出Excel
        ModelAndView mv = new ModelAndView(new JeecgEntityExcelView());
        List<KuaishouTemplate> pageList = kuaishouTemplateService.list(queryWrapper);
        //导出文件名称
        mv.addObject(NormalExcelConstants.FILE_NAME, "快手-定向模板列表");
        mv.addObject(NormalExcelConstants.CLASS, KuaishouTemplate.class);
        mv.addObject(NormalExcelConstants.PARAMS, new ExportParams("快手-定向模板列表数据", "导出人:Jeecg", "导出信息"));
        mv.addObject(NormalExcelConstants.DATA_LIST, pageList);
        return mv;
    }

    /**
     * 通过excel导入数据
     *
     * @param request
     * @param response
     * @return
     */
    @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
        MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
        Map<String, MultipartFile> fileMap = multipartRequest.getFileMap();
        for (Map.Entry<String, MultipartFile> entity : fileMap.entrySet()) {
            MultipartFile file = entity.getValue();
            ImportParams params = new ImportParams();
            params.setTitleRows(2);
            params.setHeadRows(1);
            params.setNeedSave(true);
            try {
                List<KuaishouTemplate> listKuaishouTemplates = ExcelImportUtil.importExcel(file.getInputStream(), KuaishouTemplate.class, params);
                kuaishouTemplateService.saveBatch(listKuaishouTemplates);
                return Result.ok("文件导入成功！数据行数:" + listKuaishouTemplates.size());
            } catch (Exception e) {
                log.error(e.getMessage(), e);
                return Result.error("文件导入失败:" + e.getMessage());
            } finally {
                try {
                    file.getInputStream().close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        return Result.ok("文件导入失败！");
    }

}
