package cn.com.ctop.kuaishou.modules.report.controller;

import cn.com.ctop.common.module.annotation.AutoLog;
import cn.com.ctop.common.module.utils.QueryGenerator;
import cn.com.ctop.kuaishou.modules.report.entity.EtlKuaiShouAccountMaterialOverview;
import cn.com.ctop.kuaishou.modules.report.service.IEtlKuaiShouAccountMaterialOverviewService;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.util.oConvertUtils;
import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

/**
 * 清洗数据账户素材概览数据
 *
 * @author jeecg-boot
 * @version V1.0
 * @date 2021-03-10
 */
@Slf4j
@Api(tags = "清洗数据账户素材概览数据")
@RestController
@RequestMapping("/material/etlKuaiShouAccountMaterialOverview")
public class EtlKuaiShouAccountMaterialOverviewController {
    @Autowired
    private IEtlKuaiShouAccountMaterialOverviewService etlKuaiShouAccountMaterialOverviewService;


    @GetMapping(value = "/getMaterialOverview")
    public Result<JSONObject> queryPageList(Long accountId, String statDate) {
        Result<JSONObject> result = new Result<>();
        JSONObject returnJson = etlKuaiShouAccountMaterialOverviewService.getMaterialOverview(accountId, statDate);
        result.setSuccess(true);
        result.setResult(returnJson);
        return result;
    }

    /**
     * 分页列表查询
     *
     * @param etlKuaiShouAccountMaterialOverview
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @AutoLog(value = "清洗数据账户素材概览数据-分页列表查询")
    @ApiOperation(value = "清洗数据账户素材概览数据-分页列表查询", notes = "清洗数据账户素材概览数据-分页列表查询")
    @GetMapping(value = "/list")
    public Result<IPage<EtlKuaiShouAccountMaterialOverview>> queryPageList(EtlKuaiShouAccountMaterialOverview etlKuaiShouAccountMaterialOverview,
                                                                           @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                                           @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                                           HttpServletRequest req) {
        Result<IPage<EtlKuaiShouAccountMaterialOverview>> result = new Result<>();
        QueryWrapper<EtlKuaiShouAccountMaterialOverview> queryWrapper = QueryGenerator.initQueryWrapper(etlKuaiShouAccountMaterialOverview, req.getParameterMap());
        Page<EtlKuaiShouAccountMaterialOverview> page = new Page<EtlKuaiShouAccountMaterialOverview>(pageNo, pageSize);
        IPage<EtlKuaiShouAccountMaterialOverview> pageList = etlKuaiShouAccountMaterialOverviewService.page(page, queryWrapper);
        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }

    /**
     * 添加
     *
     * @param etlKuaiShouAccountMaterialOverview
     * @return
     */
    @AutoLog(value = "清洗数据账户素材概览数据-添加")
    @ApiOperation(value = "清洗数据账户素材概览数据-添加", notes = "清洗数据账户素材概览数据-添加")
    @PostMapping(value = "/add")
    public Result<EtlKuaiShouAccountMaterialOverview> add(@RequestBody EtlKuaiShouAccountMaterialOverview etlKuaiShouAccountMaterialOverview) {
        Result<EtlKuaiShouAccountMaterialOverview> result = new Result<>();
        try {
            etlKuaiShouAccountMaterialOverviewService.save(etlKuaiShouAccountMaterialOverview);
            result.success("添加成功！");
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            result.error500("操作失败");
        }
        return result;
    }

    /**
     * 编辑
     *
     * @param etlKuaiShouAccountMaterialOverview
     * @return
     */
    @AutoLog(value = "清洗数据账户素材概览数据-编辑")
    @ApiOperation(value = "清洗数据账户素材概览数据-编辑", notes = "清洗数据账户素材概览数据-编辑")
    @PutMapping(value = "/edit")
    public Result<EtlKuaiShouAccountMaterialOverview> edit(@RequestBody EtlKuaiShouAccountMaterialOverview etlKuaiShouAccountMaterialOverview) {
        Result<EtlKuaiShouAccountMaterialOverview> result = new Result<EtlKuaiShouAccountMaterialOverview>();
        EtlKuaiShouAccountMaterialOverview etlKuaiShouAccountMaterialOverviewEntity = etlKuaiShouAccountMaterialOverviewService.getById(etlKuaiShouAccountMaterialOverview.getId());
        if (etlKuaiShouAccountMaterialOverviewEntity == null) {
            result.error500("未找到对应实体");
        } else {
            boolean ok = etlKuaiShouAccountMaterialOverviewService.updateById(etlKuaiShouAccountMaterialOverview);
            if (ok) {
                result.success("修改成功!");
            }
        }

        return result;
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @AutoLog(value = "清洗数据账户素材概览数据-通过id删除")
    @ApiOperation(value = "清洗数据账户素材概览数据-通过id删除", notes = "清洗数据账户素材概览数据-通过id删除")
    @DeleteMapping(value = "/delete")
    public Result<?> delete(@RequestParam(name = "id", required = true) String id) {
        try {
            etlKuaiShouAccountMaterialOverviewService.removeById(id);
        } catch (Exception e) {
            log.error("删除失败", e.getMessage());
            return Result.error("删除失败!");
        }
        return Result.ok("删除成功!");
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @AutoLog(value = "清洗数据账户素材概览数据-批量删除")
    @ApiOperation(value = "清洗数据账户素材概览数据-批量删除", notes = "清洗数据账户素材概览数据-批量删除")
    @DeleteMapping(value = "/deleteBatch")
    public Result<EtlKuaiShouAccountMaterialOverview> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        Result<EtlKuaiShouAccountMaterialOverview> result = new Result<>();
        if (ids == null || "".equals(ids.trim())) {
            result.error500("参数不识别！");
        } else {
            this.etlKuaiShouAccountMaterialOverviewService.removeByIds(Arrays.asList(ids.split(",")));
            result.success("删除成功!");
        }
        return result;
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    @AutoLog(value = "清洗数据账户素材概览数据-通过id查询")
    @ApiOperation(value = "清洗数据账户素材概览数据-通过id查询", notes = "清洗数据账户素材概览数据-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<EtlKuaiShouAccountMaterialOverview> queryById(@RequestParam(name = "id", required = true) String id) {
        Result<EtlKuaiShouAccountMaterialOverview> result = new Result<>();
        EtlKuaiShouAccountMaterialOverview etlKuaiShouAccountMaterialOverview = etlKuaiShouAccountMaterialOverviewService.getById(id);
        if (etlKuaiShouAccountMaterialOverview == null) {
            result.error500("未找到对应实体");
        } else {
            result.setResult(etlKuaiShouAccountMaterialOverview);
            result.setSuccess(true);
        }
        return result;
    }

    /**
     * 导出excel
     *
     * @param request
     * @param response
     */
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(HttpServletRequest request, HttpServletResponse response) {
        // Step.1 组装查询条件
        QueryWrapper<EtlKuaiShouAccountMaterialOverview> queryWrapper = null;
        try {
            String paramsStr = request.getParameter("paramsStr");
            if (oConvertUtils.isNotEmpty(paramsStr)) {
                String deString = URLDecoder.decode(paramsStr, "UTF-8");
                EtlKuaiShouAccountMaterialOverview etlKuaiShouAccountMaterialOverview = JSON.parseObject(deString, EtlKuaiShouAccountMaterialOverview.class);
                queryWrapper = QueryGenerator.initQueryWrapper(etlKuaiShouAccountMaterialOverview, request.getParameterMap());
            }
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }

        //Step.2 AutoPoi 导出Excel
        ModelAndView mv = new ModelAndView(new JeecgEntityExcelView());
        List<EtlKuaiShouAccountMaterialOverview> pageList = etlKuaiShouAccountMaterialOverviewService.list(queryWrapper);
        //导出文件名称
        mv.addObject(NormalExcelConstants.FILE_NAME, "清洗数据账户素材概览数据列表");
        mv.addObject(NormalExcelConstants.CLASS, EtlKuaiShouAccountMaterialOverview.class);
        mv.addObject(NormalExcelConstants.PARAMS, new ExportParams("清洗数据账户素材概览数据列表数据", "导出人:Jeecg", "导出信息"));
        mv.addObject(NormalExcelConstants.DATA_LIST, pageList);
        return mv;
    }

    /**
     * 通过excel导入数据
     *
     * @param request
     * @param response
     * @return
     */
    @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
        MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
        Map<String, MultipartFile> fileMap = multipartRequest.getFileMap();
        for (Map.Entry<String, MultipartFile> entity : fileMap.entrySet()) {
            MultipartFile file = entity.getValue();
            ImportParams params = new ImportParams();
            params.setTitleRows(2);
            params.setHeadRows(1);
            params.setNeedSave(true);
            try {
                List<EtlKuaiShouAccountMaterialOverview> listEtlKuaiShouAccountMaterialOverviews = ExcelImportUtil.importExcel(file.getInputStream(), EtlKuaiShouAccountMaterialOverview.class, params);
                etlKuaiShouAccountMaterialOverviewService.saveBatch(listEtlKuaiShouAccountMaterialOverviews);
                return Result.ok("文件导入成功！数据行数:" + listEtlKuaiShouAccountMaterialOverviews.size());
            } catch (Exception e) {
                log.error(e.getMessage(), e);
                return Result.error("文件导入失败:" + e.getMessage());
            } finally {
                try {
                    file.getInputStream().close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        return Result.ok("文件导入失败！");
    }

}
