package cn.com.ctop.kuaishou.modules.report.controller;

import cn.com.ctop.common.module.annotation.AutoLog;
import cn.com.ctop.common.module.utils.QueryGenerator;
import cn.com.ctop.kuaishou.modules.report.entity.EtlKuaishouReportAccountHourly;
import cn.com.ctop.kuaishou.modules.report.service.IEtlKuaishouReportAccountHourlyService;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.util.oConvertUtils;
import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

/**
 * 账户维度-每小时盯盘数据
 *
 * @author jeecg-boot
 * @version V1.0
 * @date 2021-03-04
 */
@Slf4j
@Api(tags = "账户维度-每小时盯盘数据")
@RestController
@RequestMapping("/etl/reportAccountHourly")
public class EtlKuaishouReportAccountHourlyController {
    @Autowired
    private IEtlKuaishouReportAccountHourlyService etlKuaishouReportAccountHourlyService;

    /**
     * 分页列表查询
     *
     * @param etlKuaishouReportAccountHourly
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @AutoLog(value = "账户维度-每小时盯盘数据-分页列表查询")
    @ApiOperation(value = "账户维度-每小时盯盘数据-分页列表查询", notes = "账户维度-每小时盯盘数据-分页列表查询")
    @GetMapping(value = "/list")
    public Result<IPage<EtlKuaishouReportAccountHourly>> queryPageList(EtlKuaishouReportAccountHourly etlKuaishouReportAccountHourly,
                                                                       @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                                       @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                                       HttpServletRequest req) {
        Result<IPage<EtlKuaishouReportAccountHourly>> result = new Result<>();
        QueryWrapper<EtlKuaishouReportAccountHourly> queryWrapper = QueryGenerator.initQueryWrapper(etlKuaishouReportAccountHourly, req.getParameterMap());
        Page<EtlKuaishouReportAccountHourly> page = new Page<EtlKuaishouReportAccountHourly>(pageNo, pageSize);
        IPage<EtlKuaishouReportAccountHourly> pageList = etlKuaishouReportAccountHourlyService.page(page, queryWrapper);
        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }

    @GetMapping(value = "/createDate")
    public Result<Object> createDate() {
        try {
            etlKuaishouReportAccountHourlyService.queryEveryhourlyDataForAccount();
            return Result.ok("success");
        } catch (Exception e) {
            log.error("error:", e.getMessage());
            e.printStackTrace();
        }
        return Result.error("error");
    }


    /**
     * 获取时间趋势汇总投放数据
     *
     * @param
     * @return
     * @throws
     * @author ZHAOXA
     */
    @AutoLog(value = "获取当日花费最大的账户")
    @ApiOperation(value = "获取当日花费最大的账户", notes = "获取当日花费最大的账户")
    @GetMapping(value = "/getMaxChargeAccountId")
    public Result<Object> getMaxChargeAccountId(Long accountId) {
        try {
            return etlKuaishouReportAccountHourlyService.getMaxChargeAccountId(accountId);
        } catch (Exception e) {
            log.error("查询异常,", e.getMessage());
            e.printStackTrace();
        }
        return Result.error("查询失败");
    }
    /**
     * 获取时间趋势汇总投放数据
     *
     * @param
     * @return
     * @throws
     * @author ZHAOXA
     */
    @AutoLog(value = "获取时间趋势汇总投放数据")
    @ApiOperation(value = "获取时间趋势汇总投放数据", notes = "获取时间趋势汇总投放数据")
    @PostMapping(value = "/getAggregateByHour")
    public Result<Object> getAggregateByHour(@RequestBody JSONObject request) {
        try {
            return etlKuaishouReportAccountHourlyService.getAggregateByHour(request);
        } catch (Exception e) {
            log.error("查询异常,", e.getMessage());
            e.printStackTrace();
        }
        return Result.error("查询失败");
    }

    /**
     * 获取时间趋势数据集
     *
     * @param
     * @return
     * @throws
     * @author ZHAOXA
     */
    @AutoLog(value = "获取时间趋势数据集")
    @ApiOperation(value = "获取时间趋势数据集", notes = "获取时间趋势数据集")
    @PostMapping(value = "/getEtlReportList")
    public Result<Object> getEtlReportList(@RequestBody JSONObject request) {
        try {
            return etlKuaishouReportAccountHourlyService.getEtlReportList(request);
        } catch (Exception e) {
            log.error("查询异常,", e.getMessage());
            e.printStackTrace();
        }
        return Result.error("查询失败");
    }

    /**
     * 添加
     *
     * @param etlKuaishouReportAccountHourly
     * @return
     */
    @AutoLog(value = "账户维度-每小时盯盘数据-添加")
    @ApiOperation(value = "账户维度-每小时盯盘数据-添加", notes = "账户维度-每小时盯盘数据-添加")
    @PostMapping(value = "/add")
    public Result<EtlKuaishouReportAccountHourly> add(@RequestBody EtlKuaishouReportAccountHourly etlKuaishouReportAccountHourly) {
        Result<EtlKuaishouReportAccountHourly> result = new Result<>();
        try {
            etlKuaishouReportAccountHourlyService.save(etlKuaishouReportAccountHourly);
            result.success("添加成功！");
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            result.error500("操作失败");
        }
        return result;
    }

    /**
     * 编辑
     *
     * @param etlKuaishouReportAccountHourly
     * @return
     */
    @AutoLog(value = "账户维度-每小时盯盘数据-编辑")
    @ApiOperation(value = "账户维度-每小时盯盘数据-编辑", notes = "账户维度-每小时盯盘数据-编辑")
    @PutMapping(value = "/edit")
    public Result<EtlKuaishouReportAccountHourly> edit(@RequestBody EtlKuaishouReportAccountHourly etlKuaishouReportAccountHourly) {
        Result<EtlKuaishouReportAccountHourly> result = new Result<EtlKuaishouReportAccountHourly>();
        EtlKuaishouReportAccountHourly etlKuaishouReportAccountHourlyEntity = etlKuaishouReportAccountHourlyService.getById(etlKuaishouReportAccountHourly.getId());
        if (etlKuaishouReportAccountHourlyEntity == null) {
            result.error500("未找到对应实体");
        } else {
            boolean ok = etlKuaishouReportAccountHourlyService.updateById(etlKuaishouReportAccountHourly);
            if (ok) {
                result.success("修改成功!");
            }
        }

        return result;
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @AutoLog(value = "账户维度-每小时盯盘数据-通过id删除")
    @ApiOperation(value = "账户维度-每小时盯盘数据-通过id删除", notes = "账户维度-每小时盯盘数据-通过id删除")
    @DeleteMapping(value = "/delete")
    public Result<?> delete(@RequestParam(name = "id", required = true) String id) {
        try {
            etlKuaishouReportAccountHourlyService.removeById(id);
        } catch (Exception e) {
            log.error("删除失败", e.getMessage());
            return Result.error("删除失败!");
        }
        return Result.ok("删除成功!");
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @AutoLog(value = "账户维度-每小时盯盘数据-批量删除")
    @ApiOperation(value = "账户维度-每小时盯盘数据-批量删除", notes = "账户维度-每小时盯盘数据-批量删除")
    @DeleteMapping(value = "/deleteBatch")
    public Result<EtlKuaishouReportAccountHourly> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        Result<EtlKuaishouReportAccountHourly> result = new Result<>();
        if (ids == null || "".equals(ids.trim())) {
            result.error500("参数不识别！");
        } else {
            this.etlKuaishouReportAccountHourlyService.removeByIds(Arrays.asList(ids.split(",")));
            result.success("删除成功!");
        }
        return result;
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    @AutoLog(value = "账户维度-每小时盯盘数据-通过id查询")
    @ApiOperation(value = "账户维度-每小时盯盘数据-通过id查询", notes = "账户维度-每小时盯盘数据-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<EtlKuaishouReportAccountHourly> queryById(@RequestParam(name = "id", required = true) String id) {
        Result<EtlKuaishouReportAccountHourly> result = new Result<>();
        EtlKuaishouReportAccountHourly etlKuaishouReportAccountHourly = etlKuaishouReportAccountHourlyService.getById(id);
        if (etlKuaishouReportAccountHourly == null) {
            result.error500("未找到对应实体");
        } else {
            result.setResult(etlKuaishouReportAccountHourly);
            result.setSuccess(true);
        }
        return result;
    }

    /**
     * 导出excel
     *
     * @param request
     * @param response
     */
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(HttpServletRequest request, HttpServletResponse response) {
        // Step.1 组装查询条件
        QueryWrapper<EtlKuaishouReportAccountHourly> queryWrapper = null;
        try {
            String paramsStr = request.getParameter("paramsStr");
            if (oConvertUtils.isNotEmpty(paramsStr)) {
                String deString = URLDecoder.decode(paramsStr, "UTF-8");
                EtlKuaishouReportAccountHourly etlKuaishouReportAccountHourly = JSON.parseObject(deString, EtlKuaishouReportAccountHourly.class);
                queryWrapper = QueryGenerator.initQueryWrapper(etlKuaishouReportAccountHourly, request.getParameterMap());
            }
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }

        //Step.2 AutoPoi 导出Excel
        ModelAndView mv = new ModelAndView(new JeecgEntityExcelView());
        List<EtlKuaishouReportAccountHourly> pageList = etlKuaishouReportAccountHourlyService.list(queryWrapper);
        //导出文件名称
        mv.addObject(NormalExcelConstants.FILE_NAME, "账户维度-每小时盯盘数据列表");
        mv.addObject(NormalExcelConstants.CLASS, EtlKuaishouReportAccountHourly.class);
        mv.addObject(NormalExcelConstants.PARAMS, new ExportParams("账户维度-每小时盯盘数据列表数据", "导出人:Jeecg", "导出信息"));
        mv.addObject(NormalExcelConstants.DATA_LIST, pageList);
        return mv;
    }

    /**
     * 通过excel导入数据
     *
     * @param request
     * @param response
     * @return
     */
    @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
        MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
        Map<String, MultipartFile> fileMap = multipartRequest.getFileMap();
        for (Map.Entry<String, MultipartFile> entity : fileMap.entrySet()) {
            MultipartFile file = entity.getValue();
            ImportParams params = new ImportParams();
            params.setTitleRows(2);
            params.setHeadRows(1);
            params.setNeedSave(true);
            try {
                List<EtlKuaishouReportAccountHourly> listEtlKuaishouReportAccountHourlys = ExcelImportUtil.importExcel(file.getInputStream(), EtlKuaishouReportAccountHourly.class, params);
                etlKuaishouReportAccountHourlyService.saveBatch(listEtlKuaishouReportAccountHourlys);
                return Result.ok("文件导入成功！数据行数:" + listEtlKuaishouReportAccountHourlys.size());
            } catch (Exception e) {
                log.error(e.getMessage(), e);
                return Result.error("文件导入失败:" + e.getMessage());
            } finally {
                try {
                    file.getInputStream().close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        return Result.ok("文件导入失败！");
    }

}
