package cn.com.ctop.kuaishou.modules.report.controller;

import cn.com.ctop.common.module.annotation.AutoLog;
import cn.com.ctop.common.module.utils.QueryGenerator;
import cn.com.ctop.kuaishou.modules.report.entity.KuaishouGroupExplore;
import cn.com.ctop.kuaishou.modules.report.service.IKuaishouGroupExploreService;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.util.oConvertUtils;
import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

/**
 * 广告组的加速探索报表
 *
 * @author jeecg-boot
 * @version V1.0
 * @date 2021-02-02
 */
@Slf4j
@Api(tags = "广告组的加速探索报表")
@RestController
@RequestMapping("/explore")
public class KuaishouGroupExploreController {
    @Autowired
    private IKuaishouGroupExploreService kuaishouGroupExploreService;

    /**
     * 分页列表查询
     *
     * @param kuaishouGroupExplore
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @AutoLog(value = "广告组的加速探索报表-分页列表查询")
    @ApiOperation(value = "广告组的加速探索报表-分页列表查询", notes = "广告组的加速探索报表-分页列表查询")
    @GetMapping(value = "/list")
    public Result<IPage<KuaishouGroupExplore>> queryPageList(KuaishouGroupExplore kuaishouGroupExplore,
                                                             @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                             @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                             HttpServletRequest req) {
        Result<IPage<KuaishouGroupExplore>> result = new Result<>();
        QueryWrapper<KuaishouGroupExplore> queryWrapper = QueryGenerator.initQueryWrapper(kuaishouGroupExplore, req.getParameterMap());
        Page<KuaishouGroupExplore> page = new Page<KuaishouGroupExplore>(pageNo, pageSize);
        IPage<KuaishouGroupExplore> pageList = kuaishouGroupExploreService.page(page, queryWrapper);
        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }

    /**
     * 查询加速探索信息接口
     *
     * @param kuaishouGroupExplore
     * @return
     */
    @AutoLog(value = "查询加速探索信息接口")
    @ApiOperation(value = "查询加速探索信息接口", notes = "查询加速探索信息接口")
    @PostMapping(value = "/unitExploreInfoGet")
    public Result<Object> unitExploreInfoGet(@RequestBody JSONObject data) {
        try {
            return kuaishouGroupExploreService.unitExploreInfoGet(data);
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            return Result.error("操作失败");
        }
    }

    /**
     * 广告组的加速探索预算设置接口
     *
     * @param kuaishouGroupExplore
     * @return
     */
    @AutoLog(value = "广告组的加速探索预算设置接口")
    @ApiOperation(value = "广告组的加速探索预算设置接口", notes = "广告组的加速探索预算设置接口")
    @PostMapping(value = "/unitExploreBudgetUpdate")
    public Result<Object> unitExploreBudgetUpdate(@RequestBody JSONObject data) {
        try {
            return kuaishouGroupExploreService.unitExploreBudgetUpdate(data);
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            return Result.error("操作失败");
        }
    }

    /**
     * 暂停广告组的加速探索预算接口
     *
     * @param kuaishouGroupExplore
     * @return
     */
    @AutoLog(value = "暂停广告组的加速探索预算接口")
    @ApiOperation(value = "暂停广告组的加速探索预算接口", notes = "暂停广告组的加速探索预算接口")
    @PostMapping(value = "/unitExploreStatusPause")
    public Result<Object> unitExploreStatusPause(@RequestBody JSONObject data) {
        try {
            return kuaishouGroupExploreService.unitExploreStatusPause(data);
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            return Result.error("操作失败");
        }
    }

    /**
     * 查询广告组的加速探索报表接口
     *
     * @param kuaishouGroupExplore
     * @return
     */
    @AutoLog(value = "查询广告组的加速探索报表接口")
    @ApiOperation(value = "查询广告组的加速探索报表接口", notes = "查询广告组的加速探索报表接口")
    @PostMapping(value = "/querySpeedExploreReport")
    public Result<Object> querySpeedExploreReport(@RequestBody JSONObject data) {
        try {
            return kuaishouGroupExploreService.querySpeedExploreReport(data);
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            return Result.error("操作失败");
        }
    }


    /**
     * 添加
     *
     * @param kuaishouGroupExplore
     * @return
     */
    @AutoLog(value = "广告组的加速探索报表-添加")
    @ApiOperation(value = "广告组的加速探索报表-添加", notes = "广告组的加速探索报表-添加")
    @PostMapping(value = "/add")
    public Result<KuaishouGroupExplore> add(@RequestBody KuaishouGroupExplore kuaishouGroupExplore) {
        Result<KuaishouGroupExplore> result = new Result<>();
        try {
            kuaishouGroupExploreService.save(kuaishouGroupExplore);
            result.success("添加成功！");
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            result.error500("操作失败");
        }
        return result;
    }

    /**
     * 编辑
     *
     * @param kuaishouGroupExplore
     * @return
     */
    @AutoLog(value = "广告组的加速探索报表-编辑")
    @ApiOperation(value = "广告组的加速探索报表-编辑", notes = "广告组的加速探索报表-编辑")
    @PutMapping(value = "/edit")
    public Result<KuaishouGroupExplore> edit(@RequestBody KuaishouGroupExplore kuaishouGroupExplore) {
        Result<KuaishouGroupExplore> result = new Result<KuaishouGroupExplore>();
        KuaishouGroupExplore kuaishouGroupExploreEntity = kuaishouGroupExploreService.getById(kuaishouGroupExplore.getId());
        if (kuaishouGroupExploreEntity == null) {
            result.error500("未找到对应实体");
        } else {
            boolean ok = kuaishouGroupExploreService.updateById(kuaishouGroupExplore);
            if (ok) {
                result.success("修改成功!");
            }
        }

        return result;
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @AutoLog(value = "广告组的加速探索报表-通过id删除")
    @ApiOperation(value = "广告组的加速探索报表-通过id删除", notes = "广告组的加速探索报表-通过id删除")
    @DeleteMapping(value = "/delete")
    public Result<?> delete(@RequestParam(name = "id", required = true) String id) {
        try {
            kuaishouGroupExploreService.removeById(id);
        } catch (Exception e) {
            log.error("删除失败", e.getMessage());
            return Result.error("删除失败!");
        }
        return Result.ok("删除成功!");
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @AutoLog(value = "广告组的加速探索报表-批量删除")
    @ApiOperation(value = "广告组的加速探索报表-批量删除", notes = "广告组的加速探索报表-批量删除")
    @DeleteMapping(value = "/deleteBatch")
    public Result<KuaishouGroupExplore> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        Result<KuaishouGroupExplore> result = new Result<>();
        if (ids == null || "".equals(ids.trim())) {
            result.error500("参数不识别！");
        } else {
            this.kuaishouGroupExploreService.removeByIds(Arrays.asList(ids.split(",")));
            result.success("删除成功!");
        }
        return result;
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    @AutoLog(value = "广告组的加速探索报表-通过id查询")
    @ApiOperation(value = "广告组的加速探索报表-通过id查询", notes = "广告组的加速探索报表-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<KuaishouGroupExplore> queryById(@RequestParam(name = "id", required = true) String id) {
        Result<KuaishouGroupExplore> result = new Result<>();
        KuaishouGroupExplore kuaishouGroupExplore = kuaishouGroupExploreService.getById(id);
        if (kuaishouGroupExplore == null) {
            result.error500("未找到对应实体");
        } else {
            result.setResult(kuaishouGroupExplore);
            result.setSuccess(true);
        }
        return result;
    }

    /**
     * 导出excel
     *
     * @param request
     * @param response
     */
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(HttpServletRequest request, HttpServletResponse response) {
        // Step.1 组装查询条件
        QueryWrapper<KuaishouGroupExplore> queryWrapper = null;
        try {
            String paramsStr = request.getParameter("paramsStr");
            if (oConvertUtils.isNotEmpty(paramsStr)) {
                String deString = URLDecoder.decode(paramsStr, "UTF-8");
                KuaishouGroupExplore kuaishouGroupExplore = JSON.parseObject(deString, KuaishouGroupExplore.class);
                queryWrapper = QueryGenerator.initQueryWrapper(kuaishouGroupExplore, request.getParameterMap());
            }
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }

        //Step.2 AutoPoi 导出Excel
        ModelAndView mv = new ModelAndView(new JeecgEntityExcelView());
        List<KuaishouGroupExplore> pageList = kuaishouGroupExploreService.list(queryWrapper);
        //导出文件名称
        mv.addObject(NormalExcelConstants.FILE_NAME, "广告组的加速探索报表列表");
        mv.addObject(NormalExcelConstants.CLASS, KuaishouGroupExplore.class);
        mv.addObject(NormalExcelConstants.PARAMS, new ExportParams("广告组的加速探索报表列表数据", "导出人:Jeecg", "导出信息"));
        mv.addObject(NormalExcelConstants.DATA_LIST, pageList);
        return mv;
    }

    /**
     * 通过excel导入数据
     *
     * @param request
     * @param response
     * @return
     */
    @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
        MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
        Map<String, MultipartFile> fileMap = multipartRequest.getFileMap();
        for (Map.Entry<String, MultipartFile> entity : fileMap.entrySet()) {
            MultipartFile file = entity.getValue();
            ImportParams params = new ImportParams();
            params.setTitleRows(2);
            params.setHeadRows(1);
            params.setNeedSave(true);
            try {
                List<KuaishouGroupExplore> listKuaishouGroupExplores = ExcelImportUtil.importExcel(file.getInputStream(), KuaishouGroupExplore.class, params);
                kuaishouGroupExploreService.saveBatch(listKuaishouGroupExplores);
                return Result.ok("文件导入成功！数据行数:" + listKuaishouGroupExplores.size());
            } catch (Exception e) {
                log.error(e.getMessage(), e);
                return Result.error("文件导入失败:" + e.getMessage());
            } finally {
                try {
                    file.getInputStream().close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        return Result.ok("文件导入失败！");
    }

}
