package cn.com.ctop.kuaishou.modules.report.controller;

import cn.com.ctop.common.module.annotation.AutoLog;
import cn.com.ctop.common.module.utils.QueryGenerator;
import cn.com.ctop.kuaishou.modules.report.entity.KuaishouReportDailyAgent;
import cn.com.ctop.kuaishou.modules.report.service.IKuaishouReportDailyAgentService;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.util.oConvertUtils;
import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

/**
 * 代理商报表
 * @author jeecg-boot
 * @date   2019-12-13
 * @version V1.0
 */
@Slf4j
@Api(tags="代理商报表")
@RestController
@RequestMapping("/report/kuaishouReportDailyAgent")
public class KuaishouReportDailyAgentController {
    @Autowired
    private IKuaishouReportDailyAgentService kuaishouReportDailyAgentService;
    @GetMapping(value = "/get")
    @ResponseBody
    public String getReport(@RequestParam(name="start")String start,@RequestParam(name="end")String end){
        kuaishouReportDailyAgentService.getReport(start,end);
        return "ok";
    }
    /**
     * 分页列表查询
     * @param kuaishouReportDailyAgent
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @AutoLog(value = "代理商报表-分页列表查询")
    @ApiOperation(value="代理商报表-分页列表查询", notes="代理商报表-分页列表查询")
    @GetMapping(value = "/list")
    public Result<IPage<KuaishouReportDailyAgent>> queryPageList(KuaishouReportDailyAgent kuaishouReportDailyAgent,
                                                                 @RequestParam(name="pageNo", defaultValue="1") Integer pageNo,
                                                                 @RequestParam(name="pageSize", defaultValue="10") Integer pageSize,
                                                                 HttpServletRequest req) {
        Result<IPage<KuaishouReportDailyAgent>> result = new Result<IPage<KuaishouReportDailyAgent>>();
        QueryWrapper<KuaishouReportDailyAgent> queryWrapper = QueryGenerator.initQueryWrapper(kuaishouReportDailyAgent, req.getParameterMap());
        Page<KuaishouReportDailyAgent> page = new Page<KuaishouReportDailyAgent>(pageNo, pageSize);
        IPage<KuaishouReportDailyAgent> pageList = kuaishouReportDailyAgentService.page(page, queryWrapper);
        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }

    /**
     *   添加
     * @param kuaishouReportDailyAgent
     * @return
     */
    @AutoLog(value = "代理商报表-添加")
    @ApiOperation(value="代理商报表-添加", notes="代理商报表-添加")
    @PostMapping(value = "/add")
    public Result<KuaishouReportDailyAgent> add(@RequestBody KuaishouReportDailyAgent kuaishouReportDailyAgent) {
        Result<KuaishouReportDailyAgent> result = new Result<KuaishouReportDailyAgent>();
        try {
            kuaishouReportDailyAgentService.save(kuaishouReportDailyAgent);
            result.success("添加成功！");
        } catch (Exception e) {
            log.error(e.getMessage(),e);
            result.error500("操作失败");
        }
        return result;
    }


    /**
     *   通过id删除
     * @param id
     * @return
     */
    @AutoLog(value = "代理商报表-通过id删除")
    @ApiOperation(value="代理商报表-通过id删除", notes="代理商报表-通过id删除")
    @DeleteMapping(value = "/delete")
    public Result<?> delete(@RequestParam(name="id",required=true) String id) {
        try {
            kuaishouReportDailyAgentService.removeById(id);
        } catch (Exception e) {
            log.error("删除失败",e.getMessage());
            return Result.error("删除失败!");
        }
        return Result.ok("删除成功!");
    }

    /**
     *  批量删除
     * @param ids
     * @return
     */
    @AutoLog(value = "代理商报表-批量删除")
    @ApiOperation(value="代理商报表-批量删除", notes="代理商报表-批量删除")
    @DeleteMapping(value = "/deleteBatch")
    public Result<KuaishouReportDailyAgent> deleteBatch(@RequestParam(name="ids",required=true) String ids) {
        Result<KuaishouReportDailyAgent> result = new Result<KuaishouReportDailyAgent>();
        if(ids==null || "".equals(ids.trim())) {
            result.error500("参数不识别！");
        }else {
            this.kuaishouReportDailyAgentService.removeByIds(Arrays.asList(ids.split(",")));
            result.success("删除成功!");
        }
        return result;
    }

    /**
     * 通过id查询
     * @param id
     * @return
     */
    @AutoLog(value = "代理商报表-通过id查询")
    @ApiOperation(value="代理商报表-通过id查询", notes="代理商报表-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<KuaishouReportDailyAgent> queryById(@RequestParam(name="id",required=true) String id) {
        Result<KuaishouReportDailyAgent> result = new Result<KuaishouReportDailyAgent>();
        KuaishouReportDailyAgent kuaishouReportDailyAgent = kuaishouReportDailyAgentService.getById(id);
        if(kuaishouReportDailyAgent==null) {
            result.error500("未找到对应实体");
        }else {
            result.setResult(kuaishouReportDailyAgent);
            result.setSuccess(true);
        }
        return result;
    }

    /**
     * 导出excel
     *
     * @param request
     * @param response
     */
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(HttpServletRequest request, HttpServletResponse response) {
        // Step.1 组装查询条件
        QueryWrapper<KuaishouReportDailyAgent> queryWrapper = null;
        try {
            String paramsStr = request.getParameter("paramsStr");
            if (oConvertUtils.isNotEmpty(paramsStr)) {
                String deString = URLDecoder.decode(paramsStr, "UTF-8");
                KuaishouReportDailyAgent kuaishouReportDailyAgent = JSON.parseObject(deString, KuaishouReportDailyAgent.class);
                queryWrapper = QueryGenerator.initQueryWrapper(kuaishouReportDailyAgent, request.getParameterMap());
            }
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }

        //Step.2 AutoPoi 导出Excel
        ModelAndView mv = new ModelAndView(new JeecgEntityExcelView());
        List<KuaishouReportDailyAgent> pageList = kuaishouReportDailyAgentService.list(queryWrapper);
        //导出文件名称
        mv.addObject(NormalExcelConstants.FILE_NAME, "代理商报表列表");
        mv.addObject(NormalExcelConstants.CLASS, KuaishouReportDailyAgent.class);
        mv.addObject(NormalExcelConstants.PARAMS, new ExportParams("代理商报表列表数据", "导出人:Jeecg", "导出信息"));
        mv.addObject(NormalExcelConstants.DATA_LIST, pageList);
        return mv;
    }

    /**
     * 通过excel导入数据
     *
     * @param request
     * @param response
     * @return
     */
    @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
        MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
        Map<String, MultipartFile> fileMap = multipartRequest.getFileMap();
        for (Map.Entry<String, MultipartFile> entity : fileMap.entrySet()) {
            MultipartFile file = entity.getValue();
            ImportParams params = new ImportParams();
            params.setTitleRows(2);
            params.setHeadRows(1);
            params.setNeedSave(true);
            try {
                List<KuaishouReportDailyAgent> listKuaishouReportDailyAgents = ExcelImportUtil.importExcel(file.getInputStream(), KuaishouReportDailyAgent.class, params);
                kuaishouReportDailyAgentService.saveBatch(listKuaishouReportDailyAgents);
                return Result.ok("文件导入成功！数据行数:" + listKuaishouReportDailyAgents.size());
            } catch (Exception e) {
                log.error(e.getMessage(),e);
                return Result.error("文件导入失败:"+e.getMessage());
            } finally {
                try {
                    file.getInputStream().close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        return Result.ok("文件导入失败！");
    }

}
