package cn.com.ctop.kuaishou.modules.report.service.impl.service.impl;

import cn.com.ctop.common.module.utils.Check;
import cn.com.ctop.kuaishou.modules.report.entity.EtlKuaishouAudienceAccountDaily;
import cn.com.ctop.kuaishou.modules.report.enums.CheckParametersEnum;
import cn.com.ctop.kuaishou.modules.report.mapper.EtlKuaishouAudienceAccountDailyMapper;
import cn.com.ctop.kuaishou.modules.report.mapper.KuaishouAudienceReportDailyAccountMapper;
import cn.com.ctop.kuaishou.modules.report.service.IEtlKuaishouAudienceAccountDailyService;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.jeecg.common.api.vo.Result;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * 人群分析报表——年龄、性别数据
 *
 * @author jeecg-boot
 * @version V1.0
 * @date 2021-03-09
 */
@Service
public class EtlKuaishouAudienceAccountDailyServiceImpl extends ServiceImpl<EtlKuaishouAudienceAccountDailyMapper, EtlKuaishouAudienceAccountDaily> implements IEtlKuaishouAudienceAccountDailyService {
    @Autowired
    private KuaishouAudienceReportDailyAccountMapper accountMapper;
    @Autowired
    private EtlKuaishouAudienceAccountDailyMapper accountDailyMapper;

    @Override
    public void queryEveryDayAudienceInfo() {
        List<EtlKuaishouAudienceAccountDaily> list = accountMapper.queryEveryDayAudienceInfo();
        if (!Check.isNull(list)) {
            accountDailyMapper.replaceBatch(list);
        }
    }

    @Override
    public Result<Object> getSexInfo(JSONObject request) {
        Long accountId = request.getLong("accountId");
        String value = request.getString("value");
        String dataType = request.getString("dataType");
        if (Check.isNull(accountId) || Check.isNull(value)) {
            return Result.error("必填参数为空");
        }
        String parType = "more";
        if (CheckParametersEnum.isInclude(value)) {
            value = CheckParametersEnum.getSqlName(value);
            parType = "alone";
        }
        String audienceType = "gender";
        String type = "gender";
        List<JSONObject> list = null;
        if ("pie".equals(dataType)) {
            list = accountDailyMapper.queryAudienceTypeInfo(value, type, accountId, audienceType, parType);
        } else if ("line".equals(dataType)) {
            list = accountDailyMapper.queryEveryDayInfo(value, type, accountId, audienceType, parType);
        }
        if (Check.isNull(list)) {
            return Result.error("未查询到数据");
        }
        return Result.ok(list);
    }

    @Override
    public Result<Object> getAgeInfo(JSONObject request) {
        Long accountId = request.getLong("accountId");
        String value = request.getString("value");
        String dataType = request.getString("dataType");
        if (Check.isNull(accountId) || Check.isNull(value) || Check.isNull(dataType)) {
            return Result.error("必填参数为空");
        }
        String parType = "more";
        if (CheckParametersEnum.isInclude(value)) {
            value = CheckParametersEnum.getSqlName(value);
            parType = "alone";
        }
        String audienceType = "ageSegment";
        String type = "age_segment";
        List<JSONObject> list = null;
        if ("pie".equals(dataType)) {
            list = accountDailyMapper.queryAudienceTypeInfo(value, type, accountId, audienceType, parType);
        } else if ("line".equals(dataType)) {
            list = accountDailyMapper.queryEveryDayInfo(value, type, accountId, audienceType, parType);
        }
        if (Check.isNull(list)) {
            return Result.error("未查询到数据");
        }
        return Result.ok(list);
    }

    @Override
    public Result<Object> getSceneInfo(JSONObject request) {
        Long accountId = request.getLong("accountId");
        String value = request.getString("value");
        String dataType = request.getString("dataType");
        if (Check.isNull(accountId) || Check.isNull(value) || Check.isNull(dataType)) {
            return Result.error("必填参数为空");
        }
        String parType = "more";
        if (CheckParametersEnum.isInclude(value)) {
            value = CheckParametersEnum.getSqlName(value);
            parType = "alone";
        }
        List<JSONObject> list = null;
        if ("pie".equals(dataType)) {
            list = accountDailyMapper.getSceneInfo(value, accountId, parType);
        } else if ("line".equals(dataType)) {
            list = accountDailyMapper.getSceneEveryDayInfo(value, accountId, parType);
        }
        if (Check.isNull(list)) {
            return Result.error("未查询到数据");
        }
        return Result.ok(list);
    }

}
