package cn.com.ctop.kuaishou.modules.report.service.impl;


import cn.com.ctop.common.module.entity.CtopOauthToken;
import cn.com.ctop.common.module.entity.UserAllocation;
import cn.com.ctop.common.module.mapper.UserAllocationMapper;
import cn.com.ctop.common.module.service.ICtopOauthTokenService;
import cn.com.ctop.common.module.utils.Check;
import cn.com.ctop.kuaishou.modules.batch.mapper.KuaiShouCreativeMapper;
import cn.com.ctop.kuaishou.modules.batch.service.IKuaishouInterfaceService;
import cn.com.ctop.kuaishou.modules.report.entity.EtlKuaishouReportAccountHourly;
import cn.com.ctop.kuaishou.modules.report.enums.CheckParametersEnum;
import cn.com.ctop.kuaishou.modules.report.mapper.EtlKuaishouReportAccountDailyMapper;
import cn.com.ctop.kuaishou.modules.report.mapper.EtlKuaishouReportAccountHourlyMapper;
import cn.com.ctop.kuaishou.modules.report.mapper.KuaishouReportHourlyAccountMapper;
import cn.com.ctop.kuaishou.modules.report.service.IEtlKuaishouReportAccountHourlyService;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.apache.shiro.SecurityUtils;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.system.vo.LoginUser;
import org.jeecg.common.util.DateUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.text.ParseException;
import java.util.List;

/**
 * 账户维度-每小时盯盘数据
 *
 * @author jeecg-boot
 * @version V1.0
 * @date 2021-03-04
 */
@Service
public class EtlKuaishouReportAccountHourlyServiceImpl extends ServiceImpl<EtlKuaishouReportAccountHourlyMapper, EtlKuaishouReportAccountHourly> implements IEtlKuaishouReportAccountHourlyService {

    @Autowired
    private KuaishouReportHourlyAccountMapper hourlyAccountMapper;
    @Autowired
    private KuaiShouCreativeMapper kuaiShouCreativeMapper;
    @Autowired
    private UserAllocationMapper allocationMapper;

    @Autowired
    private EtlKuaishouReportAccountHourlyMapper etlHourlyAccountMapper;

    @Autowired
    private IKuaishouInterfaceService iKuaishouInterfaceService;
    @Autowired
    private EtlKuaishouReportAccountDailyMapper etlDailyAccountMapper;
    @Autowired
    private ICtopOauthTokenService oauthTokenService;

    @Override
    public void queryEveryhourlyDataForAccount() {
        List<EtlKuaishouReportAccountHourly> list = hourlyAccountMapper.queryEveryhourlyDataForAccount();
        if (!Check.isNull(list)) {
            etlHourlyAccountMapper.replaceBatch(list);
        }
    }

    @Override
    public Result<Object> getAggregateByHour(JSONObject request) throws ParseException {
        Long accountId = request.getLong("accountId");
        String startTime = request.getString("startTime");
        if (Check.isNull(accountId) || Check.isNull(startTime)) {
            return Result.error("请选择时间和账户");
        }
        String endTime = request.getString("endTime");
        EtlKuaishouReportAccountHourly today = null;
        EtlKuaishouReportAccountHourly yesterday = null;
        if (Check.isNull(endTime)) {
            Integer hour = DateUtils.getHour(DateUtils.now());
            today = etlHourlyAccountMapper.getOnedayAggregateByHour(startTime, accountId, hour);
            String lastDay = DateUtils.getLastDay(startTime, 1);
            yesterday = etlHourlyAccountMapper.getOnedayAggregateByHour(lastDay, accountId, hour);
            if (Check.isNull(today) && hour > 0) {
                today = etlHourlyAccountMapper.getOnedayAggregateByHour(startTime, accountId, hour - 1);
            }
        } else {
            today = etlDailyAccountMapper.getOnedayAggregatebyDaily(startTime, endTime, accountId);
            int days = DateUtils.dateDiff(startTime, endTime);
            String endTime2 = DateUtils.getLastDay(startTime, 1);
            String startTime2 = DateUtils.getLastDay(endTime2, days);
            yesterday = etlDailyAccountMapper.getOnedayAggregatebyDaily(startTime2, endTime2, accountId);
        }
        if (!Check.isNull(today) && !Check.isNull(yesterday)) {
            BigDecimal tcharge = today.getCharge();
            BigDecimal ycharge = yesterday.getCharge();
            if (ycharge.compareTo(BigDecimal.ZERO) != 0 && ycharge.compareTo(BigDecimal.ZERO) != 0) {
                BigDecimal grew = tcharge.subtract(ycharge).divide(ycharge, 4, BigDecimal.ROUND_HALF_UP).multiply(new BigDecimal(100)).setScale(2, RoundingMode.HALF_UP);
                today.setGrew(grew);
            }
        }
        JSONObject obj = new JSONObject();
        obj.put("today", today);
        obj.put("yesterday", yesterday);
        return Result.ok(obj);
    }

    @Override
    public Result<Object> getEtlReportList(JSONObject request) {
        Long accountId = request.getLong("accountId");
        String startTime = request.getString("startTime");
        String value = request.getString("value");
        if (Check.isNull(accountId) || Check.isNull(startTime) || Check.isNull(value)) {
            return Result.error("必填参数为空");
        }
        //type alone 是独立字段例如:charge,activation，more是复杂字段
        String type = "more";
        if (CheckParametersEnum.isInclude(value)) {
            type = "alone";
            value = CheckParametersEnum.getSqlName(value);
        }
        String endTime = request.getString("endTime");
        JSONObject obj = new JSONObject();
        List<JSONObject> todayList = null;
        List<JSONObject> yesterdayList = null;
        if (Check.isNull(endTime)) {
            todayList = etlHourlyAccountMapper.getOnedayEtlReportList(startTime, accountId, value, type);
            String lastDay = DateUtils.getLastDay(startTime, 1);
            yesterdayList = etlHourlyAccountMapper.getOnedayEtlReportList(lastDay, accountId, value, type);
        } else {
            todayList = etlDailyAccountMapper.getOnedayEtlReportList(startTime, endTime, accountId, value, type);
            int days = DateUtils.dateDiff(startTime, endTime);
            String endTime2 = DateUtils.getLastDay(startTime, 1);
            String startTime2 = DateUtils.getLastDay(endTime2, days);
            yesterdayList = etlDailyAccountMapper.getOnedayEtlReportList(startTime2, endTime2, accountId, value, type);
        }
        obj.put("today", todayList);
        obj.put("yesterday", yesterdayList);
        return Result.ok(obj);
    }

    @Override
    public Result<Object> getMaxChargeAccountId(Long accountId) {
        if (Check.isNull(accountId)) {
            LoginUser sysUser = (LoginUser) SecurityUtils.getSubject().getPrincipal();
            accountId = etlDailyAccountMapper.getMaxChargeAccountId(sysUser.getId());
        }
        JSONObject obj = new JSONObject();
        CtopOauthToken oauthToken = oauthTokenService.getTokenByAccountId(accountId);
        if (Check.isNull(oauthToken)) {
            log.warn("未获取到账户信息");
        }
        JSONObject fundJson = iKuaishouInterfaceService.fundGet(oauthToken);
        if (!Check.isNull(fundJson)) {
            obj.put("totalBalance", fundJson.getBigDecimal("balance")); // 余额
        }
        UserAllocation userAllocation = allocationMapper.getUserAllocation(accountId);
        if (!Check.isNull(userAllocation)) {
            obj.put("accountId", userAllocation.getAccountId());
            obj.put("accountName", userAllocation.getAuthName());
            obj.put("status", userAllocation.getAccountStatus());
            obj.put("statusStr", userAllocation.getAccountStatus() == 0 ? "投放中" : "无效");
        }
        Integer counts = kuaiShouCreativeMapper.getRejectedCountsforAccountId(accountId);
        obj.put("rejectedCounts", counts);
        return Result.ok(obj);
    }
}
