package cn.com.ctop.oa.modules.controller;

import cn.com.ctop.common.module.annotation.AutoLog;
import cn.com.ctop.common.module.utils.Check;
import cn.com.ctop.common.module.utils.QueryGenerator;
import cn.com.ctop.oa.modules.entity.WechatDepartment;
import cn.com.ctop.oa.modules.service.IWechatDepartmentService;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.xxl.job.core.enums.NoEn;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.util.oConvertUtils;
import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

/**
 * 部门列表
 *
 * @author jeecg-boot
 * @version V1.0
 * @date 2020-09-10
 */
@Slf4j
@Api(tags = "部门列表")
@RestController
@RequestMapping("/oa/wechatDepartment")
public class WechatDepartmentController {
    @Autowired
    private IWechatDepartmentService wechatDepartmentService;

    /**
     * 分页列表查询
     *
     * @param wechatDepartment
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @AutoLog(value = "部门列表-分页列表查询")
    @ApiOperation(value = "部门列表-分页列表查询", notes = "部门列表-分页列表查询")
    @GetMapping(value = "/list")
    public Result<IPage<WechatDepartment>> queryPageList(WechatDepartment wechatDepartment,
                                                         @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                         @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                         HttpServletRequest req) {
        Result<IPage<WechatDepartment>> result = new Result<>();
        QueryWrapper<WechatDepartment> queryWrapper = QueryGenerator.initQueryWrapper(wechatDepartment, req.getParameterMap());
        Page<WechatDepartment> page = new Page<WechatDepartment>(pageNo, pageSize);
        IPage<WechatDepartment> pageList = wechatDepartmentService.page(page, queryWrapper);
        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }

    @AutoLog(value = "部门列表List-列表查询")
    @ApiOperation(value = "部门列表List-列表查询", notes = "部门列表List-列表查询")
    @PostMapping(value = "/queryList")
    public Result<List<WechatDepartment>> queryList() {
        Result<List<WechatDepartment>> result = new Result<>();
        try {
            List<WechatDepartment> pageList = wechatDepartmentService.queryList(NoEn.NO0.valueInt());
            if (!Check.isNull(pageList)) {
                result.setResult(pageList);
                result.setSuccess(true);
            }
        } catch (Exception e) {
            log.error("部门列表List-列表查询 error ", e);
            result.error500("操作失败");
        }
        return result;
    }

    /**
     * 添加
     *
     * @param wechatDepartment
     * @return
     */
    @AutoLog(value = "部门列表-添加")
    @ApiOperation(value = "部门列表-添加", notes = "部门列表-添加")
    @PostMapping(value = "/add")
    public Result<WechatDepartment> add(@RequestBody WechatDepartment wechatDepartment) {
        Result<WechatDepartment> result = new Result<>();
        try {
            wechatDepartmentService.save(wechatDepartment);
            result.success("添加成功！");
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            result.error500("操作失败");
        }
        return result;
    }

    /**
     * 编辑
     *
     * @param wechatDepartment
     * @return
     */
    @AutoLog(value = "部门列表-编辑")
    @ApiOperation(value = "部门列表-编辑", notes = "部门列表-编辑")
    @PutMapping(value = "/edit")
    public Result<WechatDepartment> edit(@RequestBody WechatDepartment wechatDepartment) {
        Result<WechatDepartment> result = new Result<WechatDepartment>();
        WechatDepartment wechatDepartmentEntity = wechatDepartmentService.getById(wechatDepartment.getId());
        if (wechatDepartmentEntity == null) {
            result.error500("未找到对应实体");
        } else {
            boolean ok = wechatDepartmentService.updateById(wechatDepartment);
            if (ok) {
                result.success("修改成功!");
            }
        }

        return result;
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @AutoLog(value = "部门列表-通过id删除")
    @ApiOperation(value = "部门列表-通过id删除", notes = "部门列表-通过id删除")
    @DeleteMapping(value = "/delete")
    public Result<?> delete(@RequestParam(name = "id", required = true) String id) {
        try {
            wechatDepartmentService.removeById(id);
        } catch (Exception e) {
            log.error("删除失败", e.getMessage());
            return Result.error("删除失败!");
        }
        return Result.ok("删除成功!");
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @AutoLog(value = "部门列表-批量删除")
    @ApiOperation(value = "部门列表-批量删除", notes = "部门列表-批量删除")
    @DeleteMapping(value = "/deleteBatch")
    public Result<WechatDepartment> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        Result<WechatDepartment> result = new Result<>();
        if (ids == null || "".equals(ids.trim())) {
            result.error500("参数不识别！");
        } else {
            this.wechatDepartmentService.removeByIds(Arrays.asList(ids.split(",")));
            result.success("删除成功!");
        }
        return result;
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    @AutoLog(value = "部门列表-通过id查询")
    @ApiOperation(value = "部门列表-通过id查询", notes = "部门列表-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<WechatDepartment> queryById(@RequestParam(name = "id", required = true) String id) {
        Result<WechatDepartment> result = new Result<>();
        WechatDepartment wechatDepartment = wechatDepartmentService.getById(id);
        if (wechatDepartment == null) {
            result.error500("未找到对应实体");
        } else {
            result.setResult(wechatDepartment);
            result.setSuccess(true);
        }
        return result;
    }

    /**
     * 导出excel
     *
     * @param request
     * @param response
     */
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(HttpServletRequest request, HttpServletResponse response) {
        // Step.1 组装查询条件
        QueryWrapper<WechatDepartment> queryWrapper = null;
        try {
            String paramsStr = request.getParameter("paramsStr");
            if (oConvertUtils.isNotEmpty(paramsStr)) {
                String deString = URLDecoder.decode(paramsStr, "UTF-8");
                WechatDepartment wechatDepartment = JSON.parseObject(deString, WechatDepartment.class);
                queryWrapper = QueryGenerator.initQueryWrapper(wechatDepartment, request.getParameterMap());
            }
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }

        //Step.2 AutoPoi 导出Excel
        ModelAndView mv = new ModelAndView(new JeecgEntityExcelView());
        List<WechatDepartment> pageList = wechatDepartmentService.list(queryWrapper);
        //导出文件名称
        mv.addObject(NormalExcelConstants.FILE_NAME, "部门列表列表");
        mv.addObject(NormalExcelConstants.CLASS, WechatDepartment.class);
        mv.addObject(NormalExcelConstants.PARAMS, new ExportParams("部门列表列表数据", "导出人:Jeecg", "导出信息"));
        mv.addObject(NormalExcelConstants.DATA_LIST, pageList);
        return mv;
    }

    /**
     * 通过excel导入数据
     *
     * @param request
     * @param response
     * @return
     */
    @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
        MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
        Map<String, MultipartFile> fileMap = multipartRequest.getFileMap();
        for (Map.Entry<String, MultipartFile> entity : fileMap.entrySet()) {
            MultipartFile file = entity.getValue();
            ImportParams params = new ImportParams();
            params.setTitleRows(2);
            params.setHeadRows(1);
            params.setNeedSave(true);
            try {
                List<WechatDepartment> listWechatDepartments = ExcelImportUtil.importExcel(file.getInputStream(), WechatDepartment.class, params);
                wechatDepartmentService.saveBatch(listWechatDepartments);
                return Result.ok("文件导入成功！数据行数:" + listWechatDepartments.size());
            } catch (Exception e) {
                log.error(e.getMessage(), e);
                return Result.error("文件导入失败:" + e.getMessage());
            } finally {
                try {
                    file.getInputStream().close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        return Result.ok("文件导入失败！");
    }

}
