package cn.com.ctop.toutiao.modules.batch.controller;

import cn.com.ctop.common.module.utils.QueryGenerator;
import cn.com.ctop.toutiao.modules.batch.entity.BytedanceIndustryInfo;
import cn.com.ctop.toutiao.modules.batch.service.IBytedanceIndustryInfoService;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.Arrays;

/**
 * 头条行业类别
 *
 * @author jeecg-boot
 * @version V1.0
 * @date 2020-07-30
 */
@Slf4j
@Api(tags = "头条行业类别")
@RestController
@RequestMapping("/ctop/bytedanceIndustryInfo")
public class BytedanceIndustryInfoController {
    @Autowired
    private IBytedanceIndustryInfoService bytedanceIndustryInfoService;

    /**
     * 分页列表查询
     *
     * @param bytedanceIndustryInfo
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @ApiOperation(value = "头条行业类别-分页列表查询", notes = "头条行业类别-分页列表查询")
    @GetMapping(value = "/list")
    public Result<IPage<BytedanceIndustryInfo>> queryPageList(BytedanceIndustryInfo bytedanceIndustryInfo,
                                                              @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                              @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                              HttpServletRequest req) {
        Result<IPage<BytedanceIndustryInfo>> result = new Result<>();
        QueryWrapper<BytedanceIndustryInfo> queryWrapper = QueryGenerator.initQueryWrapper(bytedanceIndustryInfo, req.getParameterMap());
        Page<BytedanceIndustryInfo> page = new Page<BytedanceIndustryInfo>(pageNo, pageSize);
        IPage<BytedanceIndustryInfo> pageList = bytedanceIndustryInfoService.page(page, queryWrapper);
        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }

    /**
     * 添加
     *
     * @param bytedanceIndustryInfo
     * @return
     */
    @ApiOperation(value = "头条行业类别-添加", notes = "头条行业类别-添加")
    @PostMapping(value = "/add")
    public Result<BytedanceIndustryInfo> add(@RequestBody BytedanceIndustryInfo bytedanceIndustryInfo) {
        Result<BytedanceIndustryInfo> result = new Result<>();
        try {
            bytedanceIndustryInfoService.save(bytedanceIndustryInfo);
            result.success("添加成功！");
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            result.error500("操作失败");
        }
        return result;
    }

    /**
     * 编辑
     *
     * @param bytedanceIndustryInfo
     * @return
     */
    @ApiOperation(value = "头条行业类别-编辑", notes = "头条行业类别-编辑")
    @PutMapping(value = "/edit")
    public Result<BytedanceIndustryInfo> edit(@RequestBody BytedanceIndustryInfo bytedanceIndustryInfo) {
        Result<BytedanceIndustryInfo> result = new Result<BytedanceIndustryInfo>();
        BytedanceIndustryInfo bytedanceIndustryInfoEntity = bytedanceIndustryInfoService.getById(bytedanceIndustryInfo.getId());
        if (bytedanceIndustryInfoEntity == null) {
            result.error500("未找到对应实体");
        } else {
            boolean ok = bytedanceIndustryInfoService.updateById(bytedanceIndustryInfo);
            if (ok) {
                result.success("修改成功!");
            }
        }

        return result;
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @ApiOperation(value = "头条行业类别-通过id删除", notes = "头条行业类别-通过id删除")
    @DeleteMapping(value = "/delete")
    public Result<?> delete(@RequestParam(name = "id", required = true) String id) {
        try {
            bytedanceIndustryInfoService.removeById(id);
        } catch (Exception e) {
            log.error("删除失败", e.getMessage());
            return Result.error("删除失败!");
        }
        return Result.ok("删除成功!");
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @ApiOperation(value = "头条行业类别-批量删除", notes = "头条行业类别-批量删除")
    @DeleteMapping(value = "/deleteBatch")
    public Result<BytedanceIndustryInfo> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        Result<BytedanceIndustryInfo> result = new Result<>();
        if (ids == null || "".equals(ids.trim())) {
            result.error500("参数不识别！");
        } else {
            this.bytedanceIndustryInfoService.removeByIds(Arrays.asList(ids.split(",")));
            result.success("删除成功!");
        }
        return result;
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    @ApiOperation(value = "头条行业类别-通过id查询", notes = "头条行业类别-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<BytedanceIndustryInfo> queryById(@RequestParam(name = "id", required = true) String id) {
        Result<BytedanceIndustryInfo> result = new Result<>();
        BytedanceIndustryInfo bytedanceIndustryInfo = bytedanceIndustryInfoService.getById(id);
        if (bytedanceIndustryInfo == null) {
            result.error500("未找到对应实体");
        } else {
            result.setResult(bytedanceIndustryInfo);
            result.setSuccess(true);
        }
        return result;
    }
}
