package cn.com.ctop.toutiao.modules.batch.controller;

import cn.com.ctop.common.module.annotation.AutoLog;
import cn.com.ctop.common.module.utils.QueryGenerator;
import cn.com.ctop.toutiao.modules.batch.entity.BytedanceInterestCategory;
import cn.com.ctop.toutiao.modules.batch.service.IBytedanceInterestCategoryService;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.Arrays;

@RestController
@Slf4j
public class BytedanceInterestCategoryController {
    @Autowired
    private IBytedanceInterestCategoryService bytedanceInterestCategoryService;

    @GetMapping(value = "/listbycode")
    public Result<IPage<BytedanceInterestCategory>> queryPageListByCode(@RequestParam(name = "parentCode", defaultValue = "0") String code,
                                                                        @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                                        @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                                        HttpServletRequest req) {
        Result<IPage<BytedanceInterestCategory>> result = new Result<>();
        BytedanceInterestCategory category = new BytedanceInterestCategory();
        QueryWrapper<BytedanceInterestCategory> queryWrapper = QueryGenerator.initQueryWrapper(category, req.getParameterMap());
        queryWrapper.eq("parent_code", code);
        Page<BytedanceInterestCategory> page = new Page<>(pageNo, pageSize);
        IPage<BytedanceInterestCategory> pageList = bytedanceInterestCategoryService.page(page, queryWrapper);
        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }

    /**
     * 分页列表查询
     *
     * @param bytedanceInterestCategory
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @AutoLog(value = "兴趣分类信息-分页列表查询")
    @ApiOperation(value = "兴趣分类信息-分页列表查询", notes = "兴趣分类信息-分页列表查询")
    @GetMapping(value = "/list")
    public Result<IPage<BytedanceInterestCategory>> queryPageList(BytedanceInterestCategory bytedanceInterestCategory,
                                                                  @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                                  @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                                  HttpServletRequest req) {
        Result<IPage<BytedanceInterestCategory>> result = new Result<>();
        QueryWrapper<BytedanceInterestCategory> queryWrapper = QueryGenerator.initQueryWrapper(bytedanceInterestCategory, req.getParameterMap());
        Page<BytedanceInterestCategory> page = new Page<>(pageNo, pageSize);
        IPage<BytedanceInterestCategory> pageList = bytedanceInterestCategoryService.page(page, queryWrapper);
        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }

    /**
     * 添加
     *
     * @param bytedanceInterestCategory
     * @return
     */
    @AutoLog(value = "兴趣分类信息-添加")
    @ApiOperation(value = "兴趣分类信息-添加", notes = "兴趣分类信息-添加")
    @PostMapping(value = "/add")
    public Result<BytedanceInterestCategory> add(@RequestBody BytedanceInterestCategory bytedanceInterestCategory) {
        Result<BytedanceInterestCategory> result = new Result<>();
        try {
            bytedanceInterestCategoryService.save(bytedanceInterestCategory);
            result.success("添加成功！");
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            result.error500("操作失败");
        }
        return result;
    }

    /**
     * 编辑
     *
     * @param bytedanceInterestCategory
     * @return
     */
    @AutoLog(value = "兴趣分类信息-编辑")
    @ApiOperation(value = "兴趣分类信息-编辑", notes = "兴趣分类信息-编辑")
    @PutMapping(value = "/edit")
    public Result<BytedanceInterestCategory> edit(@RequestBody BytedanceInterestCategory bytedanceInterestCategory) {
        Result<BytedanceInterestCategory> result = new Result<>();
        BytedanceInterestCategory bytedanceInterestCategoryEntity = bytedanceInterestCategoryService.getById(bytedanceInterestCategory.getId());
        if (bytedanceInterestCategoryEntity == null) {
            result.error500("未找到对应实体");
        } else {
            boolean ok = bytedanceInterestCategoryService.updateById(bytedanceInterestCategory);
            if (ok) {
                result.success("修改成功!");
            }
        }

        return result;
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @AutoLog(value = "兴趣分类信息-通过id删除")
    @ApiOperation(value = "兴趣分类信息-通过id删除", notes = "兴趣分类信息-通过id删除")
    @DeleteMapping(value = "/delete")
    public Result<?> delete(@RequestParam(name = "id", required = true) String id) {
        try {
            bytedanceInterestCategoryService.removeById(id);
        } catch (Exception e) {
            log.error("删除失败", e.getMessage());
            return Result.error("删除失败!");
        }
        return Result.ok("删除成功!");
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @AutoLog(value = "兴趣分类信息-批量删除")
    @ApiOperation(value = "兴趣分类信息-批量删除", notes = "兴趣分类信息-批量删除")
    @DeleteMapping(value = "/deleteBatch")
    public Result<BytedanceInterestCategory> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        Result<BytedanceInterestCategory> result = new Result<>();
        if (ids == null || "".equals(ids.trim())) {
            result.error500("参数不识别！");
        } else {
            this.bytedanceInterestCategoryService.removeByIds(Arrays.asList(ids.split(",")));
            result.success("删除成功!");
        }
        return result;
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    @AutoLog(value = "兴趣分类信息-通过id查询")
    @ApiOperation(value = "兴趣分类信息-通过id查询", notes = "兴趣分类信息-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<BytedanceInterestCategory> queryById(@RequestParam(name = "id", required = true) String id) {
        Result<BytedanceInterestCategory> result = new Result<>();
        BytedanceInterestCategory bytedanceInterestCategory = bytedanceInterestCategoryService.getById(id);
        if (bytedanceInterestCategory == null) {
            result.error500("未找到对应实体");
        } else {
            result.setResult(bytedanceInterestCategory);
            result.setSuccess(true);
        }
        return result;
    }
}

