package cn.com.ctop.toutiao.modules.material.controller;

import cn.com.ctop.common.module.annotation.AutoLog;
import cn.com.ctop.common.module.utils.QueryGenerator;
import cn.com.ctop.common.module.utils.StringUtils;
import cn.com.ctop.toutiao.modules.material.entity.ByteDanceAdvertiser;
import cn.com.ctop.toutiao.modules.material.service.IByteDanceAdvertiserService;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.util.Arrays;

/**
 * @Description: 今日头条广告主信息
 * @Author: jeecg-boot
 * @Date: 2019-07-22
 * @Version: V1.0
 */
@Slf4j
@Api(tags = "今日头条广告主信息")
@RestController
@RequestMapping("/ctop/byteDanceAdvertiser")
public class ByteDanceAdvertiserController {
    @Resource
    private IByteDanceAdvertiserService byteDanceAdvertiserService;

    /**
     * 分页列表查询
     *
     * @param byteDanceAdvertiser
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @AutoLog(value = "今日头条广告主信息-分页列表查询")
    @ApiOperation(value = "今日头条广告主信息-分页列表查询", notes = "今日头条广告主信息-分页列表查询")
    @GetMapping(value = "/list")
    public Result<IPage<ByteDanceAdvertiser>> queryPageList(ByteDanceAdvertiser byteDanceAdvertiser,
                                                            @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                            @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                            HttpServletRequest req) {
        Result<IPage<ByteDanceAdvertiser>> result = new Result<>();
        QueryWrapper<ByteDanceAdvertiser> queryWrapper = QueryGenerator.initQueryWrapper(byteDanceAdvertiser, req.getParameterMap());
        Page<ByteDanceAdvertiser> page = new Page<>(pageNo, pageSize);
        IPage<ByteDanceAdvertiser> pageList = byteDanceAdvertiserService.page(page, queryWrapper);
        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }

    /**
     * 添加
     *
     * @param byteDanceAdvertiser
     * @return
     */
    @AutoLog(value = "今日头条广告主信息-添加")
    @ApiOperation(value = "今日头条广告主信息-添加", notes = "今日头条广告主信息-添加")
    @PostMapping(value = "/add")
    public Result<ByteDanceAdvertiser> add(@RequestBody ByteDanceAdvertiser byteDanceAdvertiser) {
        Result<ByteDanceAdvertiser> result = new Result<>();
        try {
            byteDanceAdvertiserService.save(byteDanceAdvertiser);
            result.success("添加成功！");
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            result.error500("操作失败");
        }
        return result;
    }

    /**
     * 编辑
     *
     * @param byteDanceAdvertiser
     * @return
     */
    @AutoLog(value = "今日头条广告主信息-编辑")
    @ApiOperation(value = "今日头条广告主信息-编辑", notes = "今日头条广告主信息-编辑")
    @PutMapping(value = "/edit")
    public Result<ByteDanceAdvertiser> edit(@RequestBody ByteDanceAdvertiser byteDanceAdvertiser) {
        Result<ByteDanceAdvertiser> result = new Result<>();
        ByteDanceAdvertiser byteDanceAdvertiserEntity = byteDanceAdvertiserService.getById(byteDanceAdvertiser.getId());
        if (byteDanceAdvertiserEntity == null) {
            result.error500("未找到对应实体");
        } else {
            boolean ok = byteDanceAdvertiserService.updateById(byteDanceAdvertiser);
            if (ok) {
                result.success("修改成功!");
            } else {
                result.error500("未知错误");
            }
        }

        return result;
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @AutoLog(value = "今日头条广告主信息-通过id删除")
    @ApiOperation(value = "今日头条广告主信息-通过id删除", notes = "今日头条广告主信息-通过id删除")
    @DeleteMapping(value = "/delete")
    public Result<Object> delete(@RequestParam(name = "id", required = true) String id) {
        try {
            byteDanceAdvertiserService.removeById(id);
        } catch (Exception e) {
            log.error("删除失败", e.getMessage());
            return Result.error("删除失败!");
        }
        return Result.ok("删除成功!");
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @AutoLog(value = "今日头条广告主信息-批量删除")
    @ApiOperation(value = "今日头条广告主信息-批量删除", notes = "今日头条广告主信息-批量删除")
    @DeleteMapping(value = "/deleteBatch")
    public Result<ByteDanceAdvertiser> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        Result<ByteDanceAdvertiser> result = new Result<>();
        if (ids == null || "".equals(ids.trim())) {
            result.error500("参数不识别！");
        } else {
            this.byteDanceAdvertiserService.removeByIds(Arrays.asList(ids.split(StringUtils.COMMA)));
            result.success("删除成功!");
        }
        return result;
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    @AutoLog(value = "今日头条广告主信息-通过id查询")
    @ApiOperation(value = "今日头条广告主信息-通过id查询", notes = "今日头条广告主信息-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<ByteDanceAdvertiser> queryById(@RequestParam(name = "id", required = true) String id) {
        Result<ByteDanceAdvertiser> result = new Result<>();
        ByteDanceAdvertiser byteDanceAdvertiser = byteDanceAdvertiserService.getById(id);
        if (byteDanceAdvertiser == null) {
            result.error500("未找到对应实体");
        } else {
            result.setResult(byteDanceAdvertiser);
            result.setSuccess(true);
        }
        return result;
    }
}
