package cn.com.ctop.toutiao.modules.material.controller;

import cn.com.ctop.common.module.annotation.AutoLog;
import cn.com.ctop.common.module.utils.ResultMapUtils;
import cn.com.ctop.common.module.utils.StatusCode;
import cn.com.ctop.common.module.utils.StringUtils;
import cn.com.ctop.toutiao.modules.material.entity.ByteDanceCampaign;
import cn.com.ctop.toutiao.modules.material.service.IByteDanceCampaignService;
import cn.com.ctop.toutiao.modules.material.vo.BytedanceCampaignEditVo;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Description: 今日头条广告组信息
 * @Author: jeecg-boot
 * @Date: 2019-07-22
 * @Version: V1.0
 */
@Slf4j
@Api(tags = "今日头条广告组信息")
@RestController
@RequestMapping("/ctop/byteDanceCampaign")
public class ByteDanceCampaignController {
    @Resource
    private IByteDanceCampaignService byteDanceCampaignService;

    /**
     * 分页列表查询
     *
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @AutoLog(value = "今日头条广告组信息-分页列表查询")
    @ApiOperation(value = "今日头条广告组信息-分页列表查询", notes = "今日头条广告组信息-分页列表查询")
    @GetMapping(value = "/list")
    public Result<IPage<ByteDanceCampaign>> queryPageList(
            @RequestParam(name = "name", defaultValue = "") String name,
            @RequestParam(name = "accountId", defaultValue = "0") String accountId,
            @RequestParam(name = "status", defaultValue = "") String status,
            @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
            @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
            @RequestParam(name = "startDate", defaultValue = "") String startDate,
            @RequestParam(name = "endDate", defaultValue = "") String endDate,
            HttpServletRequest req) {
        Result<IPage<ByteDanceCampaign>> result = new Result<>();
        QueryWrapper<ByteDanceCampaign> queryWrapper = new QueryWrapper<>();
        if (null != name && !"".equalsIgnoreCase(name)) {
            queryWrapper.like("name", name);
        }
        if (null != status && !"".equals(status.trim())) {
            queryWrapper.eq("status", "CAMPAIGN_STATUS_" + status);
        }
        if (null != startDate && !"".equals(startDate)) {
            queryWrapper.ge("campaign_create_time", startDate + " 00:00:00");
        }
        if (null != endDate && !"".equals(endDate)) {
            queryWrapper.le("campaign_create_time", endDate + " 11:59:59");
        }
        if (null != accountId && !"".equals(accountId.trim())) {
            queryWrapper.eq("account_id", accountId);
        }
        queryWrapper.ne("status", "CAMPAIGN_STATUS_DELETE");
        queryWrapper.orderByDesc("campaign_create_time");
        Page<ByteDanceCampaign> page = new Page<>(pageNo, pageSize);
        IPage<ByteDanceCampaign> pageList = byteDanceCampaignService.page(page, queryWrapper);
        List<ByteDanceCampaign> records = pageList.getRecords();
        if (null != records && !records.isEmpty()) {
            List<ByteDanceCampaign> campaigns = records.stream().map(campaign -> campaign.setCampaignStatus("CAMPAIGN_STATUS_ENABLE".equalsIgnoreCase(campaign.getStatus()))).collect(Collectors.toList());
            pageList.setRecords(campaigns);
        }
        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }

    /**
     * 添加
     *
     * @return
     */
    @AutoLog(value = "今日头条广告组信息-添加")
    @ApiOperation(value = "今日头条广告组信息-添加", notes = "今日头条广告组信息-添加")
    @PostMapping(value = "/add")
    public Map<String, Object> add(@RequestParam(name = "accountId") Long accountId,
                                   @RequestParam(name = "campaignName") String campaignName,
                                   @RequestParam(name = "budgetMode") String budgetMode,
                                   @RequestParam(name = "budget", required = false) String budget,
                                   @RequestParam(name = "landingType") String landingType) {
        return byteDanceCampaignService.createCampaign(accountId, campaignName, null, budgetMode, budget, landingType, null);
    }

    /**
     * 编辑
     *
     * @return
     */
    @AutoLog(value = "今日头条广告组信息-编辑")
    @ApiOperation(value = "今日头条广告组信息-编辑", notes = "今日头条广告组信息-编辑")
    @PostMapping(value = "/edit")
    public Map<String, Object> edit(@RequestBody BytedanceCampaignEditVo editVo) {
        return byteDanceCampaignService.updateCampaign(editVo);
    }

    /**
     * 修改广告组状态
     *
     * @return
     */
    @PostMapping("/editStatus")
    public Map<String, Object> editStatus(@RequestParam(name = "accountId", defaultValue = "0") Long accountId,
                                          @RequestParam(name = "ids") String ids,
                                          @RequestParam(name = "status") String status) {
        Map<String, Object> result = new HashMap();
        if (null == accountId || accountId == 0 || null == ids || "".equals(ids.trim()) || null == status || "".equals(status.trim())) {
            ResultMapUtils.setResultMap(result, StatusCode.COMMON_PARAM_ERROR);
            return result;
        }
        return byteDanceCampaignService.updateCampaignStatus(accountId, ids, status);
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @AutoLog(value = "今日头条广告组信息-通过id删除")
    @ApiOperation(value = "今日头条广告组信息-通过id删除", notes = "今日头条广告组信息-通过id删除")
    @DeleteMapping(value = "/delete")
    public Result<Object> delete(@RequestParam(name = "id", required = true) String id) {
        try {
            byteDanceCampaignService.removeById(id);
        } catch (Exception e) {
            log.error("删除失败", e.getMessage());
            return Result.error("删除失败!");
        }
        return Result.ok("删除成功!");
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @AutoLog(value = "今日头条广告组信息-批量删除")
    @ApiOperation(value = "今日头条广告组信息-批量删除", notes = "今日头条广告组信息-批量删除")
    @DeleteMapping(value = "/deleteBatch")
    public Result<ByteDanceCampaign> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        Result<ByteDanceCampaign> result = new Result<>();
        if (ids == null || "".equals(ids.trim())) {
            result.error500("参数不识别！");
        } else {
            this.byteDanceCampaignService.removeByIds(Arrays.asList(ids.split(StringUtils.COMMA)));
            result.success("删除成功!");
        }
        return result;
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    @AutoLog(value = "今日头条广告组信息-通过id查询")
    @ApiOperation(value = "今日头条广告组信息-通过id查询", notes = "今日头条广告组信息-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<ByteDanceCampaign> queryById(@RequestParam(name = "id", required = true) String id) {
        Result<ByteDanceCampaign> result = new Result<>();
        ByteDanceCampaign byteDanceCampaign = byteDanceCampaignService.getById(id);
        if (byteDanceCampaign == null) {
            result.error500("未找到对应实体");
        } else {
            result.setResult(byteDanceCampaign);
            result.setSuccess(true);
        }
        return result;
    }
}
