package cn.com.ctop.toutiao.modules.material.controller;

import cn.com.ctop.common.module.annotation.AutoLog;
import cn.com.ctop.common.module.entity.CtopOauthToken;
import cn.com.ctop.common.module.service.ICtopOauthTokenService;
import cn.com.ctop.common.module.utils.ResultMapUtils;
import cn.com.ctop.common.module.utils.StatusCode;
import cn.com.ctop.common.module.utils.StringUtils;
import cn.com.ctop.toutiao.modules.material.entity.ByteDanceCreative;
import cn.com.ctop.toutiao.modules.material.service.IByteDanceCreativeService;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Description: 今日头条创意信息
 * @Author: jeecg-boot
 * @Date: 2019-07-22
 * @Version: V1.0
 */
@Slf4j
@Api(tags = "今日头条创意信息")
@RestController
@RequestMapping("/ctop/byteDanceCreative")
public class ByteDanceCreativeController {
    @Autowired
    private IByteDanceCreativeService byteDanceCreativeService;
    @Autowired
    ICtopOauthTokenService tokenService;

    /**
     * 分页列表查询
     *
     * @param pageNo
     * @param pageSize
     * @return
     */
    @AutoLog(value = "今日头条创意信息-分页列表查询")
    @ApiOperation(value = "今日头条创意信息-分页列表查询", notes = "今日头条创意信息-分页列表查询")
    @GetMapping(value = "/list")
    public Result<IPage<ByteDanceCreative>> queryPageList(
            @RequestParam(name = "planId", required = false) Long planId,
            @RequestParam(name = "accountId", defaultValue = "0") Long accountId,
            @RequestParam(name = "status", required = false) String status,
            @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
            @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
            @RequestParam(name = "startDate", defaultValue = "", required = false) String startDate,
            @RequestParam(name = "endDate", defaultValue = "", required = false) String endDate,
            @RequestParam(name = "name", defaultValue = "", required = false) String name) {
        Result<IPage<ByteDanceCreative>> result = new Result<>();
        QueryWrapper<ByteDanceCreative> queryWrapper = new QueryWrapper<>();
        if (null != startDate && !"".equals(startDate.trim())) {
            queryWrapper.ge("creative_create_time", startDate + " 00:00:00");
        }
        if (null != accountId) {
            queryWrapper.eq("account_id", accountId);
        }
        if (null != planId && planId != 0) {
            queryWrapper.eq("plan_id", planId);
        }
        if (null != endDate && !"".equals(endDate.trim())) {
            queryWrapper.le("creative_create_time", endDate + " 23:59:59");
        }
        if (null != name && !"".equals(name.trim())) {
            queryWrapper.like("title", name);
        }
        if (null != status && !"".equals(status.trim())) {
            queryWrapper.eq("opt_status", "CREATIVE_STATUS_" + status);
        }
        queryWrapper.orderByDesc("creative_create_time");
        Page<ByteDanceCreative> page = new Page<>(pageNo, pageSize);
        IPage<ByteDanceCreative> pageList = byteDanceCreativeService.page(page, queryWrapper);
        List<ByteDanceCreative> records = pageList.getRecords();
        if (null != records && !records.isEmpty()) {
            List<ByteDanceCreative> creatives = records.stream().map(creative -> creative.setCreateStatus("CREATIVE_STATUS_ENABLE".equalsIgnoreCase(creative.getOptStatus()))).collect(Collectors.toList());
            pageList.setRecords(creatives);
        }
        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }

    /**
     * 添加
     *
     * @param byteDanceCreative
     * @return
     */
    @AutoLog(value = "今日头条创意信息-添加")
    @ApiOperation(value = "今日头条创意信息-添加", notes = "今日头条创意信息-添加")
    @PostMapping(value = "/add")
    public Result<ByteDanceCreative> add(@RequestBody ByteDanceCreative byteDanceCreative) {
        Result<ByteDanceCreative> result = new Result<>();
        try {
            byteDanceCreativeService.save(byteDanceCreative);
            result.success("添加成功！");
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            result.error500("操作失败");
        }
        return result;
    }

    /**
     * 修改广告组状态
     *
     * @return
     */
    @PostMapping("/editStatus")
    public Map<String, Object> editStatus(@RequestParam(name = "accountId", defaultValue = "0") Long accountId,
                                          @RequestParam(name = "ids") String ids,
                                          @RequestParam(name = "status") String status) {
        Map<String, Object> result = new HashMap();
        if (null == accountId || accountId == 0 || null == ids || "".equals(ids.trim()) || null == status || "".equals(status.trim())) {
            ResultMapUtils.setResultMap(result, StatusCode.COMMON_PARAM_ERROR);
            return result;
        }
        CtopOauthToken cTopOauthToken = tokenService.getOauthTokenByAccountId(accountId + "");
        return byteDanceCreativeService.advertiserCreativeUpdateStatus(cTopOauthToken, accountId, ids, status);
    }

    /**
     * 编辑
     *
     * @param byteDanceCreative
     * @return
     */
    @AutoLog(value = "今日头条创意信息-编辑")
    @ApiOperation(value = "今日头条创意信息-编辑", notes = "今日头条创意信息-编辑")
    @PutMapping(value = "/edit")
    public Result<ByteDanceCreative> edit(@RequestBody ByteDanceCreative byteDanceCreative) {
        Result<ByteDanceCreative> result = new Result<>();
        ByteDanceCreative byteDanceCreativeEntity = byteDanceCreativeService.getById(byteDanceCreative.getId());
        if (byteDanceCreativeEntity == null) {
            result.error500("未找到对应实体");
        } else {
            boolean ok = byteDanceCreativeService.updateById(byteDanceCreative);
            if (ok) {
                result.success("修改成功!");
            } else {
                result.error500("未知错误");
            }
        }

        return result;
    }

    @PostMapping("/detail")
    public Map<String, Object> creativeDetail(@RequestParam(name = "planId", required = true, defaultValue = "0") Long planId,
                                              @RequestParam(name = "accountId", required = true, defaultValue = "0") Long accountId) {
        Map<String, Object> result = new HashMap<>();
        CtopOauthToken token = tokenService.getTokenByAccountId(accountId);
        if (null == token) {
            ResultMapUtils.setResultMap(result, StatusCode.COMMON_PARAM_ERROR);
            return result;
        }
        return byteDanceCreativeService.getCreativeDetail(token, planId);
    }


    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @AutoLog(value = "今日头条创意信息-通过id删除")
    @ApiOperation(value = "今日头条创意信息-通过id删除", notes = "今日头条创意信息-通过id删除")
    @DeleteMapping(value = "/delete")
    public Result<Object> delete(@RequestParam(name = "id", required = true) String id) {
        try {
            byteDanceCreativeService.removeById(id);
        } catch (Exception e) {
            log.error("删除失败", e.getMessage());
            return Result.error("删除失败!");
        }
        return Result.ok("删除成功!");
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @AutoLog(value = "今日头条创意信息-批量删除")
    @ApiOperation(value = "今日头条创意信息-批量删除", notes = "今日头条创意信息-批量删除")
    @DeleteMapping(value = "/deleteBatch")
    public Result<ByteDanceCreative> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        Result<ByteDanceCreative> result = new Result<>();
        if (ids == null || "".equals(ids.trim())) {
            result.error500("参数不识别！");
        } else {
            this.byteDanceCreativeService.removeByIds(Arrays.asList(ids.split(StringUtils.COMMA)));
            result.success("删除成功!");
        }
        return result;
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    @AutoLog(value = "今日头条创意信息-通过id查询")
    @ApiOperation(value = "今日头条创意信息-通过id查询", notes = "今日头条创意信息-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<ByteDanceCreative> queryById(@RequestParam(name = "id", required = true) String id) {
        Result<ByteDanceCreative> result = new Result<>();
        ByteDanceCreative byteDanceCreative = byteDanceCreativeService.getById(id);
        if (byteDanceCreative == null) {
            result.error500("未找到对应实体");
        } else {
            result.setResult(byteDanceCreative);
            result.setSuccess(true);
        }
        return result;
    }

    @PostMapping("/updateCreative")
    public Map<String, Object> updateCreative(@RequestBody JSONObject data) {
        return byteDanceCreativeService.updateCreative(data);
    }
}
