package cn.com.ctop.toutiao.modules.material.controller;

import cn.com.ctop.common.module.annotation.AutoLog;
import cn.com.ctop.common.module.utils.QueryGenerator;
import cn.com.ctop.toutiao.modules.material.entity.ByteDanceVideoInfo;
import cn.com.ctop.toutiao.modules.material.service.IByteDanceVideoInfoService;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.Arrays;

/**
 * @Description: 今日头条视频素材信息
 * @Author: jeecg-boot
 * @Date: 2019-07-29
 * @Version: V1.0
 */
@Slf4j
@Api(tags = "今日头条视频素材信息")
@RestController
@RequestMapping("/ctop/byteDanceVideoInfo")
public class ByteDanceVideoInfoController {
    @Autowired
    private IByteDanceVideoInfoService byteDanceVideoInfoService;

    /**
     * 分页列表查询
     *
     * @param byteDanceVideoInfo
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @AutoLog(value = "今日头条视频素材信息-分页列表查询")
    @ApiOperation(value = "今日头条视频素材信息-分页列表查询", notes = "今日头条视频素材信息-分页列表查询")
    @GetMapping(value = "/list")
    public Result<IPage<ByteDanceVideoInfo>> queryPageList(ByteDanceVideoInfo byteDanceVideoInfo,
                                                           @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                           @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                           HttpServletRequest req) {
        Result<IPage<ByteDanceVideoInfo>> result = new Result<>();
        QueryWrapper<ByteDanceVideoInfo> queryWrapper = QueryGenerator.initQueryWrapper(byteDanceVideoInfo, req.getParameterMap());
        Page<ByteDanceVideoInfo> page = new Page<>(pageNo, pageSize);
        IPage<ByteDanceVideoInfo> pageList = byteDanceVideoInfoService.page(page, queryWrapper);
        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }

    /**
     * 添加
     *
     * @param byteDanceVideoInfo
     * @return
     */
    @AutoLog(value = "今日头条视频素材信息-添加")
    @ApiOperation(value = "今日头条视频素材信息-添加", notes = "今日头条视频素材信息-添加")
    @PostMapping(value = "/add")
    public Result<ByteDanceVideoInfo> add(@RequestBody ByteDanceVideoInfo byteDanceVideoInfo) {
        Result<ByteDanceVideoInfo> result = new Result<>();
        try {
            byteDanceVideoInfoService.save(byteDanceVideoInfo);
            result.success("添加成功！");
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            result.error500("操作失败");
        }
        return result;
    }

    /**
     * 编辑
     *
     * @param byteDanceVideoInfo
     * @return
     */
    @AutoLog(value = "今日头条视频素材信息-编辑")
    @ApiOperation(value = "今日头条视频素材信息-编辑", notes = "今日头条视频素材信息-编辑")
    @PutMapping(value = "/edit")
    public Result<ByteDanceVideoInfo> edit(@RequestBody ByteDanceVideoInfo byteDanceVideoInfo) {
        Result<ByteDanceVideoInfo> result = new Result<>();
        ByteDanceVideoInfo byteDanceVideoInfoEntity = byteDanceVideoInfoService.getById(byteDanceVideoInfo.getId());
        if (byteDanceVideoInfoEntity == null) {
            result.error500("未找到对应实体");
        } else {
            boolean ok = byteDanceVideoInfoService.updateById(byteDanceVideoInfo);
            if (ok) {
                result.success("修改成功!");
            } else {
                result.error500("未知错误");
            }
        }

        return result;
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @AutoLog(value = "今日头条视频素材信息-通过id删除")
    @ApiOperation(value = "今日头条视频素材信息-通过id删除", notes = "今日头条视频素材信息-通过id删除")
    @DeleteMapping(value = "/delete")
    public Result<Object> delete(@RequestParam(name = "id", required = true) String id) {
        try {
            byteDanceVideoInfoService.removeById(id);
        } catch (Exception e) {
            log.error("删除失败", e.getMessage());
            return Result.error("删除失败!");
        }
        return Result.ok("删除成功!");
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @AutoLog(value = "今日头条视频素材信息-批量删除")
    @ApiOperation(value = "今日头条视频素材信息-批量删除", notes = "今日头条视频素材信息-批量删除")
    @DeleteMapping(value = "/deleteBatch")
    public Result<ByteDanceVideoInfo> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        Result<ByteDanceVideoInfo> result = new Result<>();
        if (ids == null || "".equals(ids.trim())) {
            result.error500("参数不识别！");
        } else {
            this.byteDanceVideoInfoService.removeByIds(Arrays.asList(ids.split(",")));
            result.success("删除成功!");
        }
        return result;
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    @AutoLog(value = "今日头条视频素材信息-通过id查询")
    @ApiOperation(value = "今日头条视频素材信息-通过id查询", notes = "今日头条视频素材信息-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<ByteDanceVideoInfo> queryById(@RequestParam(name = "id", required = true) String id) {
        Result<ByteDanceVideoInfo> result = new Result<>();
        ByteDanceVideoInfo byteDanceVideoInfo = byteDanceVideoInfoService.getById(id);
        if (byteDanceVideoInfo == null) {
            result.error500("未找到对应实体");
        } else {
            result.setResult(byteDanceVideoInfo);
            result.setSuccess(true);
        }
        return result;
    }
}
