package cn.com.ctop.toutiao.modules.report.controller;

import cn.com.ctop.common.module.annotation.AutoLog;
import cn.com.ctop.common.module.entity.ByteDanceCleanMaterialReport;
import cn.com.ctop.common.module.service.IByteDanceCleanMaterialReportService;
import cn.com.ctop.common.module.utils.QueryGenerator;
import cn.com.ctop.common.module.utils.StringUtils;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.Arrays;

/**
 * 头条-数据清洗
 *
 * @author jeecg-boot
 * @version V1.0
 * @date 2020-02-25
 */
@Slf4j
@Api(tags = "头条-数据清洗")
@RestController
@RequestMapping("/ctop/byteDanceCleanMaterialReport")
public class ByteDanceCleanMaterialReportController {
    @Autowired
    private IByteDanceCleanMaterialReportService byteDanceCleanMaterialReportService;

    /**
     * 分页列表查询
     *
     * @param byteDanceCleanMaterialReport
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @AutoLog(value = "头条-数据清洗-分页列表查询")
    @ApiOperation(value = "头条-数据清洗-分页列表查询", notes = "头条-数据清洗-分页列表查询")
    @GetMapping(value = "/list")
    public Result<IPage<ByteDanceCleanMaterialReport>> queryPageList(ByteDanceCleanMaterialReport byteDanceCleanMaterialReport,
                                                                     @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                                     @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                                     HttpServletRequest req) {
        Result<IPage<ByteDanceCleanMaterialReport>> result = new Result<>();
        QueryWrapper<ByteDanceCleanMaterialReport> queryWrapper = QueryGenerator.initQueryWrapper(byteDanceCleanMaterialReport, req.getParameterMap());
        Page<ByteDanceCleanMaterialReport> page = new Page<>(pageNo, pageSize);
        IPage<ByteDanceCleanMaterialReport> pageList = byteDanceCleanMaterialReportService.page(page, queryWrapper);
        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }

    /**
     * 添加
     *
     * @param byteDanceCleanMaterialReport
     * @return
     */
    @AutoLog(value = "头条-数据清洗-添加")
    @ApiOperation(value = "头条-数据清洗-添加", notes = "头条-数据清洗-添加")
    @PostMapping(value = "/add")
    public Result<ByteDanceCleanMaterialReport> add(@RequestBody ByteDanceCleanMaterialReport byteDanceCleanMaterialReport) {
        Result<ByteDanceCleanMaterialReport> result = new Result<>();
        try {
            byteDanceCleanMaterialReportService.save(byteDanceCleanMaterialReport);
            result.success("添加成功！");
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            result.error500("操作失败");
        }
        return result;
    }

    /**
     * 编辑
     *
     * @param byteDanceCleanMaterialReport
     * @return
     */
    @AutoLog(value = "头条-数据清洗-编辑")
    @ApiOperation(value = "头条-数据清洗-编辑", notes = "头条-数据清洗-编辑")
    @PutMapping(value = "/edit")
    public Result<ByteDanceCleanMaterialReport> edit(@RequestBody ByteDanceCleanMaterialReport byteDanceCleanMaterialReport) {
        Result<ByteDanceCleanMaterialReport> result = new Result<>();
        ByteDanceCleanMaterialReport byteDanceCleanMaterialReportEntity = byteDanceCleanMaterialReportService.getById(byteDanceCleanMaterialReport.getId());
        if (byteDanceCleanMaterialReportEntity == null) {
            result.error500("未找到对应实体");
        } else {
            boolean ok = byteDanceCleanMaterialReportService.updateById(byteDanceCleanMaterialReport);
            if (ok) {
                result.success("修改成功!");
            }
        }

        return result;
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @AutoLog(value = "头条-数据清洗-通过id删除")
    @ApiOperation(value = "头条-数据清洗-通过id删除", notes = "头条-数据清洗-通过id删除")
    @DeleteMapping(value = "/delete")
    public Result<Object> delete(@RequestParam(name = "id", required = true) String id) {
        try {
            byteDanceCleanMaterialReportService.removeById(id);
        } catch (Exception e) {
            log.error("删除失败", e.getMessage());
            return Result.error("删除失败!");
        }
        return Result.ok("删除成功!");
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @AutoLog(value = "头条-数据清洗-批量删除")
    @ApiOperation(value = "头条-数据清洗-批量删除", notes = "头条-数据清洗-批量删除")
    @DeleteMapping(value = "/deleteBatch")
    public Result<ByteDanceCleanMaterialReport> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        Result<ByteDanceCleanMaterialReport> result = new Result<>();
        if (ids == null || "".equals(ids.trim())) {
            result.error500("参数不识别！");
        } else {
            this.byteDanceCleanMaterialReportService.removeByIds(Arrays.asList(ids.split(StringUtils.COMMA)));
            result.success("删除成功!");
        }
        return result;
    }
}
