package cn.com.ctop.toutiao.modules.report.controller;

import cn.com.ctop.common.module.annotation.AutoLog;
import cn.com.ctop.common.module.utils.QueryGenerator;
import cn.com.ctop.common.module.utils.StringUtils;
import cn.com.ctop.toutiao.modules.report.entity.BytedanceCampaignHourlyReport;
import cn.com.ctop.toutiao.modules.report.service.IBytedanceCampaignHourlyReportService;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.Arrays;

/**
 * @Description: 广告组时报表信息
 * @Author: jeecg-boot
 * @Date: 2019-09-11
 * @Version: V1.0
 */
@Slf4j
@Api(tags = "广告组时报表信息")
@RestController
@RequestMapping("/ctop/bytedanceCampaignHourlyReport")
public class BytedanceCampaignHourlyReportController {
    @Autowired
    private IBytedanceCampaignHourlyReportService bytedanceCampaignHourlyReportService;

    /**
     * 分页列表查询
     *
     * @param bytedanceCampaignHourlyReport
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @AutoLog(value = "广告组时报表信息-分页列表查询")
    @ApiOperation(value = "广告组时报表信息-分页列表查询", notes = "广告组时报表信息-分页列表查询")
    @GetMapping(value = "/list")
    public Result<IPage<BytedanceCampaignHourlyReport>> queryPageList(BytedanceCampaignHourlyReport bytedanceCampaignHourlyReport,
                                                                      @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                                      @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                                      HttpServletRequest req) {
        Result<IPage<BytedanceCampaignHourlyReport>> result = new Result<>();
        QueryWrapper<BytedanceCampaignHourlyReport> queryWrapper = QueryGenerator.initQueryWrapper(bytedanceCampaignHourlyReport, req.getParameterMap());
        Page<BytedanceCampaignHourlyReport> page = new Page<>(pageNo, pageSize);
        IPage<BytedanceCampaignHourlyReport> pageList = bytedanceCampaignHourlyReportService.page(page, queryWrapper);
        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }

    /**
     * 添加
     *
     * @param bytedanceCampaignHourlyReport
     * @return
     */
    @AutoLog(value = "广告组时报表信息-添加")
    @ApiOperation(value = "广告组时报表信息-添加", notes = "广告组时报表信息-添加")
    @PostMapping(value = "/add")
    public Result<BytedanceCampaignHourlyReport> add(@RequestBody BytedanceCampaignHourlyReport bytedanceCampaignHourlyReport) {
        Result<BytedanceCampaignHourlyReport> result = new Result<>();
        try {
            bytedanceCampaignHourlyReportService.save(bytedanceCampaignHourlyReport);
            result.success("添加成功！");
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            result.error500("操作失败");
        }
        return result;
    }

    /**
     * 编辑
     *
     * @param bytedanceCampaignHourlyReport
     * @return
     */
    @AutoLog(value = "广告组时报表信息-编辑")
    @ApiOperation(value = "广告组时报表信息-编辑", notes = "广告组时报表信息-编辑")
    @PutMapping(value = "/edit")
    public Result<BytedanceCampaignHourlyReport> edit(@RequestBody BytedanceCampaignHourlyReport bytedanceCampaignHourlyReport) {
        Result<BytedanceCampaignHourlyReport> result = new Result<>();
        BytedanceCampaignHourlyReport bytedanceCampaignHourlyReportEntity = bytedanceCampaignHourlyReportService.getById(bytedanceCampaignHourlyReport.getId());
        if (bytedanceCampaignHourlyReportEntity == null) {
            result.error500("未找到对应实体");
        } else {
            boolean ok = bytedanceCampaignHourlyReportService.updateById(bytedanceCampaignHourlyReport);
            if (ok) {
                result.success("修改成功!");
            } else {
                result.error500("未知错误");
            }
        }

        return result;
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @AutoLog(value = "广告组时报表信息-通过id删除")
    @ApiOperation(value = "广告组时报表信息-通过id删除", notes = "广告组时报表信息-通过id删除")
    @DeleteMapping(value = "/delete")
    public Result<Object> delete(@RequestParam(name = "id", required = true) String id) {
        try {
            bytedanceCampaignHourlyReportService.removeById(id);
        } catch (Exception e) {
            log.error("删除失败", e.getMessage());
            return Result.error("删除失败!");
        }
        return Result.ok("删除成功!");
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @AutoLog(value = "广告组时报表信息-批量删除")
    @ApiOperation(value = "广告组时报表信息-批量删除", notes = "广告组时报表信息-批量删除")
    @DeleteMapping(value = "/deleteBatch")
    public Result<BytedanceCampaignHourlyReport> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        Result<BytedanceCampaignHourlyReport> result = new Result<>();
        if (ids == null || "".equals(ids.trim())) {
            result.error500("参数不识别！");
        } else {
            this.bytedanceCampaignHourlyReportService.removeByIds(Arrays.asList(ids.split(StringUtils.COMMA)));
            result.success("删除成功!");
        }
        return result;
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    @AutoLog(value = "广告组时报表信息-通过id查询")
    @ApiOperation(value = "广告组时报表信息-通过id查询", notes = "广告组时报表信息-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<BytedanceCampaignHourlyReport> queryById(@RequestParam(name = "id", required = true) String id) {
        Result<BytedanceCampaignHourlyReport> result = new Result<>();
        BytedanceCampaignHourlyReport bytedanceCampaignHourlyReport = bytedanceCampaignHourlyReportService.getById(id);
        if (bytedanceCampaignHourlyReport == null) {
            result.error500("未找到对应实体");
        } else {
            result.setResult(bytedanceCampaignHourlyReport);
            result.setSuccess(true);
        }
        return result;
    }

}
