package cn.com.ctop.toutiao.modules.report.controller;

import cn.com.ctop.common.module.annotation.AutoLog;
import cn.com.ctop.common.module.utils.QueryGenerator;
import cn.com.ctop.common.module.utils.StringUtils;
import cn.com.ctop.toutiao.modules.report.entity.BytedanceCreativeDailyReport;
import cn.com.ctop.toutiao.modules.report.service.IBytedanceCreativeDailyReportService;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.Arrays;
import java.util.List;

/**
 * @Description: 广告创意日报表信息
 * @Author: jeecg-boot
 * @Date: 2019-09-11
 * @Version: V1.0
 */
@Slf4j
@Api(tags = "广告创意日报表信息")
@RestController
@RequestMapping("/ctop/bytedanceCreativeDailyReport")
public class BytedanceCreativeDailyReportController {
    @Autowired
    private IBytedanceCreativeDailyReportService bytedanceCreativeDailyReportService;

    /**
     * 分页列表查询
     *
     * @param bytedanceCreativeDailyReport
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @AutoLog(value = "广告创意日报表信息-分页列表查询")
    @ApiOperation(value = "广告创意日报表信息-分页列表查询", notes = "广告创意日报表信息-分页列表查询")
    @GetMapping(value = "/list")
    public Result<IPage<BytedanceCreativeDailyReport>> queryPageList(BytedanceCreativeDailyReport bytedanceCreativeDailyReport,
                                                                     @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                                     @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                                     HttpServletRequest req) {
        Result<IPage<BytedanceCreativeDailyReport>> result = new Result<>();
        QueryWrapper<BytedanceCreativeDailyReport> queryWrapper = QueryGenerator.initQueryWrapper(bytedanceCreativeDailyReport, req.getParameterMap());
        Page<BytedanceCreativeDailyReport> page = new Page<>(pageNo, pageSize);
        IPage<BytedanceCreativeDailyReport> pageList = bytedanceCreativeDailyReportService.page(page, queryWrapper);
        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }

    /**
     * 添加
     *
     * @param bytedanceCreativeDailyReport
     * @return
     */
    @AutoLog(value = "广告创意日报表信息-添加")
    @ApiOperation(value = "广告创意日报表信息-添加", notes = "广告创意日报表信息-添加")
    @PostMapping(value = "/add")
    public Result<BytedanceCreativeDailyReport> add(@RequestBody BytedanceCreativeDailyReport bytedanceCreativeDailyReport) {
        Result<BytedanceCreativeDailyReport> result = new Result<>();
        try {
            bytedanceCreativeDailyReportService.save(bytedanceCreativeDailyReport);
            result.success("添加成功！");
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            result.error500("操作失败");
        }
        return result;
    }

    /**
     * 编辑
     *
     * @param bytedanceCreativeDailyReport
     * @return
     */
    @AutoLog(value = "广告创意日报表信息-编辑")
    @ApiOperation(value = "广告创意日报表信息-编辑", notes = "广告创意日报表信息-编辑")
    @PutMapping(value = "/edit")
    public Result<BytedanceCreativeDailyReport> edit(@RequestBody BytedanceCreativeDailyReport bytedanceCreativeDailyReport) {
        Result<BytedanceCreativeDailyReport> result = new Result<>();
        BytedanceCreativeDailyReport bytedanceCreativeDailyReportEntity = bytedanceCreativeDailyReportService.getById(bytedanceCreativeDailyReport.getId());
        if (bytedanceCreativeDailyReportEntity == null) {
            result.error500("未找到对应实体");
        } else {
            boolean ok = bytedanceCreativeDailyReportService.updateById(bytedanceCreativeDailyReport);
            if (ok) {
                result.success("修改成功!");
            } else {
                result.error500("未知错误");
            }
        }

        return result;
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @AutoLog(value = "广告创意日报表信息-通过id删除")
    @ApiOperation(value = "广告创意日报表信息-通过id删除", notes = "广告创意日报表信息-通过id删除")
    @DeleteMapping(value = "/delete")
    public Result<Object> delete(@RequestParam(name = "id", required = true) String id) {
        try {
            bytedanceCreativeDailyReportService.removeById(id);
        } catch (Exception e) {
            log.error("删除失败", e.getMessage());
            return Result.error("删除失败!");
        }
        return Result.ok("删除成功!");
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @AutoLog(value = "广告创意日报表信息-批量删除")
    @ApiOperation(value = "广告创意日报表信息-批量删除", notes = "广告创意日报表信息-批量删除")
    @DeleteMapping(value = "/deleteBatch")
    public Result<BytedanceCreativeDailyReport> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        Result<BytedanceCreativeDailyReport> result = new Result<>();
        if (ids == null || "".equals(ids.trim())) {
            result.error500("参数不识别！");
        } else {
            this.bytedanceCreativeDailyReportService.removeByIds(Arrays.asList(ids.split(StringUtils.COMMA)));
            result.success("删除成功!");
        }
        return result;
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    @AutoLog(value = "广告创意日报表信息-通过id查询")
    @ApiOperation(value = "广告创意日报表信息-通过id查询", notes = "广告创意日报表信息-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<BytedanceCreativeDailyReport> queryById(@RequestParam(name = "id", required = true) String id) {
        Result<BytedanceCreativeDailyReport> result = new Result<>();
        BytedanceCreativeDailyReport bytedanceCreativeDailyReport = bytedanceCreativeDailyReportService.getById(id);
        if (bytedanceCreativeDailyReport == null) {
            result.error500("未找到对应实体");
        } else {
            result.setResult(bytedanceCreativeDailyReport);
            result.setSuccess(true);
        }
        return result;
    }

    /**
     * TODO 需要重写 avg问题
     *
     * @param json
     * @return
     */
    @AutoLog(value = "头条素材报表")
    @ApiOperation(value = "头条素材报表", notes = "头条素材报表")
    @PostMapping(value = "/bytedanceCreativeDailyReportList")
    public Result<List<BytedanceCreativeDailyReport>> bytedanceCreativeDailyReportList(@RequestBody JSONObject json,
                                                                                       HttpServletRequest request, HttpServletResponse response) {
        Result<List<BytedanceCreativeDailyReport>> result = new Result<>();
        List<BytedanceCreativeDailyReport> bytedanceCreativeDailyReportList = bytedanceCreativeDailyReportService.bytedanceCreativeDailyReportList(json);
        result.setResult(bytedanceCreativeDailyReportList);
        result.setSuccess(true);
        return result;
    }

    /**
     * TODO 需要重写 avg问题
     *
     * @param json
     * @return
     */
    @AutoLog(value = "头条素材账户报表")
    @ApiOperation(value = "头条素材账户报表", notes = "头条素材账户报表")
    @PostMapping(value = "/bytedanceCreativeAccountReportList")
    public Result<List<BytedanceCreativeDailyReport>> bytedanceCreativeAccountReportList(@RequestBody JSONObject json,
                                                                                         HttpServletRequest request, HttpServletResponse response) {
        Result<List<BytedanceCreativeDailyReport>> result = new Result<>();
        List<BytedanceCreativeDailyReport> bytedanceCreativeDailyReportList = bytedanceCreativeDailyReportService.bytedanceCreativeAccountReportList(json);
        result.setResult(bytedanceCreativeDailyReportList);
        result.setSuccess(true);
        return result;
    }

}
